| package |
package := Package name: 'Comet-Core'.
package paxVersion: 1;
	basicComment: ''.

package basicPackageVersion: '51'.


package classNames
	add: #CTComet;
	add: #CTFilter;
	add: #CTHandler;
	add: #CTLibrary;
	add: #CTPusher;
	add: #CTStreamingServerRequired;
	yourself.

package methodNames
	add: #GRCodecStream -> #flushComet;
	add: #GRPlatform -> #cometForkProcessDo:;
	add: #GRPlatform -> #cometWait;
	add: #WARenderer -> #comet;
	add: #WARequestFilter -> #isCometFilter;
	add: #WAStreamedResponse -> #flushComet;
	add: 'GRPackage class' -> #cometCore;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Grease\Grease-Core';
	add: '..\Javascript\Javascript-Core';
	add: '..\Seaside\Seaside-Core';
	yourself).

package!

"Class Definitions"!

WAError subclass: #CTStreamingServerRequired
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAObject subclass: #CTHandler
	instanceVariableNames: 'id pusher session response'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAObject subclass: #CTPusher
	instanceVariableNames: 'mutex handlers rendererClass'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: 'process'!
JSObject subclass: #CTComet
	instanceVariableNames: 'pusher url'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAFileLibrary subclass: #CTLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WARequestFilter subclass: #CTFilter
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!GRCodecStream methodsFor!

flushComet
	^ stream flushComet! !
!GRCodecStream categoriesFor: #flushComet!*comet-core!public! !

!GRPackage class methodsFor!

cometCore
	^ self new
		name: 'Comet-Core';
		description: 'Server push, stream events from the server to client.';
		addDependency: 'Javascript-Core';
		addDependency: 'Seaside-Canvas';
		url: #seasideUrl;
		yourself! !
!GRPackage class categoriesFor: #cometCore!*comet-core!public! !

!GRPlatform methodsFor!

cometForkProcessDo: aBlock
	self subclassResponsibility!

cometWait
	"Pause the process for a period of time before pinging the instances again."

	self subclassResponsibility! !
!GRPlatform categoriesFor: #cometForkProcessDo:!*comet-core!public! !
!GRPlatform categoriesFor: #cometWait!*comet-core!public! !

!WARenderer methodsFor!

comet
	^ (CTComet context: self context)
		rendererClass: self species;
		yourself! !
!WARenderer categoriesFor: #comet!*comet-core!public! !

!WARequestFilter methodsFor!

isCometFilter
	^ false! !
!WARequestFilter categoriesFor: #isCometFilter!*comet-core!public! !

!WAStreamedResponse methodsFor!

flushComet
	^ externalStream flushComet! !
!WAStreamedResponse categoriesFor: #flushComet!*comet-core!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

CTStreamingServerRequired guid: (GUID fromString: '{81E398D7-CA20-45A6-91D6-2A52A4FBE351}')!
CTStreamingServerRequired comment: ''!
!CTStreamingServerRequired categoriesForClass!Unclassified! !
!CTStreamingServerRequired methodsFor!

seasidePossibleCauses
	^ #(
		'The ServerAdaptor handling this request does not support the streaming of HTTP responses, which is required when using Comet. On Squeak/Pharo, you can use WAListenerAdaptor in the Seaside-Adaptors-Comanche package.'
		)! !
!CTStreamingServerRequired categoriesFor: #seasidePossibleCauses!accessing!public! !

CTHandler guid: (GUID fromString: '{C0D64E20-4B76-4156-B0A5-2BAC1CFCF201}')!
CTHandler comment: ''!
!CTHandler categoriesForClass!Unclassified! !
!CTHandler methodsFor!

basicConnect: aRequestContext
	session := aRequestContext session.
	response := aRequestContext response.
	response
		doNotCache;
		contentType: aRequestContext application contentType;
		nextPutAll: (self preambleFor: aRequestContext);
		nextPutAll: ((String new: 1024) atAllPut: Character space)!

basicDisconnect
	response ifNil: [ ^ self ].
	response destroy.
	response := nil!

basicTerminate
	thisContext swapSender: nil.
	Processor activeProcess terminate!

connect: aRequestContext
	self
		basicDisconnect;
		basicConnect: aRequestContext.
	self pusher register: self.
	self basicTerminate!

id
	^ id!

initializeOn: aPusher
	self initialize.
	id := WAKeyGenerator current 
		keyOfLength: 16.
	pusher := aPusher!

notifySession
	self session application cache 
		notifyRetrieved: self session 
		key: self session key!

preambleFor: aRequestContext
	^ '<!!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"><html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en"><head><title>Comet</title></head><body>'
		, ((String new: 1024) atAllPut: Character space)!

push: aString
	"Push aString to the receiver. Notify the session about the update, so that it does not timeout. Answer true, if the connection is still valid."

	| success |
	response isNil
		ifTrue: [ ^ false ].
	success := [ response nextPutAll: aString; flushComet ]
		on: Error
		do: [ :err | ^ false ].
	success
		ifTrue: [ self notifySession ].
	^ success!

pusher
	^ pusher!

session
	^ session! !
!CTHandler categoriesFor: #basicConnect:!private! !
!CTHandler categoriesFor: #basicDisconnect!private! !
!CTHandler categoriesFor: #basicTerminate!private! !
!CTHandler categoriesFor: #connect:!actions!public! !
!CTHandler categoriesFor: #id!accessing!public! !
!CTHandler categoriesFor: #initializeOn:!initialization!public! !
!CTHandler categoriesFor: #notifySession!private! !
!CTHandler categoriesFor: #preambleFor:!private! !
!CTHandler categoriesFor: #push:!actions!public! !
!CTHandler categoriesFor: #pusher!accessing!public! !
!CTHandler categoriesFor: #session!accessing!public! !

!CTHandler class methodsFor!

on: aPusher
	^ self basicNew initializeOn: aPusher! !
!CTHandler class categoriesFor: #on:!instance creation!public! !

CTPusher guid: (GUID fromString: '{0172195A-80B5-483E-8B92-BDDB7F837373}')!
CTPusher comment: ''!
!CTPusher categoriesForClass!Unclassified! !
!CTPusher methodsFor!

handlers
	"Answer the current list of handlers."

	^ handlers!

initialize
	super initialize.
	mutex := GRPlatform current semaphoreClass forMutualExclusion.
	handlers := Array new!

javascript: aBlock
	"Evaluate aBlock with an JSScript instance and pushes the resulting script-string aString to all associated handlers."

	| script |
	script := (JSScript context: self renderContext)
		rendererClass: self rendererClass;
		yourself.
	aBlock value: script.
	self push: (String streamContents: [ :stream |
		stream
			nextPutAll: '<script type="text/javascript">parent.Comet.eval(';
			javascript: script asJavascript;
			nextPutAll: ')</script>' ])!

mutex
	"Answer the mutex in use with the receiver."

	^ mutex!

ping
	"Ping all the associated handlers."

	self push: ' '!

push: aString
	"Push aString to all associated handlers."

	self mutex critical: [ handlers := handlers select: [ :each | each push: aString ] ]!

register: aHandler
	"Register a new handler into this pusher."

	mutex critical: [ handlers := handlers copyWith: aHandler ]!

renderContext
	"Answer a fake rendering context for this pusher."

	^ WARenderContext new
		document: (WAHtmlDocument on: (WriteStream on: String new));
		actionUrl: WAUrl new;
		yourself!

rendererClass
	"Answer the default renderer for this pusher."

	^ rendererClass ifNil: [ WARenderer default ]!

rendererClass: aRendererClass
	rendererClass := aRendererClass! !
!CTPusher categoriesFor: #handlers!accessing!public! !
!CTPusher categoriesFor: #initialize!initialization!public! !
!CTPusher categoriesFor: #javascript:!public! !
!CTPusher categoriesFor: #mutex!accessing!public! !
!CTPusher categoriesFor: #ping!protected!public! !
!CTPusher categoriesFor: #push:!protected!public! !
!CTPusher categoriesFor: #register:!protected!public! !
!CTPusher categoriesFor: #renderContext!accessing-rendering!public! !
!CTPusher categoriesFor: #rendererClass!accessing-rendering!public! !
!CTPusher categoriesFor: #rendererClass:!accessing-rendering!public! !

!CTPusher class methodsFor!

initialize
	GRPlatform current
		addToStartUpList: self;
		addToShutDownList: self!

pingProcess
	self allInstancesDo: [ :each | each ping ].
	GRPlatform current cometWait!

shutDown
	process ifNil: [ ^ self ].
	GRPlatform current terminateProcess: process.
	process := nil!

startUp
	self shutDown.
	process := GRPlatform current cometForkProcessDo: [ [ self pingProcess ] repeat ]!

unload
	GRPlatform current
		removeFromStartUpList: self;
		removeFromShutDownList: self.
	self shutDown! !
!CTPusher class categoriesFor: #initialize!initialization!public! !
!CTPusher class categoriesFor: #pingProcess!private! !
!CTPusher class categoriesFor: #shutDown!initialization!public! !
!CTPusher class categoriesFor: #startUp!initialization!public! !
!CTPusher class categoriesFor: #unload!initialization!public! !

CTComet guid: (GUID fromString: '{D448FB23-A78B-4BCB-B4CC-F6096BAA6785}')!
CTComet comment: ''!
!CTComet categoriesForClass!Unclassified! !
!CTComet methodsFor!

connect
	self call: 'connect'!

defaultUrl
	^ self renderContext actionUrl
		addField: self class headerField
		value: self pusher id!

disconnect
	self call: 'disconnect'!

filter
	^ self session filters
		detect: [ :each | each isCometFilter ]
		ifNone: [ self session addFilterFirst: CTFilter new ]!

javascriptContentOn: aStream
	aStream
		nextPutAll: 'new Comet(';
		javascript: self url;
		nextPut: $)!

pusher
	^ pusher!

pusher: aPusher
	self url: (self renderContext actionUrl
		withField: self class headerField
		value: (self filter
			registerPusher: (pusher := aPusher)
			context: self requestContext))!

url
	^ url ifNil: [ url := self defaultUrl ]!

url: aUrl
	url := aUrl! !
!CTComet categoriesFor: #connect!actions!public! !
!CTComet categoriesFor: #defaultUrl!private! !
!CTComet categoriesFor: #disconnect!actions!public! !
!CTComet categoriesFor: #filter!accessing-readonly!public! !
!CTComet categoriesFor: #javascriptContentOn:!printing!public! !
!CTComet categoriesFor: #pusher!accessing!public! !
!CTComet categoriesFor: #pusher:!accessing!public! !
!CTComet categoriesFor: #url!accessing!public! !
!CTComet categoriesFor: #url:!accessing!public! !

!CTComet class methodsFor!

headerField
	^ '_c'! !
!CTComet class categoriesFor: #headerField!configuration!public! !

CTLibrary guid: (GUID fromString: '{C3E7A296-0C95-41EC-AAF1-C3858D013578}')!
CTLibrary comment: ''!
!CTLibrary categoriesForClass!Unclassified! !
!CTLibrary methodsFor!

cometJs
	^ '/* Seaside Comet API
 * (c) 2006-2009 Lukas Renggli (renggli@gmail.com)
 */

var Comet = function(url) {
	this.url = url;
	this.iframe = this.createIframe();
	document.body.appendChild(this.iframe);
};

Comet.prototype.connect = function() {
	this.iframe.onLoad = this.onLoad;
	this.iframe.src = this.url;
};

Comet.prototype.disconnect = function() {
	this.iframe.onLoad = this.iframe.src = null;
};

Comet.prototype.onLoad = function() {
	this.disconnect();
	this.connect();
};

Comet.prototype.createIframe = function() {
	var frame = document.createElement("iframe");
	frame.style.display = "block";
	frame.style.visibility = frame.style.overflow = "hidden";
	frame.style.width = frame.style.height = frame.style.border = frame.style.margin = frame.style.padding = "0";
	return frame;
};

Comet.eval = function(code) {
 	return eval(code);
};'!

selectorsToInclude
	^ #( cometJs )! !
!CTLibrary categoriesFor: #cometJs!public!scripts! !
!CTLibrary categoriesFor: #selectorsToInclude!accessing!public! !

CTFilter guid: (GUID fromString: '{B3CFBD2E-32F2-4485-BF45-F36D11B6A520}')!
CTFilter comment: ''!
!CTFilter categoriesForClass!Unclassified! !
!CTFilter methodsFor!

cometForContext: aRequestContext
	^ (handlers
		at: (aRequestContext request fields
			at: CTComet headerField
			ifAbsent: [ ^ nil ])
		ifAbsent: [ ^ nil ])
			connect: aRequestContext!

handleFiltered: aRequestContext
	(aRequestContext response isKindOf: WAStreamedResponse)
		ifFalse: [ CTStreamingServerRequired signal: 'Streaming server required.' ].
	(aRequestContext request includesKey: CTComet headerField)
		ifTrue: [ self cometForContext: aRequestContext ]
		ifFalse: [ super handleFiltered: aRequestContext ]!

initialize
	super initialize.
	handlers := Dictionary new!

isCometFilter
	^ true!

registerPusher: aPusher context: aContext
	| handler |
	handler := CTHandler on: aPusher.
	handlers at: handler id put: handler.
	^ handler id! !
!CTFilter categoriesFor: #cometForContext:!processing!public! !
!CTFilter categoriesFor: #handleFiltered:!processing!public! !
!CTFilter categoriesFor: #initialize!initialization!public! !
!CTFilter categoriesFor: #isCometFilter!public!testing! !
!CTFilter categoriesFor: #registerPusher:context:!public! !

"Binary Globals"!

