| package |
package := Package name: 'Comet-Examples'.
package paxVersion: 1;
	basicComment: ''.

package basicPackageVersion: '21'.


package classNames
	add: #CTChat;
	add: #CTChatSession;
	add: #CTCounter;
	add: #CTCounterModel;
	add: #CTReport;
	add: #CTReportItem;
	add: #CTSlider;
	yourself.

package methodNames
	add: 'GRPackage class' -> #cometExamples;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: 'Comet-Core';
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Grease\Grease-Core';
	add: '..\JQuery\JQuery-Core';
	add: '..\JQuery\JQuery-UI';
	add: '..\Seaside\Seaside-Component';
	add: '..\Seaside\Seaside-Session';
	add: '..\Seaside\Seaside-Tools-Core';
	yourself).

package!

"Class Definitions"!

GRObject subclass: #CTCounterModel
	instanceVariableNames: 'count'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
GRObject subclass: #CTReportItem
	instanceVariableNames: 'name phone'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAComponent subclass: #CTChat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: 'pusher'!
WAComponent subclass: #CTCounter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: 'model pusher'!
WAComponent subclass: #CTReport
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: 'items pusher'!
WAComponent subclass: #CTSlider
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: 'value pusher'!
WASession subclass: #CTChatSession
	instanceVariableNames: 'username'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!GRPackage class methodsFor!

cometExamples
	^ self new
		name: 'Comet-Examples';
		description: 'Examples for server push technology.';
		addDependency: 'JQuery-UI';
		addDependency: 'Comet-Core';
		url: #seasideUrl;
		yourself! !
!GRPackage class categoriesFor: #cometExamples!*comet-examples!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

CTCounterModel guid: (GUID fromString: '{47BA6266-2F9D-4544-AF4D-8C9DF7511378}')!
CTCounterModel comment: ''!
!CTCounterModel categoriesForClass!Unclassified! !
!CTCounterModel methodsFor!

count
	^ count!

count: anInteger
	count := anInteger!

decrease
	count := count - 1!

increase
	count := count + 1!

initialize
	super initialize.
	count := 0! !
!CTCounterModel categoriesFor: #count!accessing!public! !
!CTCounterModel categoriesFor: #count:!accessing!public! !
!CTCounterModel categoriesFor: #decrease!actions!public! !
!CTCounterModel categoriesFor: #increase!actions!public! !
!CTCounterModel categoriesFor: #initialize!initialization!public! !

CTReportItem guid: (GUID fromString: '{6450E4D1-B690-4468-AB08-6AE05EC0458A}')!
CTReportItem comment: ''!
!CTReportItem categoriesForClass!Unclassified! !
!CTReportItem methodsFor!

name
	^ name!

name: aString
	name := aString!

phone
	^ phone!

phone: aString
	phone := aString! !
!CTReportItem categoriesFor: #name!accessing!public! !
!CTReportItem categoriesFor: #name:!accessing!public! !
!CTReportItem categoriesFor: #phone!accessing!public! !
!CTReportItem categoriesFor: #phone:!accessing!public! !

CTChat guid: (GUID fromString: '{CFC2EB79-9FE5-4805-AA94-AAA73D542E9A}')!
CTChat comment: ''!
!CTChat categoriesForClass!Unclassified! !
!CTChat methodsFor!

isLoggedIn
	^ self username notNil!

login: aString
	"Join the chat-room with the given username aString."
	
	aString isEmptyOrNil ifTrue: [ ^ self ].
	self session username: aString.
	self push: 'event' with: 'joined'!

logout
	"Leave the chat-room."

	self push: 'event' with: 'left'.
	self session username: nil!

message: aString
	"Send the message aString to the chat-room."
	
	self push: 'message' with: aString!

push: aString with: aBlock
	"This code pushes the new message to all its clients. With the exception of part 2 adding the message itself, all other parts are optional."
	
	self pusher javascript: [ :script |
		script add: ((script jQuery: #messages)
		"1. limit listing to 20 messages"
			children;
			sliceFrom: 0 to: -19;
			remove;
			end;
		"2. append message to bottom"
			append: [ :html | self renderMessage: aBlock class: aString on: html ];
			end;
		"3. scroll to bottom"
			scrollTop: 65536) ]!

pusher
	^ self class pusher!

renderChatInputOn: html
	html textArea
		id: #input;
		callback: [ :value | self message: value ];
		onEnter: (html jQuery ajax
			onSuccess: ((html jQuery: #input) 
				value: String new);
			serializeThis)!

renderChatMessagesOn: html
	html div id: 'messages'; with: [
		self 
			renderMessage: [
				self users isEmpty
					ifTrue: [ html text: 'You are the only one here!!' ]
					ifFalse: [
						html text: 'Also there: '.
						self users
							do: [ :each | html text: each ]
							separatedBy: [ html text: ', ' ] ] ]
			class: 'event' on: html ]!

renderChatOn: html
	self renderChatStatusOn: html.
	self renderChatMessagesOn: html.
	self renderChatInputOn: html.
	self renderChatPusherOn: html!

renderChatPusherOn: html
	html document 
		addLoadScript: (html comet
			pusher: self pusher;
			connect)!

renderChatStatusOn: html
	html div id: 'status'; with: [
		html text: 'Currently logged in as ' , self username , '. '.
		html anchor
			callback: [ self logout ];
			with: 'Logout' ]!

renderContentOn: html
	html heading: 'Seaside Chat'.
	self isLoggedIn
		ifTrue: [ self renderChatOn: html ]
		ifFalse: [ self renderLoginOn: html ]!

renderLoginOn: html
	html form: [
		html textInput
			value: self username;
			callback: [ :value | self login: value ].
		html submitButton
			text: 'Login' ]!

renderMessage: aBlock class: aString on: html
	html div class: 'message'; with: [
		html span class: 'user'; with: self username.
		html span class: aString; with: aBlock ]!

style
	^ 'a {
	color: #00f;
	cursor: pointer;
	text-decoration: underline;	
}
#status {
	background-color: #eee;
}
#messages {
	height: 10em;
	overflow: auto;
}
#messages .message {
	margin-top: 0.25em;
	margin-bottom: 0.25em;
}
#messages .message .user {
	font-weight: bold;
	margin-right: 1em;
}
#messages .message .event {
	font-style: italic;
}
#input {
	width: 100%;
	background-color: #eee;
}'!

username
	^ self session username!

users
	^ (self pusher handlers
		reject: [ :each | self session == each session or: [ each session username isNil ] ])
		collect: [ :each | each session username ]! !
!CTChat categoriesFor: #isLoggedIn!public!testing! !
!CTChat categoriesFor: #login:!actions!public! !
!CTChat categoriesFor: #logout!actions!public! !
!CTChat categoriesFor: #message:!actions!public! !
!CTChat categoriesFor: #push:with:!private! !
!CTChat categoriesFor: #pusher!accessing!public! !
!CTChat categoriesFor: #renderChatInputOn:!public!rendering-chat! !
!CTChat categoriesFor: #renderChatMessagesOn:!public!rendering-chat! !
!CTChat categoriesFor: #renderChatOn:!public!rendering! !
!CTChat categoriesFor: #renderChatPusherOn:!public!rendering-chat! !
!CTChat categoriesFor: #renderChatStatusOn:!public!rendering-chat! !
!CTChat categoriesFor: #renderContentOn:!public!rendering! !
!CTChat categoriesFor: #renderLoginOn:!public!rendering! !
!CTChat categoriesFor: #renderMessage:class:on:!public!rendering! !
!CTChat categoriesFor: #style!public!rendering! !
!CTChat categoriesFor: #username!accessing!public! !
!CTChat categoriesFor: #users!accessing!public! !

!CTChat class methodsFor!

initialize
	| application |
	application := WAAdmin register: self asApplicationAt: 'comet/chat'.
	application preferenceAt: #sessionClass put: CTChatSession.
	application
		addLibrary: JQDeploymentLibrary;
		addLibrary: CTLibrary!

pusher
	^ pusher ifNil: [ pusher := CTPusher new ]! !
!CTChat class categoriesFor: #initialize!initialization!public! !
!CTChat class categoriesFor: #pusher!accessing!public! !

CTCounter guid: (GUID fromString: '{8F82E7D3-1EC1-4A8C-B14E-AE58CA3E21B3}')!
CTCounter comment: ''!
!CTCounter categoriesForClass!Unclassified! !
!CTCounter methodsFor!

decrease
	self model decrease.
	self update: #decrease!

increase
	self model increase.
	self update: #increase!

model
	^ self class model!

pusher
	^ self class pusher!

renderContentOn: html
	html heading
		id: #count;
		with: self model count.
	html paragraph: [
		html anchor
			onClick: (html jQuery ajax
				callback: [ self decrease ]);
			with: 'dec'.
		html space; space; space.
		html anchor
			onClick: (html jQuery ajax
				callback: [ self increase ]);
			with: 'inc' ].
	html document 
		addLoadScript: (html comet
			pusher: self pusher;
			connect)!

style
	^ 'body {
	color: #222;
	font-size: 4em;
	line-height: 1.5;
	text-align: center;
	font-family: "Helvetica Neue", Helvetica, Verdana, Arial, sans-serif;
}
h1 { font-weight: bold; }
a {
	color: #555;
	cursor: pointer;
	text-decoration: none;	
}'!

update: anAspect
	"Update the number in all connected web browser. Only the first 3 lines are really needed, the rest adds some eye-candy and can simply be removed if not needed."

	self pusher javascript: [ :script |
		script << (script jQuery: #count) 
			text: self model count;
			append: [ :html |
				html span: [
					html html: (anAspect = #increase
						ifTrue: [ '&uarr;' ]
						ifFalse: [ '&darr;' ]) ] ];
			find: 'span'; fadeOut: 'slow' ]! !
!CTCounter categoriesFor: #decrease!actions!public! !
!CTCounter categoriesFor: #increase!actions!public! !
!CTCounter categoriesFor: #model!accessing!public! !
!CTCounter categoriesFor: #pusher!accessing!public! !
!CTCounter categoriesFor: #renderContentOn:!public!rendering! !
!CTCounter categoriesFor: #style!public!rendering! !
!CTCounter categoriesFor: #update:!private! !

!CTCounter class methodsFor!

initialize
	| application |
	application := WAAdmin 	register: self asApplicationAt: 'comet/counter'.
	application addLibrary: JQDeploymentLibrary; addLibrary: CTLibrary!

model
	^ model ifNil: [ model := CTCounterModel new ]!

pusher
	^ pusher ifNil: [ pusher := CTPusher new ]! !
!CTCounter class categoriesFor: #initialize!initialization!public! !
!CTCounter class categoriesFor: #model!accessing!public! !
!CTCounter class categoriesFor: #pusher!accessing!public! !

CTReport guid: (GUID fromString: '{FC431072-7148-4AF8-82F9-E3D6FE48EBAD}')!
CTReport comment: ''!
!CTReport categoriesForClass!Unclassified! !
!CTReport methodsFor!

items
	^ self class items!

pusher
	^ self class pusher!

renderContentOn: html
	html table
		script: (html jQuery this
			bind: 'refresh' do: (html jQuery this load
				html: [ :h | self renderTableOn: h ]));
		with: [ self renderTableOn: html ].
	html document 
		addLoadScript: (html comet
			pusher: self pusher;
			connect)!

renderRowEdit: anItem on: html
	html tableData: [ 
		html textInput
			" focus the first input field "
			script: (html jQuery this triggerFocus);
			on: #name of: anItem ].
	html tableData: [ 
		html textInput 
			on: #phone of: anItem ].
	html tableData: [
		html anchor
			url: '#';
			onClick: (html jQuery ajax
				" serialize all form elements in the table "
				serialize: ((html jQuery this closest: 'table') find: ':input');
				" trigger a refresh on all connected tables "
				html: [ :r |
					self pusher javascript: [ :s |
						s add: ((s jQuery: 'table') 
							trigger: 'refresh') ] ]);
			with: 'save' ]!

renderRowView: anItem on: html
	html tableData: anItem name.
	html tableData: anItem phone.
	html tableData: [
		html anchor
			url: '#';
			" replace the owning row with an editable row "
			onClick: ((html jQuery this closest: 'tr') load
				html: [ :r | self renderRowEdit: anItem on: r ]);
			with: 'edit' ]!

renderTableOn: html
	html tableRow: [
		html tableHeading: 'Name'.
		html tableHeading: 'Phone'.
		html tableHeading ].
	self items do: [ :each |
		html tableRow: [
			self renderRowView: each on: html ] ]! !
!CTReport categoriesFor: #items!accessing!public! !
!CTReport categoriesFor: #pusher!accessing!public! !
!CTReport categoriesFor: #renderContentOn:!public!rendering! !
!CTReport categoriesFor: #renderRowEdit:on:!public!rendering! !
!CTReport categoriesFor: #renderRowView:on:!public!rendering! !
!CTReport categoriesFor: #renderTableOn:!public!rendering! !

!CTReport class methodsFor!

defaultModel
	^ OrderedCollection new
		add: (CTReportItem new
			name: 'Peter Parker';
			phone: '183 045 2037';
			yourself);
		add: (CTReportItem new
			name: 'John Hood';
			phone: '602 979 3001';
			yourself);
		add: (CTReportItem new
			name: 'Clark Kent';
			phone: '497 776 2568';
			yourself);
		add: (CTReportItem new
			name: 'Bruce Evans';
			phone: '167 604 1282';
			yourself);
		yourself!

initialize
	| application |
	application := WAAdmin register: self asApplicationAt: 'comet/report'.
	application
		addLibrary: JQDevelopmentLibrary;
		addLibrary: CTLibrary!

items
	^ items ifNil: [ items := self defaultModel ]!

pusher
	^ pusher ifNil: [ pusher := CTPusher new ]! !
!CTReport class categoriesFor: #defaultModel!private! !
!CTReport class categoriesFor: #initialize!initialization!public! !
!CTReport class categoriesFor: #items!accessing!public! !
!CTReport class categoriesFor: #pusher!accessing!public! !

CTSlider guid: (GUID fromString: '{ADB91591-A2D2-4530-819E-4B77272E4E4E}')!
CTSlider comment: ''!
!CTSlider categoriesForClass!Unclassified! !
!CTSlider methodsFor!

renderContentOn: html
	html div
		id: #slider;
		script: (html jQuery new slider
			value: self class value;
			onStop: (html jQuery ajax
				callback: [ :value | self class value: value asNumber ]
				value: (html jQuery this slider getValue))).
	html document
		addLoadScript: (html comet
			pusher: self class pusher;
			connect)! !
!CTSlider categoriesFor: #renderContentOn:!public!rendering! !

!CTSlider class methodsFor!

initialize
	| application |
	application := WAAdmin 	register: self asApplicationAt: 'comet/slider'.
	application 
		addLibrary: JQDevelopmentLibrary;
		addLibrary: JQUiDevelopmentLibrary;
		addLibrary: JQSmoothnessTheme;
		addLibrary: CTLibrary!

pusher
	^ pusher ifNil: [ pusher := CTPusher new ]!

value
	^ value ifNil: [ value := 50 ]!

value: aNumber
	value := aNumber.
	self pusher javascript: [ :script |
		script << (script jQuery: #slider) slider
			action: 'value'
			arguments: (Array with: value) ]! !
!CTSlider class categoriesFor: #initialize!initialization!public! !
!CTSlider class categoriesFor: #pusher!accessing!public! !
!CTSlider class categoriesFor: #value!accessing!public! !
!CTSlider class categoriesFor: #value:!accessing!public! !

CTChatSession guid: (GUID fromString: '{9E9246C0-9FE7-4082-8E73-E05D9DD2DEFF}')!
CTChatSession comment: ''!
!CTChatSession categoriesForClass!Unclassified! !
!CTChatSession methodsFor!

username
	^ username!

username: aString
	username := aString! !
!CTChatSession categoriesFor: #username!accessing!public! !
!CTChatSession categoriesFor: #username:!accessing!public! !

"Binary Globals"!

