| package |
package := Package name: 'RSS-Core'.
package paxVersion: 1;
	basicComment: ''.

package basicPackageVersion: '62'.


package classNames
	add: #RRCategoryTag;
	add: #RRCloudTag;
	add: #RRComponent;
	add: #RRDayTag;
	add: #RREnclosureTag;
	add: #RRGuidTag;
	add: #RRLanguageTag;
	add: #RRRfc822DateTime;
	add: #RRRssHandler;
	add: #RRRssRenderCanvas;
	add: #RRRssRoot;
	add: #RRSourceTag;
	add: #RRSystemConfiguration;
	add: #RRXmlStyleSheet;
	yourself.

package methodNames
	add: #WAHtmlRoot -> #rss;
	add: 'GRPackage class' -> #rssCore;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Grease\Grease-Core';
	add: '..\Seaside\Seaside-Canvas';
	add: '..\Seaside\Seaside-Core';
	yourself).

package!

"Class Definitions"!

WAObject subclass: #RRRfc822DateTime
	instanceVariableNames: 'dateAndTime timeZoneString'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WATagBrush subclass: #RRCategoryTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WATagBrush subclass: #RRCloudTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WATagBrush subclass: #RRDayTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WATagBrush subclass: #RREnclosureTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WATagBrush subclass: #RRGuidTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WATagBrush subclass: #RRLanguageTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WATagBrush subclass: #RRSourceTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WASystemConfiguration subclass: #RRSystemConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAContentElement subclass: #RRXmlStyleSheet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAPainter subclass: #RRComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAXmlCanvas subclass: #RRRssRenderCanvas
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WARequestHandler subclass: #RRRssHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAXmlRoot subclass: #RRRssRoot
	instanceVariableNames: 'rssAttributes channelAttributes styles'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!GRPackage class methodsFor!

rssCore
	^ self new
		name: 'RSS-Core';
		description: 'RSS integration for Seaside. Helps to create RSS 2.0 feeds.';
		addDependency: 'Seaside-Canvas';
		url: #seasideUrl;
		yourself! !
!GRPackage class categoriesFor: #rssCore!*rss-core!public! !

!WAHtmlRoot methodsFor!

rss
	^self link beAlternate; beRss; yourself! !
!WAHtmlRoot categoriesFor: #rss!*rss-core!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

RRRfc822DateTime guid: (GUID fromString: '{600CF69F-A09C-4F72-9801-7F8EF8F769AF}')!
RRRfc822DateTime comment: '
A RRRfc822DateTime is holds on to a DateAndTime and a timezone string. It is mainly used for correct formatting of RSS items. You can also use your native classes if they support the correct printing methods.

Instance Variables
	dateAndTime:		<DateAndTime>
	timezoneString:		<String>

dateAndTime
	- the date and time

timezoneString
	- the time zone string
'!
!RRRfc822DateTime categoriesForClass!Unclassified! !
!RRRfc822DateTime methodsFor!

dateAndTime
	^dateAndTime!

dateAndTime: anObject
	dateAndTime := anObject!

encodeOn: aDocument
	(GRPrinter rfc822WithTimeZone: self timeZoneString)
		print: self dateAndTime
		on: aDocument!

printString
	^ String streamContents: [ :stream | 
		self encodeOn: stream ]!

timeZoneString
	^timeZoneString!

timeZoneString: aString
	timeZoneString := aString! !
!RRRfc822DateTime categoriesFor: #dateAndTime!accessing!public! !
!RRRfc822DateTime categoriesFor: #dateAndTime:!accessing!public! !
!RRRfc822DateTime categoriesFor: #encodeOn:!printing!public! !
!RRRfc822DateTime categoriesFor: #printString!printing!public! !
!RRRfc822DateTime categoriesFor: #timeZoneString!accessing!public! !
!RRRfc822DateTime categoriesFor: #timeZoneString:!accessing!public! !

!RRRfc822DateTime class methodsFor!

dateAndTime: aDateAndTime timezoneString: aString
	self greaseDeprecatedApi: 'RRRfc822DateTime class>>#dateAndTime:timezoneString:' details: 'use #dateAndTime:timeZoneString: instead.'.
	^self dateAndTime: aDateAndTime timeZoneString: aString!

dateAndTime: aDateAndTime timeZoneString: aString
	^self new
		dateAndTime: aDateAndTime;
		timeZoneString: aString;
		yourself!

newDateAndTime: aDateAndTime
	^self
		newDateAndTime: aDateAndTime
		offset: aDateAndTime offset!

newDateAndTime: aDateAndTime offset: aDuration
	^self
		dateAndTime: aDateAndTime
		timeZoneString: (self offsetString: aDuration)!

newUtc: aDateAndTime
	^self
		dateAndTime: aDateAndTime asUTC
		timeZoneString: 'UT'!

offsetString: aDuration
	^String streamContents: [ :stream |
		stream nextPut: (aDuration positive ifTrue: [ $+ ] ifFalse: [ $- ]).
		stream nextPutAll: (aDuration hours abs printPaddedWith: $0 to: 2 base: 10).
		stream nextPutAll: (aDuration minutes abs printPaddedWith: $0 to: 2 base: 10) ]! !
!RRRfc822DateTime class categoriesFor: #dateAndTime:timezoneString:!instance creation!public! !
!RRRfc822DateTime class categoriesFor: #dateAndTime:timeZoneString:!instance creation!public! !
!RRRfc822DateTime class categoriesFor: #newDateAndTime:!instance creation!public! !
!RRRfc822DateTime class categoriesFor: #newDateAndTime:offset:!instance creation!public! !
!RRRfc822DateTime class categoriesFor: #newUtc:!instance creation!public! !
!RRRfc822DateTime class categoriesFor: #offsetString:!private! !

RRCategoryTag guid: (GUID fromString: '{D37FEEA3-EC92-442F-8D0C-4E6BFEF90DF7}')!
RRCategoryTag comment: ''!
!RRCategoryTag categoriesForClass!Unclassified! !
!RRCategoryTag methodsFor!

category: aString
	self attributeAt: 'category' put: aString!

tag
	^'category'! !
!RRCategoryTag categoriesFor: #category:!attributes!public! !
!RRCategoryTag categoriesFor: #tag!accessing!public! !

RRCloudTag guid: (GUID fromString: '{251016F3-37E6-491A-8BFE-064941EAE39E}')!
RRCloudTag comment: '
<cloud> is an optional sub-element of <channel>.

It specifies a web service that supports the rssCloud interface which can be implemented in HTTP-POST, XML-RPC or SOAP 1.1.

Its purpose is to allow processes to register with a cloud to be notified of updates to the channel, implementing a lightweight publish-subscribe protocol for RSS feeds. 

A full explanation of this element and the rssCloud interface is here:
http://www.rssboard.org/rsscloud-interface'!
!RRCloudTag categoriesForClass!Unclassified! !
!RRCloudTag methodsFor!

beSoap
	self protocol: 'soap'!

beXmlRpc
	self protocol: 'xml-rpc'!

domain: aString
	^self attributeAt: 'domain' put: aString!

path: aString
	^self attributeAt: 'path' put: aString!

port: anInteger
	^self attributeAt: 'port' put: anInteger!

protocol: aString
	^self attributeAt: 'protocol' put: aString!

registerProcedure: aString
	^self attributeAt: 'registerProcedure' put: aString!

tag
	^'cloud'! !
!RRCloudTag categoriesFor: #beSoap!protocols!public! !
!RRCloudTag categoriesFor: #beXmlRpc!protocols!public! !
!RRCloudTag categoriesFor: #domain:!attributes!public! !
!RRCloudTag categoriesFor: #path:!attributes!public! !
!RRCloudTag categoriesFor: #port:!attributes!public! !
!RRCloudTag categoriesFor: #protocol:!attributes!public! !
!RRCloudTag categoriesFor: #registerProcedure:!attributes!public! !
!RRCloudTag categoriesFor: #tag!accessing!public! !

!RRCloudTag class methodsFor!

tags
	"
	self generateTags
	"

	^ #(
		('item' item item: 'item')
	
		('title' title title: 'channel-required')
		('link' link link: 'channel-required')
		('description' description description: 'channel-required')

		
		('language' language language: 'channel-optional')
		('copyright' copyright copyright: 'channel-optional')
		('managingEditor' managingEditor managingEditor: 'channel-optional')
		('webMaster' webMaster webMaster: 'channel-optional')
		('pubDate' publicationDate publicationDate: 'channel-optional')
		('lastBuildDate' lastBuildDate lastBuildDate: 'channel-optional')
		('category' category category: 'channel-optional')
		('generator' generator generator: 'channel-optional')
		('docs' documentation documentation: 'channel-optional')
		('cloud' cloud cloud: 'channel-optional')
		('ttl' timeToLive timeToLive: 'channel-optional')
		('image' image image: 'channel-optional')
		('rating' rating rating: 'channel-optional')
		('textInput' textInput textInput: 'channel-optional')
		('skipHours' skipHours skipHours: 'channel-optional')
		('skipDays' skipDays skipDays: 'channel-optional')
		
		('comments' comments comments: 'item-optional')
		('enclosure' enclosure enclosure: 'item-optional')
		('guid' guid guid: 'item-optional')
		
		('name' name name: 'image-required')
		
		)! !
!RRCloudTag class categoriesFor: #tags!accessing-tags!public! !

RRDayTag guid: (GUID fromString: '{AF8A601E-88C7-415D-A671-8A58DBA2D24C}')!
RRDayTag comment: ''!
!RRDayTag categoriesForClass!Unclassified! !
!RRDayTag methodsFor!

friday
	^self with: 'Friday'!

monday
	^self with: 'Monday'!

saturday
	^self with: 'Saturday'!

sunday
	^self with: 'Sunday'!

tag
	^'day'!

thursday
	^self with: 'Thursday'!

tuesday
	^self with: 'Tuesday'!

wednesday
	^self with: 'Wednesday'! !
!RRDayTag categoriesFor: #friday!attributes!public! !
!RRDayTag categoriesFor: #monday!attributes!public! !
!RRDayTag categoriesFor: #saturday!attributes!public! !
!RRDayTag categoriesFor: #sunday!attributes!public! !
!RRDayTag categoriesFor: #tag!accessing!public! !
!RRDayTag categoriesFor: #thursday!attributes!public! !
!RRDayTag categoriesFor: #tuesday!attributes!public! !
!RRDayTag categoriesFor: #wednesday!attributes!public! !

RREnclosureTag guid: (GUID fromString: '{913FE40D-CA3D-4058-8E6A-5F8277DFFEF3}')!
RREnclosureTag comment: ''!
!RREnclosureTag categoriesForClass!Unclassified! !
!RREnclosureTag methodsFor!

length: anInteger
	self attributeAt: 'length' put: anInteger!

tag
	^'enclosure'!

type: aString
	self attributeAt: 'type' put: aString!

url: aString
	self attributeAt: 'url' put: aString! !
!RREnclosureTag categoriesFor: #length:!attributes!public! !
!RREnclosureTag categoriesFor: #tag!accessing!public! !
!RREnclosureTag categoriesFor: #type:!attributes!public! !
!RREnclosureTag categoriesFor: #url:!attributes!public! !

RRGuidTag guid: (GUID fromString: '{541E4F9C-E10D-483D-869F-B934C80683FD}')!
RRGuidTag comment: ''!
!RRGuidTag categoriesForClass!Unclassified! !
!RRGuidTag methodsFor!

beNoPermaLink
	self isPermaLink: false!

bePermaLink
	self isPermaLink: true!

isPermaLink: aBoolean 
	"the value really must be a string else seaside makes
	isPermaLink='isPermaLink'"
	self 
		attributeAt: 'isPermaLink'
		put: aBoolean greaseString!

tag
	^'guid'! !
!RRGuidTag categoriesFor: #beNoPermaLink!attributes!public! !
!RRGuidTag categoriesFor: #bePermaLink!attributes!public! !
!RRGuidTag categoriesFor: #isPermaLink:!attributes!public! !
!RRGuidTag categoriesFor: #tag!accessing!public! !

RRLanguageTag guid: (GUID fromString: '{24CBBC19-F8E4-412E-8E12-B7472CA2390F}')!
RRLanguageTag comment: ''!
!RRLanguageTag categoriesForClass!Unclassified! !
!RRLanguageTag methodsFor!

afrikaans
	self with: 'af'!

albanian
	self with: 'sq'!

basque
	self with: 'eu'!

belarusian
	self with: 'be'!

bulgarian
	self with: 'bg'!

catalan
	self with: 'ca'!

chineseSimplified
	self with: 'zh-cn'!

chineseTraditional
	self with: 'zh-tw'!

croatian
	self with: 'hr'!

czech
	self with: 'cs'!

danish
	self with: 'da'!

dutch
	self with: 'nl'!

dutchBelgium
	self with: 'nl-be'!

dutchNetherlands
	self with: 'nl-nl'!

english
	self with: 'en'!

englishAustralia
	self with: 'en-au'!

englishBelize
	self with: 'en-bz'!

englishCanada
	self with: 'en-ca'!

englishIreland
	self with: 'en-ie'!

englishJamaica
	self with: 'en-jm'!

englishNewZealand
	self with: 'en-nz'!

englishPhillipines
	self with: 'en-ph'!

englishSouthAfrica
	self with: 'en-za'!

englishTrinidad
	self with: 'en-tt'!

englishUnitedKingdom
	self with: 'en-gb'!

englishUnitedStates
	self with: 'en-us'!

englishZimbabwe
	self with: 'en-zw'!

estonian
	self with: 'et'!

faeroese
	self with: 'fo'!

finnish
	self with: 'fi'!

french
	self with: 'fr'!

frenchBelgium
	self with: 'fr-be'!

frenchCanada
	self with: 'fr-ca'!

frenchFrance
	self with: 'fr-fr'!

frenchLuxembourg
	self with: 'fr-lu'!

frenchMonaco
	self with: 'fr-mc'!

frenchSwitzerland
	self with: 'fr-ch'!

gaelic
	self with: 'gd'!

galician
	self with: 'gl'!

german
	self with: 'de'!

germanAustria
	self with: 'de-at'!

germanGermany
	self with: 'de-de'!

germanLiechtenstein
	self with: 'de-li'!

germanLuxembourg
	self with: 'de-lu'!

germanSwitzerland
	self with: 'de-ch'!

greek
	self with: 'el'!

hawaiian
	self with: 'haw'!

hungarian
	self with: 'hu'!

icelandic
	self with: 'is'!

indonesian
	self with: 'in'!

irish
	self with: 'ga'!

italian
	self with: 'it'!

italianItaly
	self with: 'it-it'!

italianSwitzerland
	self with: 'it-ch'!

japanese
	self with: 'ja'!

korean
	self with: 'ko'!

macedonian
	self with: 'mk'!

norwegian
	self with: 'no'!

polish
	self with: 'pl'!

portuguese
	self with: 'pt'!

portugueseBrazil
	self with: 'pt-br'!

portuguesePortugal
	self with: 'pt-pt'!

romanian
	self with: 'ro'!

romanianMoldova
	self with: 'ro-mo'!

romanianRomania
	self with: 'ro-ro'!

russian
	self with: 'ru'!

russianMoldova
	self with: 'ru-mo'!

russianRussia
	self with: 'ru-ru'!

serbian
	self with: 'sr'!

slovak
	self with: 'sk'!

slovenian
	self with: 'sl'!

spanish
	self with: 'es'!

spanishArgentina
	self with: 'es-ar'!

spanishBolivia
	self with: 'es-bo'!

spanishChile
	self with: 'es-cl'!

spanishColombia
	self with: 'es-co'!

spanishCostaRica
	self with: 'es-cr'!

spanishDominicanRepublic
	self with: 'es-do'!

spanishEcuador
	self with: 'es-ec'!

spanishElSalvador
	self with: 'es-sv'!

spanishGuatemala
	self with: 'es-gt'!

spanishHonduras
	self with: 'es-hn'!

spanishMexico
	self with: 'es-mx'!

spanishNicaragua
	self with: 'es-ni'!

spanishPanama
	self with: 'es-pa'!

spanishParaguay
	self with: 'es-py'!

spanishPeru
	self with: 'es-pe'!

spanishPuertoRico
	self with: 'es-pr'!

spanishSpain
	self with: 'es-es'!

spanishUruguay
	self with: 'es-uy'!

spanishVenezuela
	self with: 'es-ve'!

swedish
	self with: 'sv'!

swedishFinland
	self with: 'sv-fi'!

swedishSweden
	self with: 'sv-se'!

tag
	^'language'!

turkish
	self with: 'tr'!

ukranian
	self with: 'uk'! !
!RRLanguageTag categoriesFor: #afrikaans!languages-generated!public! !
!RRLanguageTag categoriesFor: #albanian!languages-generated!public! !
!RRLanguageTag categoriesFor: #basque!languages-generated!public! !
!RRLanguageTag categoriesFor: #belarusian!languages-generated!public! !
!RRLanguageTag categoriesFor: #bulgarian!languages-generated!public! !
!RRLanguageTag categoriesFor: #catalan!languages-generated!public! !
!RRLanguageTag categoriesFor: #chineseSimplified!languages-generated!public! !
!RRLanguageTag categoriesFor: #chineseTraditional!languages-generated!public! !
!RRLanguageTag categoriesFor: #croatian!languages-generated!public! !
!RRLanguageTag categoriesFor: #czech!languages-generated!public! !
!RRLanguageTag categoriesFor: #danish!languages-generated!public! !
!RRLanguageTag categoriesFor: #dutch!languages-generated!public! !
!RRLanguageTag categoriesFor: #dutchBelgium!languages-generated!public! !
!RRLanguageTag categoriesFor: #dutchNetherlands!languages-generated!public! !
!RRLanguageTag categoriesFor: #english!languages-generated!public! !
!RRLanguageTag categoriesFor: #englishAustralia!languages-generated!public! !
!RRLanguageTag categoriesFor: #englishBelize!languages-generated!public! !
!RRLanguageTag categoriesFor: #englishCanada!languages-generated!public! !
!RRLanguageTag categoriesFor: #englishIreland!languages-generated!public! !
!RRLanguageTag categoriesFor: #englishJamaica!languages-generated!public! !
!RRLanguageTag categoriesFor: #englishNewZealand!languages-generated!public! !
!RRLanguageTag categoriesFor: #englishPhillipines!languages-generated!public! !
!RRLanguageTag categoriesFor: #englishSouthAfrica!languages-generated!public! !
!RRLanguageTag categoriesFor: #englishTrinidad!languages-generated!public! !
!RRLanguageTag categoriesFor: #englishUnitedKingdom!languages-generated!public! !
!RRLanguageTag categoriesFor: #englishUnitedStates!languages-generated!public! !
!RRLanguageTag categoriesFor: #englishZimbabwe!languages-generated!public! !
!RRLanguageTag categoriesFor: #estonian!languages-generated!public! !
!RRLanguageTag categoriesFor: #faeroese!languages-generated!public! !
!RRLanguageTag categoriesFor: #finnish!languages-generated!public! !
!RRLanguageTag categoriesFor: #french!languages-generated!public! !
!RRLanguageTag categoriesFor: #frenchBelgium!languages-generated!public! !
!RRLanguageTag categoriesFor: #frenchCanada!languages-generated!public! !
!RRLanguageTag categoriesFor: #frenchFrance!languages-generated!public! !
!RRLanguageTag categoriesFor: #frenchLuxembourg!languages-generated!public! !
!RRLanguageTag categoriesFor: #frenchMonaco!languages-generated!public! !
!RRLanguageTag categoriesFor: #frenchSwitzerland!languages-generated!public! !
!RRLanguageTag categoriesFor: #gaelic!languages-generated!public! !
!RRLanguageTag categoriesFor: #galician!languages-generated!public! !
!RRLanguageTag categoriesFor: #german!languages-generated!public! !
!RRLanguageTag categoriesFor: #germanAustria!languages-generated!public! !
!RRLanguageTag categoriesFor: #germanGermany!languages-generated!public! !
!RRLanguageTag categoriesFor: #germanLiechtenstein!languages-generated!public! !
!RRLanguageTag categoriesFor: #germanLuxembourg!languages-generated!public! !
!RRLanguageTag categoriesFor: #germanSwitzerland!languages-generated!public! !
!RRLanguageTag categoriesFor: #greek!languages-generated!public! !
!RRLanguageTag categoriesFor: #hawaiian!languages-generated!public! !
!RRLanguageTag categoriesFor: #hungarian!languages-generated!public! !
!RRLanguageTag categoriesFor: #icelandic!languages-generated!public! !
!RRLanguageTag categoriesFor: #indonesian!languages-generated!public! !
!RRLanguageTag categoriesFor: #irish!languages-generated!public! !
!RRLanguageTag categoriesFor: #italian!languages-generated!public! !
!RRLanguageTag categoriesFor: #italianItaly!languages-generated!public! !
!RRLanguageTag categoriesFor: #italianSwitzerland!languages-generated!public! !
!RRLanguageTag categoriesFor: #japanese!languages-generated!public! !
!RRLanguageTag categoriesFor: #korean!languages-generated!public! !
!RRLanguageTag categoriesFor: #macedonian!languages-generated!public! !
!RRLanguageTag categoriesFor: #norwegian!languages-generated!public! !
!RRLanguageTag categoriesFor: #polish!languages-generated!public! !
!RRLanguageTag categoriesFor: #portuguese!languages-generated!public! !
!RRLanguageTag categoriesFor: #portugueseBrazil!languages-generated!public! !
!RRLanguageTag categoriesFor: #portuguesePortugal!languages-generated!public! !
!RRLanguageTag categoriesFor: #romanian!languages-generated!public! !
!RRLanguageTag categoriesFor: #romanianMoldova!languages-generated!public! !
!RRLanguageTag categoriesFor: #romanianRomania!languages-generated!public! !
!RRLanguageTag categoriesFor: #russian!languages-generated!public! !
!RRLanguageTag categoriesFor: #russianMoldova!languages-generated!public! !
!RRLanguageTag categoriesFor: #russianRussia!languages-generated!public! !
!RRLanguageTag categoriesFor: #serbian!languages-generated!public! !
!RRLanguageTag categoriesFor: #slovak!languages-generated!public! !
!RRLanguageTag categoriesFor: #slovenian!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanish!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishArgentina!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishBolivia!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishChile!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishColombia!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishCostaRica!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishDominicanRepublic!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishEcuador!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishElSalvador!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishGuatemala!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishHonduras!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishMexico!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishNicaragua!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishPanama!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishParaguay!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishPeru!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishPuertoRico!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishSpain!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishUruguay!languages-generated!public! !
!RRLanguageTag categoriesFor: #spanishVenezuela!languages-generated!public! !
!RRLanguageTag categoriesFor: #swedish!languages-generated!public! !
!RRLanguageTag categoriesFor: #swedishFinland!languages-generated!public! !
!RRLanguageTag categoriesFor: #swedishSweden!languages-generated!public! !
!RRLanguageTag categoriesFor: #tag!accessing!public! !
!RRLanguageTag categoriesFor: #turkish!languages-generated!public! !
!RRLanguageTag categoriesFor: #ukranian!languages-generated!public! !

!RRLanguageTag class methodsFor!

codes
	^Array streamContents: [ :stream |
		self codesString lines do: [ :line |
			stream nextPut: (Array
				with: (line copyUpTo: Character tab) asLegalSelector
				with: (line copyAfter: Character tab)) ] ]!

codesString
	^
'Afrikaans	af
Albanian	sq
Basque	eu
Belarusian	be
Bulgarian	bg
Catalan	ca
Chinese (Simplified)	zh-cn
Chinese (Traditional)	zh-tw
Croatian	hr
Czech	cs
Danish	da
Dutch	nl
Dutch (Belgium)	nl-be
Dutch (Netherlands)	nl-nl
English	en
English (Australia)	en-au
English (Belize)	en-bz
English (Canada)	en-ca
English (Ireland)	en-ie
English (Jamaica)	en-jm
English (New Zealand)	en-nz
English (Phillipines)	en-ph
English (South Africa)	en-za
English (Trinidad)	en-tt
English (United Kingdom)	en-gb
English (United States)	en-us
English (Zimbabwe)	en-zw
Estonian	et
Faeroese	fo
Finnish	fi
French	fr
French (Belgium)	fr-be
French (Canada)	fr-ca
French (France)	fr-fr
French (Luxembourg)	fr-lu
French (Monaco)	fr-mc
French (Switzerland)	fr-ch
Galician	gl
Gaelic	gd
German	de
German (Austria)	de-at
German (Germany)	de-de
German (Liechtenstein)	de-li
German (Luxembourg)	de-lu
German (Switzerland)	de-ch
Greek	el
Hawaiian	haw
Hungarian	hu
Icelandic	is
Indonesian	in
Irish	ga
Italian	it
Italian (Italy)	it-it
Italian (Switzerland)	it-ch
Japanese	ja
Korean	ko
Macedonian	mk
Norwegian	no
Polish	pl
Portuguese	pt
Portuguese (Brazil)	pt-br
Portuguese (Portugal)	pt-pt
Romanian	ro
Romanian (Moldova)	ro-mo
Romanian (Romania)	ro-ro
Russian	ru
Russian (Moldova)	ru-mo
Russian (Russia)	ru-ru
Serbian	sr
Slovak	sk
Slovenian	sl
Spanish	es
Spanish (Argentina)	es-ar
Spanish (Bolivia)	es-bo
Spanish (Chile)	es-cl
Spanish (Colombia)	es-co
Spanish (Costa Rica)	es-cr
Spanish (Dominican Republic)	es-do
Spanish (Ecuador)	es-ec
Spanish (El Salvador)	es-sv
Spanish (Guatemala)	es-gt
Spanish (Honduras)	es-hn
Spanish (Mexico)	es-mx
Spanish (Nicaragua)	es-ni
Spanish (Panama)	es-pa
Spanish (Paraguay)	es-py
Spanish (Peru)	es-pe
Spanish (Puerto Rico)	es-pr
Spanish (Spain)	es-es
Spanish (Uruguay)	es-uy
Spanish (Venezuela)	es-ve
Swedish	sv
Swedish (Finland)	sv-fi
Swedish (Sweden)	sv-se
Turkish	tr
Ukranian	uk'!

generatedCategory
	^'languages-generated'!

generateFor: aPair
	(self includesSelector: aPair first) ifFalse: [
		self 
			compile: aPair first, '
	self with: ''', aPair last, ''''
			classified: self generatedCategory ]!

generateTags
	^self codes do: [ :each |
		self generateFor: each ]! !
!RRLanguageTag class categoriesFor: #codes!accessing!public! !
!RRLanguageTag class categoriesFor: #codesString!accessing!public! !
!RRLanguageTag class categoriesFor: #generatedCategory!accessing!public! !
!RRLanguageTag class categoriesFor: #generateFor:!generating!public! !
!RRLanguageTag class categoriesFor: #generateTags!generating!public! !

RRSourceTag guid: (GUID fromString: '{874C0A73-24B3-42B1-A695-4FF24C54F2AF}')!
RRSourceTag comment: ''!
!RRSourceTag categoriesForClass!Unclassified! !
!RRSourceTag methodsFor!

tag
	^'source'!

url: aString
	self attributeAt: 'url' put: aString! !
!RRSourceTag categoriesFor: #tag!accessing!public! !
!RRSourceTag categoriesFor: #url:!attributes!public! !

RRSystemConfiguration guid: (GUID fromString: '{D3C9CC82-9133-4DB2-9F3D-276904FF31B1}')!
RRSystemConfiguration comment: 'A RRSystemConfiguration is the configuration of an RRRssHandler.'!
!RRSystemConfiguration categoriesForClass!Unclassified! !
!RRSystemConfiguration methodsFor!

describeOn: config
	(config list: #rootComponentClass)
		label: 'Root Component';
		options: [ self rootComponents ].
	config at: #documentClass put: WAXmlDocument!

parents
	^ Array with: WARequestHandlingConfiguration instance!

rootComponents
	^(RRComponent allSubclasses select: [ :each | each canBeRoot ])
		asSortedCollection: [ :a :b | a name <= b name ]! !
!RRSystemConfiguration categoriesFor: #describeOn:!description!public! !
!RRSystemConfiguration categoriesFor: #parents!accessing!public! !
!RRSystemConfiguration categoriesFor: #rootComponents!accessing!public! !

RRXmlStyleSheet guid: (GUID fromString: '{844292F2-12B0-4981-8C68-9C1B311D5C6F}')!
RRXmlStyleSheet comment: ''!
!RRXmlStyleSheet categoriesForClass!Unclassified! !
!RRXmlStyleSheet methodsFor!

beCss
	self type: WAMimeType textCss!

beXsl
	self type: 'text/xsl' seasideMimeType!

encodeOn: aDocument
	aDocument nextPutAll: '<?xml-stylesheet'.
	
	attributes encodeOn: aDocument.
	self childrenDo: [ :each | each encodeOn: aDocument ].
	
	aDocument nextPutAll: '?>'!

tag
	self shouldNotImplement!

url: aString
	self attributeAt: 'href' put: aString! !
!RRXmlStyleSheet categoriesFor: #beCss!attributes!public! !
!RRXmlStyleSheet categoriesFor: #beXsl!attributes!public! !
!RRXmlStyleSheet categoriesFor: #encodeOn:!printing!public! !
!RRXmlStyleSheet categoriesFor: #tag!accessing!public! !
!RRXmlStyleSheet categoriesFor: #url:!attributes!public! !

RRComponent guid: (GUID fromString: '{998AD1BE-B61D-4AC8-8A23-56CB4BC2C2C0}')!
RRComponent comment: 'Base class for all RSS rendering components. Note that RSS components are stripped down versions of Seaside components. Many things like #call: don''t work. Also #initialRequest: and #updateRoot: only get sent to the root component.'!
!RRComponent categoriesForClass!Unclassified! !
!RRComponent methodsFor!

rendererClass
	^RRRssRenderCanvas!

session
	self shouldNotImplement! !
!RRComponent categoriesFor: #rendererClass!public!rendering! !
!RRComponent categoriesFor: #session!accessing!public! !

RRRssRenderCanvas guid: (GUID fromString: '{E722578C-A68B-4C75-A312-5A170178AADF}')!
RRRssRenderCanvas comment: '
Structure of an RSS Document
Elements with a star (*) at the beginning are required. Elements with a plus sign (+) are an extension. Channel is automatically open.

channel
	*title
	*link
	*description
	language
	copyright
	managingEditor
	webMaster
	publicationDate
	lastBuildDate
	category
	generator
	documentation
	cloud
	timeToLive
	image
		*url
		*title
		*link
		width
		height
		description
	rating
	textInput
		*title
		*description
		*name
		*link
	skipHours
		hour
	skipDays
		day
	item
		*title
		*link
		*description
		author
		category
		comments
		enclosure
		guid
		publicationDate
		source
		+creator
		+comment
		+commentRss
		+contentEncoded
		
Email Addresses:
Email addresses must conform to RFC 2822
Convert the email address to a valid form. Examples of valid email addresses:
    * joesmith@example.com
    * joesmith@example.com (Joe Smith)
    * Joe Smith <joesmith@example.com>
    * joesmith.nospamplease@nospam.example.com
You can either:
- use correctly formatted strings
- implement #displayString in your domain email address class so that it return a string in a valid form
- implement #renderOn: in your domain  email address class so that it renders a string in a valid form
- use RREmailAddress

date times:
The value specified must meet the Date and Time specifications as defined by RFC822, with the exception that the year should be expressed as four digits.
Here are examples of valid RFC822 date-times:
Wed, 02 Oct 2002 08:00:00 EST
Wed, 02 Oct 2002 13:00:00 GMT
Wed, 02 Oct 2002 15:00:00 +0200

If you use Chronos you can use
printStringUsing: printStringUsing: ChronosPrintPolicy rfc822
Else you can use RRRfc822DateTime.

Here is an example of an invalid RFC822 date-time:
2002-10-02T08:00:00-05:00
'!
!RRRssRenderCanvas categoriesForClass!Unclassified! !
!RRRssRenderCanvas methodsFor!

author
	"Email address of the author of the item.
	
	It's the email address of the author of the item. For newspapers and magazines syndicating via RSS, the author is the person who wrote the article that the <item> describes. For collaborative weblogs, the author of the item might be different from the managing editor or webmaster. For a weblog authored by a single individual it would make sense to omit the <author> element."
	^ self tag: 'author'!

author: aBlock
	self author with: aBlock!

category
	"---- as optional subelement of channel ----
	Specify one or more categories that the channel belongs to. Follows the same rules as the <item>-level category element below.
	
	---- as optional subelement of item ----
	Includes the item in one or more categories.
	
	The value of the element is a forward-slash-separated string that identifies a hierarchic location in the indicated taxonomy. Processors may establish conventions for the interpretation of categories.
	
	You may include as many category elements as you need to, for different domains, and to have an item cross-referenced in different parts of the same domain.
	
	It has one optional attribute, domain, a string that identifies a categorization taxonomy.
	
	In RSS 2.0, a provision is made for linking a channel to its identifier in a cataloging system, using the channel-level category feature, described above. For example, to link a channel to its Syndic8 identifier, include a category element as a sub-element of <channel>, with domain 'Syndic8', and value the identifier for your channel in the Syndic8 database. The appropriate category element for Scripting News would be
	
	rss category
		domain: 'Syndic8';
		with: 1765"
	^ self brush: RRCategoryTag new!

category: aBlock
	self category with: aBlock!

cloud
	"Allows processes to register with a cloud to be notified of updates to the channel, implementing a lightweight publish-subscribe protocol for RSS feeds.
	
	for more info see RRCloudTag"
	^ self brush: RRCloudTag new!

cloud: aBlock
	self cloud with: aBlock!

comment
	"This element appears in RSS feeds and contains the URI that comment entries are to be POSTed to. The details of this are outlined in the CommentAPI Specification (http://wellformedweb.org/story/9)"
	^self tag: 'wfw:comment'!

comment: aString!

commentRss
	"This element also appears in RSS feeds and contains the URI of the RSS feed for comments on that Item. This is documented in Chris Sells' Specification(http://www.sellsbrothers.com/spout/default.aspx?content=archive.htm#exposingRssComments). Note that for quite a while this page has had a typo and erroneously referred to this element as 'commentRSS' as opposed to the correct 'commentRss'. Feed consumers should be aware that they may run into both spellings in the wild. Please see this page (http://www.intertwingly.net/blog/2006/04/16/commentRss) for more information."
	^self tag: 'wfw:commentRss'!

commentRss: aBlock
	^self commentRss with: aBlock!

comments
	"URL of a page for comments relating to the item.
	
	If present, it is the url of the comments page for the item."
	^ self tag: 'comments'!

comments: aBlock
	self comments with: aBlock!

contentEncoded
	"A module for the actual content of websites, in multiple formats.
	http://web.resource.org/rss/1.0/modules/content/
	
	example:
	rss contentEncoded: '<p>What a <em>beautiful</em> day!!</p>'"
	^ self tag: 'content:encoded'!

contentEncoded: aString
	"A module for the actual content of websites.
	
	aString must not include ']]>'"
	self contentEncoded
		with: [ self cdata: aString ]!

copyright
	"Copyright notice for content in the channel."
	^ self tag: 'copyright'!

copyright: aBlock
	self copyright with: aBlock!

creator
	"An entity primarily responsible for making the resource.
	Examples of a Creator include a person, an organization, or a service. Typically, the name of a Creator should be used to indicate the entity.
	
	http://dublincore.org/documents/dces/
	http://purl.org/dc/elements/1.1/creator"
	^ self tag: 'dc:creator'!

creator: aBlock
	self creator with: aBlock!

day
	^ self brush: RRDayTag new!

day: aBlock
	self day with: aBlock!

defaultDocumentation
	self documentation: 'http://www.rssboard.org/rss-specification'!

defaultGenerator
	self generator: self defaultGeneratorString!

defaultGeneratorString
	^'Seaside-RSRSS'!

description
	"
	---- as required subelement of channel ----
	Phrase or sentence describing the channel.
	e.g.: 'The latest news from GoUpstate.com, a Spartanburg Herald-Journal Web site.'
	
	---- a required subelement of item ----
	The item synopsis.
	
	---- as required subelement of textInput ----
	Explains the text input area.
	
	----  as an optional subelement of image ----
	contains text that is included in the TITLE attribute of the link formed around the image in the HTML rendering."
	^ self tag: 'description'!

description: aBlock
	self description with: aBlock!

documentation
	"A URL that points to the documentation for the format used in the RSS file. It's probably a pointer to this page. It's for people who might stumble across an RSS file on a Web server 25 years from now and wonder what it is.
	e.g. 'http://www.rssboard.org/rss-specification'
	see #defaultDocumentation"
	^ self tag: 'docs'!

documentation: aBlock
	self documentation with: aBlock!

enclosure
	"Describes a media object that is attached to the item.
	
	It has three required attributes. url says where the enclosure is located, length says how big it is in bytes, and type says what its type is, a standard MIME type.
	
	The url must be an http url.
	
	e.g.:
	rss enclosure
		url: 'http://www.scripting.com/mp3s/weatherReportSuite.mp3';
		length: 12216320;
		type: 'audio/mpeg'"
	^ self brush: RREnclosureTag new!

enclosure: aBlock
	self enclosure with: aBlock!

generator
	"A string indicating the program used to generate the channel.
	e.g.: 'RSRSS-pmm.3'
	see #defaultGenerator"
	^ self tag: 'generator'!

generator: aBlock
	self generator with: aBlock!

guid
	"A string that uniquely identifies the item.
	
	guid stands for globally unique identifier. It's a string that uniquely identifies the item. When present, an aggregator may choose to use this string to determine if an item is new.
	
	e.g.: 'http://some.server.com/weblogItem3207'
	
	There are no rules for the syntax of a guid. Aggregators must view them as a string. It's up to the source of the feed to establish the uniqueness of the string.
	
	If the guid element has an attribute named isPermaLink with a value of true, the reader may assume that it is a permalink to the item, that is, a url that can be opened in a Web browser, that points to the full item described by the <item> element. An example:
	
	e.g.:
	rss guid
		bePermaLink;
		with: 'http://inessential.com/2002/09/01.php#a2'
		
	isPermaLink is optional, its default value is true. If its value is false, the guid may not be assumed to be a url, or a url to anything in particular."
	^ self brush: RRGuidTag new!

guid: aBlock
	self guid with: aBlock!

height
	"Optional elements include <width> and <height>, numbers, indicating the width and height of the image in pixels.
	
	Maximum value for height is 400, default value is 31."
	^ self tag: 'height'!

height: aBlock
	self height with: aBlock!

hour
	^ self tag: 'hour'!

hour: aBlock
	self hour with: aBlock!

html: aString
	self flush.
	self document nextPutAll: aString!

image
	"Specifies a GIF, JPEG or PNG image that can be displayed with the channel.
	the following subelements are required:
		url
		title
		link
	the following subelements are optional:
		width
		height
		description
	"
	^ self tag: 'image'!

image: aBlock
	self image with: aBlock!

item
	^ self tag: 'item'!

item: aBlock
	self item with: aBlock!

language
	"The language the channel is written in. This allows aggregators to group all Italian language sites, for example, on a single page. A list of allowable values for this element, as provided by Netscape, is implemented in the 'languages-generated' protocl of RRLanguageTag. You may also use values defined by the W3C (http://www.w3.org/TR/REC-html40/struct/dirlang.html#langcodes)."
	^ self brush: RRLanguageTag new!

language: aBlock
	self language with: aBlock!

lastBuildDate
	"The last time the content of the channel changed.
	again an RFC 822 date like all the other dates
	e.g.: 'Sat, 07 Sep 2002 09:42:31 GMT'"
	^ self tag: 'lastBuildDate'!

lastBuildDate: aBlock
	self lastBuildDate with: aBlock!

link
	"---- as required subelement of channel ----
	The URL to the HTML website corresponding to the channel.
	e.g.: 'http://www.goupstate.com/'
	
	---- as required subelement of item ----
	The URL of the item.
	
	---- as required subelement of image ----
	is the URL of the site, when the channel is rendered, the image is a link to the site. (Note, in practice the image <title> and <link> should have the same value as the channel's <title> and <link>
	
	---- as required subelement of textInput ----
	The URL of the CGI script that processes text input requests."
	
	^ self tag: 'link'!

link: aBlock
	self link with: aBlock!

managingEditor
	"Email address for person responsible for editorial content.
	e.g.: 'George Matesky <geo@herald.com>'"
	^ self tag: 'managingEditor'!

managingEditor: aBlock
	self managingEditor with: aBlock!

name
	"The name of the text object in the text input area."
	^ self tag: 'name'!

name: aBlock
	self name with: aBlock!

publicationDate
	"---- as optional subelement of channel ----
	The publication date for the content in the channel. For example, the New York Times publishes on a daily basis, the publication date flips once every 24 hours. That's when the pubDate of the channel changes. All date-times in RSS conform to the Date and Time Specification of RFC 822(http://asg.web.cmu.edu/rfc/rfc822.html#sec-5), with the exception that the year may be expressed with two characters or four characters (four preferred).
	
	---- as optional subelement of item ----
	Indicates when the item was published.
	Its value is a date, indicating when the item was published. If it's a date in the future, aggregators may choose to not display the item until that date.
	
	e.g: 'Sat, 07 Sep 2002 00:00:01 GMT'
	minimal: '20 Jun 82 23:59 EST'"
	^ self tag: 'pubDate'!

publicationDate: aBlock
	self publicationDate with: aBlock!

rating
	"The PICS (http://www.w3.org/PICS/) rating for the channel."
	^ self tag: 'rating'!

rating: aBlock
	self rating with: aBlock!

renderContentEncoded: aOneArgumentBlock
	"renders the html content of an item
	a render canvas will be passed as an argument to the block"
	self contentEncoded
		with: [
			self renderHtml: aOneArgumentBlock ]!

renderHtml: aOneArgumentBlock
	"convenvience method to render html
	aOneArgumentBlock will be passed an html canvas"
	self text: (WARenderCanvas builder render: aOneArgumentBlock)!

skipDays
	"A hint for aggregators telling them which days they can skip.
	
	An XML element that contains up to seven <day> sub-elements whose value is Monday, Tuesday, Wednesday, Thursday, Friday, Saturday or Sunday. Aggregators may not read the channel during days listed in the skipDays element."
	^ self tag: 'skipDays'!

skipDays: aBlock
	self skipDays with: aBlock!

skipHours
	"A hint for aggregators telling them which hours they can skip.
	
	An XML element that contains up to 24 <hour> sub-elements whose value is a number between 0 and 23, representing a time in GMT, when aggregators, if they support the feature, may not read the channel on hours listed in the skipHours element. The hour beginning at midnight is hour zero."
	^ self tag: 'skipHours'!

skipHours: aBlock
	self skipHours with: aBlock!

source
	"The RSS channel that the item came from.
	
	Its value is the name of the RSS channel that the item came from, derived from its <title>. It has one required attribute, url, which links to the XMLization of the source.
	
	rss source
		url: 'http://www.tomalak.org/links2.xml';
		with: 'Tomalak''s Realm'
	
	The purpose of this element is to propagate credit for links, to publicize the sources of news items. It can be used in the Post command of an aggregator. It should be generated automatically when forwarding an item from an aggregator to a weblog authoring tool."
	^ self brush: RRSourceTag new!

source: aBlock
	self source with: aBlock!

space
	self html: self spaceEntity!

textInput
	"Specifies a text input box that can be displayed with the channel.
	The purpose of the <textInput> element is something of a mystery. You can use it to specify a search engine box. Or to allow a reader to provide feedback. Most aggregators ignore it.
	
	It has the following required subelements:
		title
		description
		name
		link"
	^ self tag: 'textInput'!

textInput: aBlock
	self textInput with: aBlock!

timeToLive
	"It's a number of minutes that indicates how long a channel can be cached before refreshing from the source. This makes it possible for RSS sources to be managed by a file-sharing network such as Gnutella.
	
	e.g.: 60"
	^ self tag: 'ttl'!

timeToLive: aBlock
	self timeToLive with: aBlock!

title
	"----- as required subelement of channel ----
	The name of the channel. It's how people refer to your service. If you have an HTML website that contains the same information as your RSS file, the title of your channel should be the same as the title of your website.
	e.g.: 'GoUpstate.com News Headlines'
	
	---- as required subelement of item ----
	The title of the item.
	
	---- as required subelement of image ----
	describes the image, it's used in the ALT attribute of the HTML <img> tag when the channel is rendered in HTML
	
	---- as required subelement of textInput ----
	The label of the Submit button in the text input area."
	
	^ self tag: 'title'!

title: aBlock
	self title with: aBlock!

url
	"is the URL of a GIF, JPEG or PNG image that represents the channel."
	^ self tag: 'url'!

url: aBlock
	self url with: aBlock!

webMaster
	"Email address for person responsible for technical issues relating to channel.
	e.g.: 'Betty Guernsey <betty@herald.com>'"
	^ self tag: 'webMaster'!

webMaster: aBlock
	self webMaster with: aBlock!

width
	"Optional elements include <width> and <height>, numbers, indicating the width and height of the image in pixels.
	
	Maximum value for width is 144, default value is 88."
	^ self tag: 'width'!

width: aBlock
	self width with: aBlock!

withLineBreaks: aString
	aString isEmptyOrNil ifTrue: [ ^self ].
	self renderHtml: [ :html |
		html withLineBreaks: aString ]! !
!RRRssRenderCanvas categoriesFor: #author!item-optional!public! !
!RRRssRenderCanvas categoriesFor: #author:!item-optional!public! !
!RRRssRenderCanvas categoriesFor: #category!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #category:!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #cloud!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #cloud:!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #comment!public!well-formed-web! !
!RRRssRenderCanvas categoriesFor: #comment:!private! !
!RRRssRenderCanvas categoriesFor: #commentRss!public!well-formed-web! !
!RRRssRenderCanvas categoriesFor: #commentRss:!public!well-formed-web! !
!RRRssRenderCanvas categoriesFor: #comments!item-optional!public! !
!RRRssRenderCanvas categoriesFor: #comments:!item-optional!public! !
!RRRssRenderCanvas categoriesFor: #contentEncoded!content!public! !
!RRRssRenderCanvas categoriesFor: #contentEncoded:!content!public! !
!RRRssRenderCanvas categoriesFor: #copyright!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #copyright:!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #creator!dublin core!public! !
!RRRssRenderCanvas categoriesFor: #creator:!dublin core!public! !
!RRRssRenderCanvas categoriesFor: #day!public!skipdays! !
!RRRssRenderCanvas categoriesFor: #day:!public!skipdays! !
!RRRssRenderCanvas categoriesFor: #defaultDocumentation!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #defaultGenerator!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #defaultGeneratorString!private! !
!RRRssRenderCanvas categoriesFor: #description!channel-required!public! !
!RRRssRenderCanvas categoriesFor: #description:!channel-required!public! !
!RRRssRenderCanvas categoriesFor: #documentation!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #documentation:!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #enclosure!item-optional!public! !
!RRRssRenderCanvas categoriesFor: #enclosure:!item-optional!public! !
!RRRssRenderCanvas categoriesFor: #generator!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #generator:!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #guid!item-optional!public! !
!RRRssRenderCanvas categoriesFor: #guid:!item-optional!public! !
!RRRssRenderCanvas categoriesFor: #height!image-optional!public! !
!RRRssRenderCanvas categoriesFor: #height:!image-optional!public! !
!RRRssRenderCanvas categoriesFor: #hour!public!skiphours! !
!RRRssRenderCanvas categoriesFor: #hour:!public!skiphours! !
!RRRssRenderCanvas categoriesFor: #html:!private! !
!RRRssRenderCanvas categoriesFor: #image!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #image:!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #item!item!public! !
!RRRssRenderCanvas categoriesFor: #item:!item!public! !
!RRRssRenderCanvas categoriesFor: #language!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #language:!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #lastBuildDate!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #lastBuildDate:!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #link!channel-required!public! !
!RRRssRenderCanvas categoriesFor: #link:!channel-required!public! !
!RRRssRenderCanvas categoriesFor: #managingEditor!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #managingEditor:!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #name!public!textinput-required! !
!RRRssRenderCanvas categoriesFor: #name:!public!textinput-required! !
!RRRssRenderCanvas categoriesFor: #publicationDate!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #publicationDate:!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #rating!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #rating:!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #renderContentEncoded:!convenience!public! !
!RRRssRenderCanvas categoriesFor: #renderHtml:!convenience!public! !
!RRRssRenderCanvas categoriesFor: #skipDays!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #skipDays:!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #skipHours!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #skipHours:!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #source!item-optional!public! !
!RRRssRenderCanvas categoriesFor: #source:!item-optional!public! !
!RRRssRenderCanvas categoriesFor: #space!convenience!public! !
!RRRssRenderCanvas categoriesFor: #textInput!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #textInput:!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #timeToLive!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #timeToLive:!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #title!channel-required!public! !
!RRRssRenderCanvas categoriesFor: #title:!channel-required!public! !
!RRRssRenderCanvas categoriesFor: #url!image-required!public! !
!RRRssRenderCanvas categoriesFor: #url:!image-required!public! !
!RRRssRenderCanvas categoriesFor: #webMaster!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #webMaster:!channel-optional!public! !
!RRRssRenderCanvas categoriesFor: #width!image-optional!public! !
!RRRssRenderCanvas categoriesFor: #width:!image-optional!public! !
!RRRssRenderCanvas categoriesFor: #withLineBreaks:!convenience!public! !

!RRRssRenderCanvas class methodsFor!

builder
	^ super builder
		documentClass: WAXmlDocument;
		rootClass: RRRssRoot;
		yourself!

generateBinaryFor: aCollection
	(aCollection size = 4 and: [(self includesSelector: aCollection third) not]) ifTrue: [
		self 
			compile: aCollection third, ' aBlock
	self ', aCollection second, ' with: aBlock'
			classified: aCollection fourth]!

tags
	"
	self generateTags
	"

	^ #(
		('item' item item: 'item')
	
		('title' title title: 'channel-required')
		('link' link link: 'channel-required')
		('description' description description: 'channel-required')

		
		('language' language language: 'channel-optional')
		('copyright' copyright copyright: 'channel-optional')
		('managingEditor' managingEditor managingEditor: 'channel-optional')
		('webMaster' webMaster webMaster: 'channel-optional')
		('pubDate' publicationDate publicationDate: 'channel-optional')
		('lastBuildDate' lastBuildDate lastBuildDate: 'channel-optional')
		('category' category category: 'channel-optional')
		('generator' generator generator: 'channel-optional')
		('docs' documentation documentation: 'channel-optional')
		('cloud' cloud cloud: 'channel-optional')
		('ttl' timeToLive timeToLive: 'channel-optional')
		('image' image image: 'channel-optional')
		('rating' rating rating: 'channel-optional')
		('textInput' textInput textInput: 'channel-optional')
		('skipHours' skipHours skipHours: 'channel-optional')
		('skipDays' skipDays skipDays: 'channel-optional')
		
		('author' author author: 'item-optional')
		('comments' comments comments: 'item-optional')
		('enclosure' enclosure enclosure: 'item-optional')
		('guid' guid guid: 'item-optional')
		('source' source source: 'item-optional')
		('language' language language: 'item-optional')
		
		('width' width width: 'image-optional')
		('height' height height: 'image-optional')
		
		('name' name name: 'textinput-required')
		
		('hour' hour hour: 'skipHours')
		('day' day day: 'skipDays')
		
		('url' url url: 'image')
		)! !
!RRRssRenderCanvas class categoriesFor: #builder!instance creation!public! !
!RRRssRenderCanvas class categoriesFor: #generateBinaryFor:!generating-tags!public! !
!RRRssRenderCanvas class categoriesFor: #tags!accessing-tags!public! !

RRRssHandler guid: (GUID fromString: '{07562507-0F28-487C-820E-60E91B1FFB16}')!
RRRssHandler comment: '
A RRRssHandler is an entry point that generates an RSS feed.
'!
!RRRssHandler categoriesForClass!Unclassified! !
!RRRssHandler methodsFor!

createDocumentRoot
	^RRRssRoot new
		charSet: self requestContext charSet;
		yourself!

createRootComponent
	^self rootComponentClass new!

defaultConfiguration
	^ super defaultConfiguration
		addParent: RRSystemConfiguration instance;
		yourself!

handleFiltered: aRequestContext
	| document context canvas root rootComponent |
	self setContentType: aRequestContext response.
	document := aRequestContext newDocument.
	root := self createDocumentRoot.
	context := WARenderContext new document: document.
	[ canvas := RRRssRenderCanvas context: context.
	rootComponent := self createRootComponent.
	
	rootComponent updateRoot: root.
	document open: root.
	canvas render: rootComponent.
		
	document close ] ensure: [ context destroy ].
	aRequestContext respond!

initialize
	super initialize.
	self addFilter: WAExceptionFilter new!

rootComponentClass
	^ self preferenceAt: #rootComponentClass!

rootComponentClass: aClass
	^ self preferenceAt: #rootComponentClass put: aClass!

setContentType: aResponse
	| contentType |
	contentType := WAMimeType applicationRssXml
		charset: self requestContext charSet;
		yourself.
	aResponse contentType: contentType.! !
!RRRssHandler categoriesFor: #createDocumentRoot!private! !
!RRRssHandler categoriesFor: #createRootComponent!private! !
!RRRssHandler categoriesFor: #defaultConfiguration!configuration!public! !
!RRRssHandler categoriesFor: #handleFiltered:!handling!public! !
!RRRssHandler categoriesFor: #initialize!initialization!public! !
!RRRssHandler categoriesFor: #rootComponentClass!configuration!public! !
!RRRssHandler categoriesFor: #rootComponentClass:!configuration!public! !
!RRRssHandler categoriesFor: #setContentType:!private! !

!RRRssHandler class methodsFor!

description
	^'RSS feed'! !
!RRRssHandler class categoriesFor: #description!accessing!public! !

RRRssRoot guid: (GUID fromString: '{B9A28915-ED30-447D-B8AF-8F850A3CEC39}')!
RRRssRoot comment: ''!
!RRRssRoot categoriesForClass!Unclassified! !
!RRRssRoot methodsFor!

channelAttributes
	^channelAttributes!

closeOn: aDocument
	self writeFootOn: aDocument.
	super closeOn: aDocument.!

initialize
	
	super initialize.
	rssAttributes := WAHtmlAttributes new
		at: 'version' put: '2.0';
		at: 'xmlns:content' put: 'http://purl.org/rss/1.0/modules/content/';
		at: 'xmlns:wfw' put: 'http://wellformedweb.org/CommentAPI/';
		at: 'xmlns:dc' put: 'http://purl.org/dc/elements/1.1/';
		yourself.
	channelAttributes := WAHtmlAttributes new.!

openOn: aDocument
	super openOn: aDocument.
	self writeHeadOn: aDocument!

rssAttributes
	^rssAttributes!

styles
	styles isNil ifTrue: [
		styles := OrderedCollection new ].
	^styles!

stylesheet
	"http://interglacial.com/~sburke/stuff/pretty_rss.html"
	^ self styles add: (RRXmlStyleSheet root: self)!

writeFootOn: aDocument
	aDocument closeTag: 'channel'.
	aDocument closeTag: 'rss'!

writeHeadOn: aDocument
	self writeStylesOn: aDocument.
	
	aDocument openTag: 'rss' attributes: rssAttributes.
	aDocument openTag: 'channel' attributes: channelAttributes.!

writeStylesOn: aDocument
	styles isNil ifTrue: [ ^self ].

	styles do: [ :each |
		aDocument print: each ]! !
!RRRssRoot categoriesFor: #channelAttributes!accessing!public! !
!RRRssRoot categoriesFor: #closeOn:!public!writing! !
!RRRssRoot categoriesFor: #initialize!initialization!public! !
!RRRssRoot categoriesFor: #openOn:!public!writing! !
!RRRssRoot categoriesFor: #rssAttributes!accessing!public! !
!RRRssRoot categoriesFor: #styles!accessing!public! !
!RRRssRoot categoriesFor: #stylesheet!elements!public! !
!RRRssRoot categoriesFor: #writeFootOn:!public!writing! !
!RRRssRoot categoriesFor: #writeHeadOn:!public!writing! !
!RRRssRoot categoriesFor: #writeStylesOn:!public!writing! !

"Binary Globals"!

