| package |
package := Package name: 'RSS-Tests-Core'.
package paxVersion: 1;
	basicComment: ''.

package basicPackageVersion: '26'.


package classNames
	add: #RRComponentRenderingTest;
	add: #RRRenderTest;
	add: #RRRfc822DateTimeTest;
	add: #RRTestComponent;
	yourself.

package methodNames
	add: 'GRPackage class' -> #rssTestsCore;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Grease\Grease-Core';
	add: '..\..\IDB\IDB DateAndTime';
	add: '..\..\IDB\IDB Duration';
	add: 'RSS-Core';
	add: '..\Seaside\Seaside-Core';
	add: '..\Seaside\Seaside-Tests-Core';
	add: '..\..\Camp Smalltalk\SUnit\SUnit';
	yourself).

package!

"Class Definitions"!

RRComponent subclass: #RRTestComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TestCase subclass: #RRRenderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TestCase subclass: #RRRfc822DateTimeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAContextTest subclass: #RRComponentRenderingTest
	instanceVariableNames: 'handler'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!GRPackage class methodsFor!

rssTestsCore
	^ self new
		name: 'RSS-Tests-Core';
		description: 'Unit tests for the "RSS-Core" package.';
		addDependency: 'RSS-Core';
		addDependency: 'Seaside-Tests-Core';
		url: #seasideUrl;
		yourself! !
!GRPackage class categoriesFor: #rssTestsCore!*rss-tests-core!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

RRTestComponent guid: (GUID fromString: '{BA1A3DDD-FC2A-470E-BAC8-0B020E908644}')!
RRTestComponent comment: ''!
!RRTestComponent categoriesForClass!Unclassified! !
!RRTestComponent methodsFor!

renderContentOn: rss
	"do nothing"!

updateRoot: anRssRoot
	super updateRoot: anRssRoot.
	anRssRoot stylesheet beXsl
		url: 'http://www.seaside.st/feed.xsl'! !
!RRTestComponent categoriesFor: #renderContentOn:!public!rendering! !
!RRTestComponent categoriesFor: #updateRoot:!public!updating! !

RRRenderTest guid: (GUID fromString: '{9BD68FB5-C050-4F6F-B349-8819DE057605}')!
RRRenderTest comment: ''!
!RRRenderTest categoriesForClass!Unclassified! !
!RRRenderTest methodsFor!

performTest
	| context |
	context := WARequestContext 
		request: nil
		response: nil
		codec: GRNullCodec new.
	WACurrentRequestContext 
		use: context
		during: [ super performTest ]!

render: aBlock
	^ RRRssRenderCanvas builder render: aBlock!

renderFull: aBlock
	^self renderFull: aBlock rootBlock: nil!

renderFull: aBlock rootBlock: anObject
	^RRRssRenderCanvas builder
		fullDocument: true;
		rootBlock: anObject;
		render: aBlock!

testEmail
	| actual expected |
	actual := self render: [ :rss |
		rss author: (WAEmailAddress address: 'joesmith@example.com' username: 'Joe Smith') ].
	expected := '<author>Joe Smith &lt;joesmith@example.com&gt;</author>'.
	self assert: actual = expected!

testExample
	| actualContent expectedContent |
	actualContent := self render: [ :rss |
		rss description: 'Das super geile Blog.'.
		rss contentEncoded: '<p>Das super geile Blog.</p>'.
		rss generator: 'Pier 1.0.9-alpha'.
		rss language english ].
	expectedContent := '<description>Das super geile Blog.</description><content:encoded><!![CDATA[<p>Das super geile Blog.</p>]]></content:encoded><generator>Pier 1.0.9-alpha</generator><language>en</language>'.
	self assert: actualContent = expectedContent!

testRenderExample
	| actualContent expectedContent |
	actualContent := self render: [ :rss |
		rss description: 'Das super geile Blog.'.
		rss renderContentEncoded: [ :html |
			html paragraph:  'Das super geile Blog.' ].
		rss generator: 'Pier 1.0.9-alpha'.
		rss language english ].
	expectedContent := '<description>Das super geile Blog.</description><content:encoded>&lt;p&gt;Das super geile Blog.&lt;/p&gt;</content:encoded><generator>Pier 1.0.9-alpha</generator><language>en</language>'.
	self assert: actualContent = expectedContent!

testRenderExampleFull
	| actualContent expectedContent |
	actualContent := self
		renderFull: [ :rss |
			rss description: 'Das super geile Blog.'.
			rss renderContentEncoded: [ :html |
				html paragraph:  'Das super geile Blog.' ].
			rss generator: 'Pier 1.0.9-alpha'.
			rss language english ]
		rootBlock: [ :root |
			root stylesheet beCss; url: 'http://yourserver.com/path/rss.css' ].
	
	expectedContent := '<?xml version="1.0" encoding="UTF-8"?><?xml-stylesheet type="text/css" href="http://yourserver.com/path/rss.css"?><rss version="2.0" xmlns:content="http://purl.org/rss/1.0/modules/content/" xmlns:wfw="http://wellformedweb.org/CommentAPI/" xmlns:dc="http://purl.org/dc/elements/1.1/"><channel><description>Das super geile Blog.</description><content:encoded>&lt;p&gt;Das super geile Blog.&lt;/p&gt;</content:encoded><generator>Pier 1.0.9-alpha</generator><language>en</language></channel></rss>'.
	self assert: actualContent = expectedContent!

testWithLineBreaks
	| actualContent expectedContent |
	actualContent := self render: [ :rss |
		rss withLineBreaks: 'das', GRPlatform current newline, 'super', GRPlatform current newline, 'geile', GRPlatform current newline, 'Blog' ].
	expectedContent := 'das&lt;br/&gt;super&lt;br/&gt;geile&lt;br/&gt;Blog'.
	self assert: actualContent = expectedContent! !
!RRRenderTest categoriesFor: #performTest!public!running! !
!RRRenderTest categoriesFor: #render:!private! !
!RRRenderTest categoriesFor: #renderFull:!private! !
!RRRenderTest categoriesFor: #renderFull:rootBlock:!private! !
!RRRenderTest categoriesFor: #testEmail!public!tests! !
!RRRenderTest categoriesFor: #testExample!public!tests! !
!RRRenderTest categoriesFor: #testRenderExample!public!tests! !
!RRRenderTest categoriesFor: #testRenderExampleFull!public!tests! !
!RRRenderTest categoriesFor: #testWithLineBreaks!public!tests! !

RRRfc822DateTimeTest guid: (GUID fromString: '{818A29B1-F310-4B14-94FE-971AEDE9F36F}')!
RRRfc822DateTimeTest comment: ''!
!RRRfc822DateTimeTest categoriesForClass!Unclassified! !
!RRRfc822DateTimeTest methodsFor!

render: anObject
	^ RRRssRenderCanvas builder render: anObject!

testNewDateAndTime
	| date emailDate |
	date := DateAndTime year: 2008 month: 8 day: 1 hour: 9 minute: 4 second: 4 offset: (Duration days: 0 hours: 2 minutes: 0 seconds: 0).
	emailDate := RRRfc822DateTime newDateAndTime: date.
	self assert: 'Fri, 01 Aug 2008 09:04:04 +0200' = (self render: emailDate)!

testNewDateAndTimeTimezoneString
	| date emailDate |
	date := DateAndTime year: 2008 month: 8 day: 31 hour: 19 minute: 14 second: 24 offset: Duration zero.
	emailDate := RRRfc822DateTime
		newDateAndTime: date
		offset: (Duration days: 0 hours: -2 minutes: 0 seconds: 0).
	self assert: 'Sun, 31 Aug 2008 19:14:24 -0200' = (self render: emailDate)!

testNewDateAndTimeUtc
	| date emailDate |
	date := DateAndTime year: 2008 month: 8 day: 31 hour: 19 minute: 14 second: 24 offset: (Duration days: 0 hours: 2 minutes: 0 seconds: 0).
	emailDate := RRRfc822DateTime newUtc: date.
	self assert: 'Sun, 31 Aug 2008 17:14:24 UT' = (self render: emailDate)!

testOffsetString
	| duration formatted |
	duration := Duration days: 0 hours: 2 minutes: 3 seconds: 0.
	formatted := RRRfc822DateTime offsetString: duration.
	self assert: '+0203' = formatted.
	
	duration := Duration days: 0 hours: -2 minutes: 0 seconds: 0.
	formatted := RRRfc822DateTime offsetString: duration.
	self assert: '-0200' = formatted.
	
	duration := Duration days: 0 hours: 0 minutes: -3 seconds: 0.
	formatted := RRRfc822DateTime offsetString: duration.
	self assert: '-0003' = formatted.
	
	"Test that 0 offset is positive and #offsetString: sends only ANSI messages."
	duration:= Duration zero.
	formatted := RRRfc822DateTime offsetString: duration.
	self assert: '+0000' = formatted! !
!RRRfc822DateTimeTest categoriesFor: #render:!private! !
!RRRfc822DateTimeTest categoriesFor: #testNewDateAndTime!public!tests! !
!RRRfc822DateTimeTest categoriesFor: #testNewDateAndTimeTimezoneString!public!tests! !
!RRRfc822DateTimeTest categoriesFor: #testNewDateAndTimeUtc!public!tests! !
!RRRfc822DateTimeTest categoriesFor: #testOffsetString!public!tests! !

RRComponentRenderingTest guid: (GUID fromString: '{6EDEAA16-53D0-45F2-A7EA-47C44E8BD489}')!
RRComponentRenderingTest comment: ''!
!RRComponentRenderingTest categoriesForClass!Unclassified! !
!RRComponentRenderingTest methodsFor!

createHandlers
	^Array with: handler!

setUp
	handler := RRRssHandler new
		rootComponentClass: RRTestComponent;
		yourself!

testUpdateRoot
	 | response |
	[ handler handleFiltered: self requestContext ]
		on: WAResponseNotification
		do: [ :notification | "ignore" ].
	response := self requestContext response.
	self assert: response status = 200.
	self assert: '<?xml version="1.0" encoding="utf-8"?>',
		'<?xml-stylesheet type="text/xsl" href="http://www.seaside.st/feed.xsl"?>',
		'<rss version="2.0" xmlns:content="http://purl.org/rss/1.0/modules/content/" xmlns:wfw="http://wellformedweb.org/CommentAPI/" xmlns:dc="http://purl.org/dc/elements/1.1/">',
		'<channel></channel></rss>' = response contents! !
!RRComponentRenderingTest categoriesFor: #createHandlers!configuration!public! !
!RRComponentRenderingTest categoriesFor: #setUp!public!running! !
!RRComponentRenderingTest categoriesFor: #testUpdateRoot!public!testing! !

"Binary Globals"!

