| package |
package := Package name: 'Seaside-FileSystem'.
package paxVersion: 1;
	basicComment: ''.

package basicPackageVersion: '23'.


package classNames
	add: #WAExternalFileLibrary;
	add: #WAExternalFileLibraryConfiguration;
	yourself.

package methodNames
	add: 'GRPackage class' -> #seasideFileSystem;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\Object Arts\Dolphin\Base\Dolphin';
	add: 'Grease\Grease-Core';
	add: 'Seaside\Seaside-Canvas';
	add: 'Seaside\Seaside-Core';
	add: 'Sport\Sport';
	yourself).

package!

"Class Definitions"!

WASystemConfiguration subclass: #WAExternalFileLibraryConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WARequestHandler subclass: #WAExternalFileLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!GRPackage class methodsFor!

seasideFileSystem
	^ self new
		name: 'Seaside-FileSystem';
		description: 'File library that serves files from external directory.';
		addDependency: 'Seaside-Core';
		url: #seasideLGPLUrl;
		license: #LGPL;
		yourself! !
!GRPackage class categoriesFor: #seasideFileSystem!*seaside-filesystem!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

WAExternalFileLibraryConfiguration guid: (GUID fromString: '{996DC3F1-1D8A-4DC3-BC0F-DB2A4DB2EA89}')!
WAExternalFileLibraryConfiguration comment: ''!
!WAExternalFileLibraryConfiguration categoriesForClass!Unclassified! !
!WAExternalFileLibraryConfiguration methodsFor!

describeOn: config
	(config string: #directory)
		label: 'Directory';
		comment: 'The base directory to serve files from.';
		default: (SpFilename named: 'xxxDoesNotExists') asAbsoluteFilename directory asString. "#asString on SPFilename"
	(config boolean: #listing)
		label: 'Allow Listing';
		comment: 'Enable listing of directory entries.';
		default: false! !
!WAExternalFileLibraryConfiguration categoriesFor: #describeOn:!attributes!public! !

WAExternalFileLibrary guid: (GUID fromString: '{BCFB1D4C-FD42-4CA0-965E-68B4CAD1848F}')!
WAExternalFileLibrary comment: ''!
!WAExternalFileLibrary categoriesForClass!Unclassified! !
!WAExternalFileLibrary methodsFor!

asAbsoluteUrl: aFileName relativeTo: aRequest
	^aRequest url withoutQuery
		addToPath: aFileName;
		yourself!

contentsOfFile: aFilename
	| stream |
	stream := aFilename readStream.
	^[
		stream binary.
		stream upToEnd ]
		ensure: [ stream close ]!

defaultConfiguration
	^ super defaultConfiguration
		addParent: WAExternalFileLibraryConfiguration instance;
		yourself!

directory
	| path filename |
	path := self preferenceAt: #directory.
	filename := SpFilename named: path.
	"hack around SPort bugs"
	^(path last = filename separator)
		ifTrue: [ SpFilename named: path allButLast ]
		ifFalse: [ filename ]!

filenameForConsumer: aPathConsumer
	"Answer the SpFilename pointing to the file for the given request."
	| filename |
	filename := self directory.
	[ aPathConsumer atEnd ] whileFalse: [
		filename := filename construct: aPathConsumer next ].
	^filename!

filesIn: aPath do: aOneArgumentBlock 
	"this can not be done with SPort"
	(GRPlatform current filesIn: aPath) do: [ :each | aOneArgumentBlock value: (GRPlatform current localNameOf: each) ]!

handleFiltered: aRequestContext
	[ self processContext: aRequestContext ] 
		on: Error
		do: [ :error | 
			aRequestContext responseGenerator
				internalError: error;
				respond ]!

listing
	^ self preferenceAt: #listing!

mimetypeForRequest: aRequest
	| path |
	path := aRequest url path.
	^(path isEmpty or: [ (path last includes: $.) not ])
		ifTrue: [ WAFileLibrary defaultMimeType seasideMimeType ]
		ifFalse: [
			| extension |
			extension := path last copyAfterLast: $..
			WAFileLibrary mimetypeFor: extension ]!

processContext: aRequestContext
	| filename |
	filename := self filenameForConsumer: aRequestContext consumer.
	(self listing and: [ filename isDirectory ])
		ifTrue: [ self
				processDirectory: filename asString "#asString on SPFilename"
				context: aRequestContext ]
		ifFalse: [ self processFile: filename context: aRequestContext ]!

processDirectory: aString context: aRequestContext 
	| contents |
	contents := WARenderCanvas builder
		fullDocument: true;
		rootBlock: [ :html | html title: aRequestContext request url greaseString ];
		render: 
			[ :html | 
			html heading
				level: 1;
				with: aRequestContext request url pathString.
			html unorderedList: 
				[ self 
					filesIn: aString
					do: 
						[ :each | 
						html listItem: 
							[ html anchor
								url: (self 
										asAbsoluteUrl: each
										relativeTo: aRequestContext request);
								with: each ] ] ] ].
	aRequestContext respond: 
		[ :response | 
		response
			contentType: WAMimeType textHtml;
			nextPutAll: contents ]!

processFile: aFilename context: aRequestContext
	aRequestContext respond: [ :response |
		response
			document: (self contentsOfFile: aFilename)
			mimeType: (self mimetypeForRequest: aRequestContext request)
			fileName: (aRequestContext request url path last) ]! !
!WAExternalFileLibrary categoriesFor: #asAbsoluteUrl:relativeTo:!private! !
!WAExternalFileLibrary categoriesFor: #contentsOfFile:!private! !
!WAExternalFileLibrary categoriesFor: #defaultConfiguration!configuration!public! !
!WAExternalFileLibrary categoriesFor: #directory!accessing!public! !
!WAExternalFileLibrary categoriesFor: #filenameForConsumer:!private! !
!WAExternalFileLibrary categoriesFor: #filesIn:do:!private! !
!WAExternalFileLibrary categoriesFor: #handleFiltered:!handling!public! !
!WAExternalFileLibrary categoriesFor: #listing!accessing!public! !
!WAExternalFileLibrary categoriesFor: #mimetypeForRequest:!private! !
!WAExternalFileLibrary categoriesFor: #processContext:!handling!public! !
!WAExternalFileLibrary categoriesFor: #processDirectory:context:!handling!public! !
!WAExternalFileLibrary categoriesFor: #processFile:context:!handling!public! !

!WAExternalFileLibrary class methodsFor!

description
	^ 'File Directory'! !
!WAExternalFileLibrary class categoriesFor: #description!accessing!public! !

"Binary Globals"!

