| package |
package := Package name: 'Seaside-Tests-FileSystem'.
package paxVersion: 1;
	basicComment: ''.

package basicPackageVersion: '13'.


package classNames
	add: #WAExternalFileLibraryTest;
	yourself.

package methodNames
	add: 'GRPackage class' -> #seasideTestsFileSystem;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\Object Arts\Dolphin\Base\Dolphin';
	add: 'Grease\Grease-Core';
	add: 'Seaside\Seaside-Core';
	add: 'Seaside-FileSystem';
	add: 'Sport\Sport';
	add: '..\Camp Smalltalk\SUnit\SUnit';
	yourself).

package!

"Class Definitions"!

TestCase subclass: #WAExternalFileLibraryTest
	instanceVariableNames: 'fileSystem'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!GRPackage class methodsFor!

seasideTestsFileSystem
	^ self new
		name: 'Seaside-Tests-FileSystem';
		description: 'Tests for the "Seaside-FileSystem" package.';
		addDependency: 'Seaside-FileSystem';
		url: #seasideLGPLUrl;
		license: #LGPL;
		yourself! !
!GRPackage class categoriesFor: #seasideTestsFileSystem!*seaside-tests-filesystem!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

WAExternalFileLibraryTest guid: (GUID fromString: '{52D6B1C8-248F-433C-8BCF-36BFA20F5DCC}')!
WAExternalFileLibraryTest comment: ''!
!WAExternalFileLibraryTest categoriesForClass!Unclassified! !
!WAExternalFileLibraryTest methodsFor!

crossPlatformPathFromArray: pathArray finalSlash: finalSlash
	| separator path|
	separator := String with: SpFilename new separator.
	
	path := String new.
	pathArray do: [:pathElement | 
		path := path, separator, pathElement ].

	finalSlash ifTrue: [ path:= path, separator]. 
	
	^ path
!

requestWithUrl: aString
	^ WARequest method: 'POST' uri: aString!

setUp
	super setUp.
	fileSystem := WAExternalFileLibrary new.
	WADispatcher new register: fileSystem at: 'culture'!

testAsAbsoluteUrlRelativeTo
	| request |
	fileSystem configuration 
		valueAt: #directory
		put: '/home/philppe/pr0n'.
	request := self requestWithUrl: '/culture/pictures'.
	self assert: (fileSystem 
			asAbsoluteUrl: 'cleopatra.jpeg'
			relativeTo: request) greaseString = '/culture/pictures/cleopatra.jpeg'.
	request := self requestWithUrl: '/culture/pictures/'.
	self assert: (fileSystem 
			asAbsoluteUrl: 'cleopatra.jpeg'
			relativeTo: request) greaseString = '/culture/pictures/cleopatra.jpeg'!

testMimetypeForRequest
	| request |
	request := self requestWithUrl: '/culture/pictures/cleopatra.jpeg'.
	self assert: (fileSystem mimetypeForRequest: request) = 'image/jpeg' seasideMimeType.
	
	request := self requestWithUrl: '/culture/pictures/cleopatra'.
	self assert: (fileSystem mimetypeForRequest: request) = 'application/octet-stream' seasideMimeType.
	
	request := self requestWithUrl: '/culture/pictures/cleopatra.'.
	self assert: (fileSystem mimetypeForRequest: request) = 'application/octet-stream' seasideMimeType.
	
	request := self requestWithUrl: '/'.
	self assert: (fileSystem mimetypeForRequest: request) = 'application/octet-stream' seasideMimeType.
	
	request := self requestWithUrl: '/culture/pictures/cleopatra.js'.
	self assert: (fileSystem mimetypeForRequest: request) = 'application/x-javascript' seasideMimeType.!

testPathForRequest
	"self debug: #testPathForRequest"

	| consumer requestPath resultPath filename |

	"/home/philppe/files"
	requestPath := self crossPlatformPathFromArray: #('home' 'philppe' 'files') finalSlash: false.
	fileSystem configuration valueAt: #directory put: requestPath.
	"/culture is mapped to /home/philppe/files"
	
	"If you're looking this test and it doesn't seem to make sense, keep in mind that a file name is this context is an SpFilename, which is a path."
	consumer := WAPathConsumer path: #('cleopatra.jpeg') asOrderedCollection.
	"/home/philppe/files/cleopatra.jpeg"
	resultPath := self crossPlatformPathFromArray: #('home' 'philppe' 'files' 'cleopatra.jpeg') finalSlash: false.
	filename := fileSystem filenameForConsumer: consumer.
	self assert: filename asString "on SpFileName" = resultPath.
	

	consumer := WAPathConsumer path: #('culture' 'cleopatra.jpeg') asOrderedCollection.
	"/home/philppe/files/"
	requestPath := self crossPlatformPathFromArray: #('home' 'philppe' 'files') finalSlash: true.
	fileSystem configuration valueAt: #directory put: requestPath.
	
	consumer := WAPathConsumer path: #('cleopatra.jpeg') asOrderedCollection.
	"/home/philppe/files/cleopatra.jpeg"
	resultPath := self crossPlatformPathFromArray: #('home' 'philppe' 'files' 'cleopatra.jpeg') finalSlash: false.
	filename := fileSystem filenameForConsumer: consumer.
	self assert: filename greaseString = resultPath.
	
	
	consumer := WAPathConsumer path: #('cleopatra.jpeg') asOrderedCollection.
	"/home/philppe/files"
	requestPath := self crossPlatformPathFromArray: #('home' 'philppe' 'files') finalSlash: false.
	fileSystem configuration valueAt: #directory put: requestPath.
	
	"/home/philppe/files/cleopatra.jpeg"
	resultPath := self crossPlatformPathFromArray: #('home' 'philppe' 'files' 'cleopatra.jpeg') finalSlash: false.
	self assert: (fileSystem filenameForConsumer: consumer) greaseString = resultPath.
	
	
	consumer := WAPathConsumer path: #('cleopatra.jpeg') asOrderedCollection.
	"/home/philppe/files/"
	requestPath := self crossPlatformPathFromArray: #('home' 'philppe' 'files') finalSlash: true.
	fileSystem configuration valueAt: #directory put: requestPath.
	
	consumer := WAPathConsumer path: #('cleopatra.jpeg') asOrderedCollection.
	"/home/philppe/files/cleopatra.jpeg"
	resultPath := self crossPlatformPathFromArray: #('home' 'philppe' 'files' 'cleopatra.jpeg') finalSlash: false.
	self assert: (fileSystem filenameForConsumer: consumer) greaseString = resultPath.! !
!WAExternalFileLibraryTest categoriesFor: #crossPlatformPathFromArray:finalSlash:!public!testing! !
!WAExternalFileLibraryTest categoriesFor: #requestWithUrl:!private! !
!WAExternalFileLibraryTest categoriesFor: #setUp!public!running! !
!WAExternalFileLibraryTest categoriesFor: #testAsAbsoluteUrlRelativeTo!public!testing! !
!WAExternalFileLibraryTest categoriesFor: #testMimetypeForRequest!public!testing! !
!WAExternalFileLibraryTest categoriesFor: #testPathForRequest!public!testing! !

"Binary Globals"!

