| package |
package := Package name: 'Seaside-Dolphin-Development'.
package paxVersion: 1;
	basicComment: ''.

package basicPackageVersion: '2'.


package classNames
	add: #WABrowser;
	add: #WABrowserHaloPlugin;
	add: #WADolphinInspector;
	add: #WADolphinWalkback;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\IDE\Base\Development System';
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Grease\Grease-Core';
	add: '..\Grease\Grease-Dolphin-Core';
	add: 'Seaside-Development';
	add: 'Seaside-Dolphin-Browser';
	add: 'Seaside-Tools-Core';
	yourself).

package!

"Class Definitions"!

WAHaloPlugin subclass: #WABrowserHaloPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WATool subclass: #WABrowser
	instanceVariableNames: 'contents model message findClassName'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAInspector subclass: #WADolphinInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAWalkback subclass: #WADolphinWalkback
	instanceVariableNames: 'frames'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

"End of package definition"!

"Source Globals"!

"Classes"!

WABrowserHaloPlugin guid: (GUID fromString: '{A3BD6C92-BE3A-4297-91BB-98B737DD00E7}')!
WABrowserHaloPlugin comment: 'I open a browser on the component class.'!
!WABrowserHaloPlugin categoriesForClass!Unclassified! !
!WABrowserHaloPlugin methodsFor!

execute
	self open: (WABrowser fullOnClass: self root class)!

label
	^ 'Browser'!

priority
	^ 500!

tooltip
	^ 'Class Browser'! !
!WABrowserHaloPlugin categoriesFor: #execute!actions!public! !
!WABrowserHaloPlugin categoriesFor: #label!accessing!public! !
!WABrowserHaloPlugin categoriesFor: #priority!accessing!public! !
!WABrowserHaloPlugin categoriesFor: #tooltip!accessing!public! !

WABrowser guid: (GUID fromString: '{D4609DD8-C5EB-426B-A1AA-DF32888C272D}')!
WABrowser comment: ''!
!WABrowser categoriesForClass!Unclassified! !
!WABrowser methodsFor!

accept
	| save |
	message := nil.
	save := contents.
	model contents: contents notifying: self.
	contents := save!

columns
	^ #( systemCategory class messageCategory message )!

contents
	^contents!

contents: aString 
	"the browser sends CRLF as line ending but Squeak wants CR"
	contents := GRPlatform current convertToSmalltalkNewlines: aString!

findClass
	self model setClass: (Smalltalk at: self findClassName asSymbol) selector: nil!

findClassName
	findClassName ifNil: [findClassName := ''].
	^findClassName!

findClassName: anObject
	findClassName := anObject!

initialize
	super initialize.
	self model: Browser new!

initialRequest: aRequest
	| theClass |
	super initialRequest: aRequest.
	aRequest fields at: 'class' ifPresent: [ :className |
		Smalltalk at: className asSymbol ifPresent: [ :class |
			theClass := (aRequest fields includesKey: 'meta')
				ifTrue: [ class class ]
				ifFalse: [ class ].
			self model setClass: theClass selector: nil.
			aRequest fields at: 'method' ifPresent: [ :selector |
				self model setClass: theClass selector: selector asSymbol ] ] ]!

model
	^model!

model: aBrowserModel
	model notNil ifTrue: [ model removeDependent: self ].
	aBrowserModel notNil ifTrue: [ aBrowserModel addDependent: self ].
	model := aBrowserModel.
	contents := model contents!

notify: aString at: location in: sourceStream
	message := aString allButLast: 3!

renderChooserOn: html 
	html form with: 
			[(html label)
				for: html nextId;
				with: 'Find class:'.
			(html textInput)
				id: html lastId;
				value: self findClassName;
				callback: [:value | self findClassName: value trimBlanks];
				script: (html jQuery this autocomplete 
							sourceCallback: [:term | self model classNamesFor: term asString trimBlanks]);
				onChange: 'submit()'.
			html submitButton
				callback: [ self findClass ];
				value: 'Find class' ]!

renderColumnsOn: html
	html div class: 'columns'; with: [
		self columns do: [ :each |
			html form class: 'column'; with: [
				| array |
				array := self model perform: (each , 'List') asSymbol.
				html div: [
					html select
						size: 10;
						list: array;
						onChange: 'submit()';
						selected: (array
							at: (self model perform: (each , 'ListIndex') asSymbol)
							ifAbsent: [ nil ]);
						callback: [ :item |
							self model
								perform: (each , 'ListIndex:') asSymbol
								with: (array indexOf: item) ] ] ] ] ] !

renderContentOn: html
	self
		renderChooserOn: html;
		renderColumnsOn: html;
		renderModeButtonsOn: html;
		renderMessageOn: html;
		renderContentPaneOn: html!

renderContentPaneOn: html
	html form class: 'definition'; with: [
	 	html div: [
			html textArea on: #contents of: self.
			html break.
			html submitButton on: #accept of: self ] ]!

renderMessageOn: html
	message isNil
		ifFalse: [ html strong: message ]!

renderModeButtonsOn: html
	html form class: 'mode'; with: [
		html div: [
			html submitButton
				callback: [ self showInstance ];
				value: 'instance'.
			html submitButton
				callback: [ self showHelp ];
				value: 'help'.
			html submitButton
				callback: [ self showClass ];
				value: 'class' ] ]!

select!

selectFrom: aPos to: anotherPos!

selectionInterval
	^ 1 to: 1!

showClass
	self model indicateClassMessages!

showHelp
	self model plusButtonHit!

showInstance
	self model indicateInstanceMessages!

states
	^ Array with: model!

style
	^ 'form, div.columns {
	padding: 2px;
	margin: 0;
}

form.column {
	padding: 0;
	width: 25%;
	float: left;
}
form.column select {
	width: 100%;
	border: 1px solid #aaa;
}

form.mode {
	clear: both;
}

form.definition textarea {
	width: 100%;
	height: 200px;
	border: 1px solid #aaa;
}'!

text
	^ Text fromString: ''!

update: aSymbol
	contents := model contents!

updateRoot: anHtmlRoot
	"This method allows you customize the <head> section of an HTML document. The API is very similar to the 'Canvas API' for rendering in the <body> section (where you 'draw' in #renderContentOn:).

	anHtmlRoot is an instance of WAHtmlRoot

	Whenever you override this method don't forget to send super"
	"super updateRoot: anHtmlRoot." "Removed intentinally, maybe is a good idea to have a basicUpdateRoot:"

	self script isNil ifFalse: [ anHtmlRoot addScript: self script ].
	self style isNil ifFalse: [ anHtmlRoot addStyle: self style ]!

updateUrl: aUrl
	super updateUrl: aUrl.
	self model selectedClass isNil ifFalse: [
		aUrl addField: 'class' value: self model selectedClass fullName.
		self model metaClassIndicated
			ifTrue: [ aUrl addField: 'meta' ] ].
	self model selectedMessageName isNil 
		ifFalse: [ aUrl addField: 'method' value: self model selectedMessageName ]! !
!WABrowser categoriesFor: #accept!actions!public! !
!WABrowser categoriesFor: #columns!accessing!public! !
!WABrowser categoriesFor: #contents!accessing!public! !
!WABrowser categoriesFor: #contents:!accessing!public! !
!WABrowser categoriesFor: #findClass!public! !
!WABrowser categoriesFor: #findClassName!accessing!private! !
!WABrowser categoriesFor: #findClassName:!accessing!private! !
!WABrowser categoriesFor: #initialize!initialization!public! !
!WABrowser categoriesFor: #initialRequest:!processing!public! !
!WABrowser categoriesFor: #model!accessing!public! !
!WABrowser categoriesFor: #model:!accessing!public! !
!WABrowser categoriesFor: #notify:at:in:!notifying!public! !
!WABrowser categoriesFor: #renderChooserOn:!public!rendering! !
!WABrowser categoriesFor: #renderColumnsOn:!public!rendering! !
!WABrowser categoriesFor: #renderContentOn:!public!rendering! !
!WABrowser categoriesFor: #renderContentPaneOn:!public!rendering! !
!WABrowser categoriesFor: #renderMessageOn:!public!rendering! !
!WABrowser categoriesFor: #renderModeButtonsOn:!public!rendering! !
!WABrowser categoriesFor: #select!notifying!public! !
!WABrowser categoriesFor: #selectFrom:to:!actions!public! !
!WABrowser categoriesFor: #selectionInterval!accessing!public! !
!WABrowser categoriesFor: #showClass!actions!public! !
!WABrowser categoriesFor: #showHelp!actions!public! !
!WABrowser categoriesFor: #showInstance!actions!public! !
!WABrowser categoriesFor: #states!accessing!public! !
!WABrowser categoriesFor: #style!public!rendering! !
!WABrowser categoriesFor: #text!accessing!public! !
!WABrowser categoriesFor: #update:!notifying!public! !
!WABrowser categoriesFor: #updateRoot:!public! !
!WABrowser categoriesFor: #updateUrl:!processing!public! !

!WABrowser class methodsFor!

applicationName
	"needed for RSRSS"
	^'tools/classbrowser'!

canBeRoot
	^ true!

description
	^ 'Smalltalk Code Browser'!

fullOnClass: aClass
	^ self fullOnClass: aClass selector: nil!

fullOnClass: aClass selector: aSelector
	^ self new model: (Browser new setClass: aClass selector: aSelector)!

initialize
	WAAdmin register: self asApplicationAt: self applicationName! !
!WABrowser class categoriesFor: #applicationName!accessing!public! !
!WABrowser class categoriesFor: #canBeRoot!accessing!public! !
!WABrowser class categoriesFor: #description!accessing!public! !
!WABrowser class categoriesFor: #fullOnClass:!instance creation!public! !
!WABrowser class categoriesFor: #fullOnClass:selector:!instance creation!public! !
!WABrowser class categoriesFor: #initialize!initialization!public! !

WADolphinInspector guid: (GUID fromString: '{CD12D45E-F5B3-4A16-ADC3-31B59B7FEB08}')!
WADolphinInspector comment: 'Previous implementation of WASqueakInspector'!
!WADolphinInspector categoriesForClass!Unclassified! !
!WADolphinInspector methodsFor!

doIt
	| result |
	result := super doIt.
	selected ifNil: [ ^ result ].
	^ selected isNumber
		ifTrue: [ self object at: selected put: result ]
		ifFalse: [ self object instVarAt: (self object class indexOfInstVar: selected) put: result ]!

evaluate
	| result |
	result := self object basicClass compilerClass 
		compileForEvaluation: input asString
				in: self object basicClass
				evaluationPools: #()
				logged: true
				flags: 512. "Interactive"
	^result ifNotNil: [result method value: self object]!

fieldsFor: anObject
	| f inspector |
	inspector := self nativeInspectorOn: anObject.
	f := OrderedCollection new.
	(inspector headerList collect: [:anAssociation | anAssociation key]), inspector getFieldList
		keysAndValuesDo: [:eachKey :eachValue | f add: eachValue -> (inspector variableAtIndex: eachKey)].
	^f!

nativeInspectorOn: anObject

	^(BasicInspector new)
		model: anObject asValue;
		yourself! !
!WADolphinInspector categoriesFor: #doIt!actions!public! !
!WADolphinInspector categoriesFor: #evaluate!actions!public! !
!WADolphinInspector categoriesFor: #fieldsFor:!private! !
!WADolphinInspector categoriesFor: #nativeInspectorOn:!public! !

!WADolphinInspector class methodsFor!

initialize
	self select!

openNativeInspectorOn: anObject
	anObject inspect!

unload
	self unselect! !
!WADolphinInspector class categoriesFor: #initialize!class initialization!public! !
!WADolphinInspector class categoriesFor: #openNativeInspectorOn:!inspecting!public! !
!WADolphinInspector class categoriesFor: #unload!class initialization!public! !

WADolphinWalkback guid: (GUID fromString: '{81C29EDD-082F-4104-8099-5AE2FB92D3D2}')!
WADolphinWalkback comment: 'Previous implementation of WASqueakWalkback
probably has things to fix'!
!WADolphinWalkback categoriesForClass!Unclassified! !
!WADolphinWalkback methodsFor!

frameCount
	^ frames size!

frames
	^ frames first: (self limit min: frames size)!

initializeWithException: anException
	| context |
	super initializeWithException: anException.
	context := anException raisingFrame.
	frames := OrderedCollection new.
	[ context isNil ] whileFalse: [
		frames add: context.
		context := context sender ]!

renderObject: anObject labelled: aString on: html
	html definitionTerm: [
		html anchor
			callback: [ (WADolphinInspector on: anObject for: self) execute ];
			with: aString ].
	html definitionData: (self safePrintString: anObject)!

renderStackFrame: aContext on: html
	html definitionList: [
		self 
			renderObject: aContext
			labelled: 'thisContext'
			on: html.
		self 
			renderObject: aContext receiver
			labelled: 'self'
			on: html.
		self 
			tempNamesAndValuesIn: aContext 
			do: [ :name :value |
				self 
					renderObject: name
					labelled: value
					on: html ] ]!

renderStackOn: html
	html heading level: 3; with: 'Stack Trace'.
	html orderedList: [
		self frames do: [ :each |
			html listItem: [
				self renderStackFrame: each on: html ] ] ]!

safePrintString: anObject
	^ [ anObject printStringLimitedTo: 100 ]
		on: Error
		do: [ :err | 'unprintable ' , anObject class name ]!

tempNamesAndValuesIn: aContext do: aTwoArgumentBlock
	"Evaluate aTwoArgumentBlock for every temp in aContext with the name of the temp and the current value. The code is supposed to work on Squeak and Pharo closure and non-closure images."

	| tempNames |
	tempNames := aContext tempNames.
	(aContext respondsTo: #namedTempAt:)
		ifTrue: [
			tempNames keysAndValuesDo: [ :index :each |
				aTwoArgumentBlock
					value: each
					value: (aContext namedTempAt: index) ] ]
		ifFalse: [
			tempNames do: [ :each |
				aTwoArgumentBlock
					value: each
					value: (aContext tempAt: (tempNames indexOf: each)) ] ]! !
!WADolphinWalkback categoriesFor: #frameCount!accessing!public! !
!WADolphinWalkback categoriesFor: #frames!accessing!public! !
!WADolphinWalkback categoriesFor: #initializeWithException:!initialization!public! !
!WADolphinWalkback categoriesFor: #renderObject:labelled:on:!public!rendering-stack! !
!WADolphinWalkback categoriesFor: #renderStackFrame:on:!public!rendering-stack! !
!WADolphinWalkback categoriesFor: #renderStackOn:!public!rendering! !
!WADolphinWalkback categoriesFor: #safePrintString:!private! !
!WADolphinWalkback categoriesFor: #tempNamesAndValuesIn:do:!private! !

!WADolphinWalkback class methodsFor!

initialize
	self select!

unload
	self unselect! !
!WADolphinWalkback class categoriesFor: #initialize!class initialization!public! !
!WADolphinWalkback class categoriesFor: #unload!class initialization!public! !

"Binary Globals"!

