| package |
package := Package name: 'Seaside-Email'.
package paxVersion: 1;
	basicComment: ''.

package basicPackageVersion: '15'.


package classNames
	add: #WAEmailBody;
	add: #WAEmailMessage;
	add: #WAStringEmailBody;
	yourself.

package methodNames
	add: #GRPlatform -> #seasideDeliverEmailMessage:;
	add: 'GRPackage class' -> #seasideEmail;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Grease\Grease-Core';
	add: '..\..\IDB\IDB DateAndTime';
	add: 'Seaside-Core';
	yourself).

package!

"Class Definitions"!

WAObject subclass: #WAEmailBody
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAObject subclass: #WAEmailMessage
	instanceVariableNames: 'to from cc bcc subject headers body'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAEmailBody subclass: #WAStringEmailBody
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!GRPackage class methodsFor!

seasideEmail
	^ self new
		name: 'Seaside-Email';
		description: 'Contains convenience classes for sending of emails and preventing header injections.';
		addDependency: 'Seaside-Core';
		url: #seasideUrl;
		yourself! !
!GRPackage class categoriesFor: #seasideEmail!*seaside-email!public! !

!GRPlatform methodsFor!

seasideDeliverEmailMessage: aWAEmailMessage

	self subclassResponsibility! !
!GRPlatform categoriesFor: #seasideDeliverEmailMessage:!*seaside-email!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

WAEmailBody guid: (GUID fromString: '{B6F43502-173F-4730-AC5D-096F75FEF8D2}')!
WAEmailBody comment: ''!
!WAEmailBody categoriesForClass!Unclassified! !
WAEmailMessage guid: (GUID fromString: '{4691280E-E08B-4639-B4B6-9A4CBBE71CF0}')!
WAEmailMessage comment: ''!
!WAEmailMessage categoriesForClass!Unclassified! !
!WAEmailMessage methodsFor!

addAllTo: aCollectionOfEmailAddress
	aCollectionOfEmailAddress do: [ :each |
		self addTo: each ]!

addBcc: anEmailAddress
	^self bcc add: anEmailAddress!

addCc: anEmailAddress
	^self cc add: anEmailAddress!

addTo: anEmailAddress
	^self to add: anEmailAddress!

bcc
	^ bcc!

body
	^ body!

body: anObject
	body := anObject!

cc
	^ cc!

from
	^ from!

from: anObject
	from := anObject!

headerAt: aKeyString put: aValueString
	^self headers at: aKeyString put: aValueString!

headers
	^ headers!

initialize
	super initialize.
	to := OrderedCollection new.
	cc := OrderedCollection new.
	bcc := OrderedCollection new.
	headers := Dictionary new.
	self 
		headerAt: 'Date'
		put: (GRPrinter rfc822 print: DateAndTime now)!

plainMessage
	^ String streamContents: 
		[ :stream | 
		self 
			renderAddress: self from
			withHeader: 'From: '
			on: stream.
		self to do: 
			[ :each | 
			self 
				renderAddress: each
				withHeader: 'To: '
				on: stream ].
		self cc do: 
			[ :each | 
			self 
				renderAddress: each
				withHeader: 'Cc: '
				on: stream ].
		self bcc do: 
			[ :each | 
			self 
				renderAddress: each
				withHeader: 'Bcc: '
				on: stream ].
		stream
			nextPutAll: 'Subject: ';
			nextPutAll: self subject;
			nextPut: Character cr;
			nextPut: Character cr.
		self body isNil ifFalse: [ stream nextPutAll: self body greaseString ] ]!

recipients
	^ self to , self cc , self bcc collect: [ :each | each greaseString ]!

renderAddress: anEmailAddress withHeader: aString on: aStream 
	aStream
		nextPutAll: aString;
		nextPutAll: anEmailAddress greaseString;
		nextPut: Character cr!

send
	GRPlatform current seasideDeliverEmailMessage: self!

setStringBody: aString wrap: aBoolean
	self body: (aBoolean
		ifTrue: [ WAStringEmailBody string: aString lineLength: 80 ]
		ifFalse: [ WAStringEmailBody string: aString ])!

subject
	^ subject!

subject: anObject
	subject := anObject!

to
	^ to! !
!WAEmailMessage categoriesFor: #addAllTo:!adding!public! !
!WAEmailMessage categoriesFor: #addBcc:!adding!public! !
!WAEmailMessage categoriesFor: #addCc:!adding!public! !
!WAEmailMessage categoriesFor: #addTo:!adding!public! !
!WAEmailMessage categoriesFor: #bcc!accessing!public! !
!WAEmailMessage categoriesFor: #body!accessing!public! !
!WAEmailMessage categoriesFor: #body:!accessing!public! !
!WAEmailMessage categoriesFor: #cc!accessing!public! !
!WAEmailMessage categoriesFor: #from!accessing!public! !
!WAEmailMessage categoriesFor: #from:!accessing!public! !
!WAEmailMessage categoriesFor: #headerAt:put:!adding!public! !
!WAEmailMessage categoriesFor: #headers!accessing!public! !
!WAEmailMessage categoriesFor: #initialize!initialization!public! !
!WAEmailMessage categoriesFor: #plainMessage!private! !
!WAEmailMessage categoriesFor: #recipients!private! !
!WAEmailMessage categoriesFor: #renderAddress:withHeader:on:!private! !
!WAEmailMessage categoriesFor: #send!actions!public! !
!WAEmailMessage categoriesFor: #setStringBody:wrap:!convenience!public! !
!WAEmailMessage categoriesFor: #subject!accessing!public! !
!WAEmailMessage categoriesFor: #subject:!accessing!public! !
!WAEmailMessage categoriesFor: #to!accessing!public! !

!WAEmailMessage class methodsFor!

from: aFromEmailAddress to: aToEmailAddress subject: aString
	^self new
		from: aFromEmailAddress;
		addTo: aToEmailAddress;
		subject: aString;
		yourself!

from: aFromEmailAddress toAll: aCollectionOfEmailAddress subject: aString
	^self new
		from: aFromEmailAddress;
		addAllTo: aCollectionOfEmailAddress;
		subject: aString;
		yourself! !
!WAEmailMessage class categoriesFor: #from:to:subject:!instance creation!public! !
!WAEmailMessage class categoriesFor: #from:toAll:subject:!instance creation!public! !

WAStringEmailBody guid: (GUID fromString: '{952E9D47-CA96-495D-BC59-C4265A472F17}')!
WAStringEmailBody comment: ''!
!WAStringEmailBody categoriesForClass!Unclassified! !
!WAStringEmailBody methodsFor!

greaseString
	^ self string!

string
	^ string!

string: aString
	string := aString! !
!WAStringEmailBody categoriesFor: #greaseString!accessing!public! !
!WAStringEmailBody categoriesFor: #string!accessing!public! !
!WAStringEmailBody categoriesFor: #string:!accessing!public! !

!WAStringEmailBody class methodsFor!

convertCrLfToCrIn: aString
	^aString
		copyReplaceAll: (String with: Character cr with: Character lf)
		with: (String with: Character cr)!

splitIntoLines: aString
	^Array streamContents: [ :stream |
		| cr start |
		cr := Character cr.
		start := 1.
		1 to: aString size do: [ :index |
			(aString at: index) = cr ifTrue: [
				stream nextPut: (aString copyFrom: start to: index - 1).
				start := index + 1 ] ].
			start <= (aString size + 1) ifTrue: [ 
				stream nextPut: (aString copyFrom: start to: aString size) ] ]!

splitString: aString intoChunksNoLongerThan: anInteger
	^Array streamContents: [ :stream |
		self
			splitString: aString
			intoChunksNoLongerThan: anInteger
			into: stream ]!

splitString: aString intoChunksNoLongerThan: anInteger into: aStream
	| space lastSpaceIndex toScan |
	space := Character space.
	toScan := aString size min: anInteger + 1.
	lastSpaceIndex := 0.
	1 to: toScan do: [ :index |
		(aString at: index) = space ifTrue: [
			lastSpaceIndex := index ] ].
	
	lastSpaceIndex = 0
		ifTrue: [
			lastSpaceIndex := aString size min: anInteger.
			aStream nextPut: (aString copyFrom: 1 to: lastSpaceIndex) ]
		ifFalse: [
			aStream nextPut: (aString copyFrom: 1 to: lastSpaceIndex - 1) ].
		
	lastSpaceIndex < aString size ifTrue: [
		self
			splitString: (aString copyFrom: lastSpaceIndex + 1 to: aString size)
			intoChunksNoLongerThan: anInteger
			into: aStream ]!

string: aString
	^self new
		string: (self convertCrLfToCrIn: aString);
		yourself!

string: aString lineLength: anInteger
	| string |
	string := String streamContents: [ :stream |
		| lines |
		lines := self splitIntoLines: (self convertCrLfToCrIn: aString).
		lines do: [ :each |
			each size <= anInteger
				ifTrue: [ stream nextPutAll: each ]
				ifFalse: [
					(self splitString: each intoChunksNoLongerThan: anInteger)
						do: [ :chunk | stream nextPutAll: chunk ]
						separatedBy: [ stream nextPut: Character cr ] ] ]
			separatedBy: [ stream nextPut: Character cr ] ].
	^self new
		string: string;
		yourself! !
!WAStringEmailBody class categoriesFor: #convertCrLfToCrIn:!private! !
!WAStringEmailBody class categoriesFor: #splitIntoLines:!private! !
!WAStringEmailBody class categoriesFor: #splitString:intoChunksNoLongerThan:!private! !
!WAStringEmailBody class categoriesFor: #splitString:intoChunksNoLongerThan:into:!private! !
!WAStringEmailBody class categoriesFor: #string:!instance creation!public! !
!WAStringEmailBody class categoriesFor: #string:lineLength:!instance creation!public! !

"Binary Globals"!

