| package |
package := Package name: 'Seaside-Environment'.
package paxVersion: 1;
	basicComment: ''.

package basicPackageVersion: '71'.


package classNames
	add: #WAEnvironment;
	add: #WAHtmlErrorHandler;
	add: #WAHtmlFileHandlerListing;
	add: #WAHtmlResponseGenerator;
	add: #WALegacyRedirectionHandler;
	yourself.

package methodNames
	add: 'GRPackage class' -> #seasideEnvironment;
	add: 'WADispatcher class' -> #resetAll;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\Grease\Grease-Core';
	add: 'Seaside-Canvas';
	add: 'Seaside-Core';
	add: 'Seaside-Tools-Core';
	yourself).

package!

"Class Definitions"!

WAObject subclass: #WAEnvironment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: 'configuredApplicationDefaults configuredExceptionHandlingDefaults'!
WAErrorHandler subclass: #WAHtmlErrorHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAFileHandlerListing subclass: #WAHtmlFileHandlerListing
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WARequestHandler subclass: #WALegacyRedirectionHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAResponseGenerator subclass: #WAHtmlResponseGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!GRPackage class methodsFor!

seasideEnvironment
	^ self new
		name: 'Seaside-Environment';
		description: 'The standard Seaside distribution environment. This package can depend on all other code packages as needed.';
		addDependency: 'Seaside-Tools-Core';
		addDependency: 'Seaside-Core';
		addDependency: 'Seaside-Component';
		addDependency: 'Seaside-RenderLoop';
		addDependency: 'Seaside-Session';
		addDependency: 'Seaside-Canvas';
		addDependency: 'Seaside-Widgets'; "Users expect #inform:, etc. to be available"
		url: #seasideUrl;
		yourself! !
!GRPackage class categoriesFor: #seasideEnvironment!*seaside-environment!public! !

!WADispatcher class methodsFor!

resetAll
	self 
		greaseDeprecatedApi: 'WADispatcher class>>#resetAll'
		details: 'Use WAEnvironment>>#reloadApplications.'.
	WAEnvironment reloadApplications! !
!WADispatcher class categoriesFor: #resetAll!*seaside-environment!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

WAEnvironment guid: (GUID fromString: '{8440C957-018C-48D5-8209-EAFAB980059F}')!
WAEnvironment comment: ''!
!WAEnvironment categoriesForClass!Unclassified! !
!WAEnvironment class methodsFor!

configureApplicationDefaults
	(configuredApplicationDefaults ifNil: [ false ]) ifFalse: [
		WAAdmin applicationDefaults
			at: #responseGenerator put: WAHtmlResponseGenerator.
		configuredApplicationDefaults := true ].
	!

configureApplicationExceptionHandlingDefaults
	(configuredExceptionHandlingDefaults ifNil: [ false ]) ifFalse: [
		WAAdmin applicationExceptionHandlingDefaults
			at: #exceptionHandler put: WAHtmlErrorHandler.
		configuredExceptionHandlingDefaults := true ].
	!

initialize
	self configureApplicationDefaults.
	self configureApplicationExceptionHandlingDefaults.
	self registerDefaultRequestHandlers.
	self registerDefaultComponents!

registerDefaultComponents
	WAPainter allSubclasses do: [ :each |
		(each class includesSelector: #initialize)
			ifTrue: [ each initialize ] ]!

registerDefaultRequestHandlers
	WAFileHandler default: WAFileHandler new.
	WAFileHandler default 
		preferenceAt: #fileHandlerListingClass
		put: WAHtmlFileHandlerListing.
	WAAdmin defaultDispatcher 
		register: WAFileHandler default
		at: 'files'.
	WAAdmin defaultDispatcher 
		register: WALegacyRedirectionHandler new
		at: 'seaside'!

reloadApplications
	WAAdmin clearAll.
	self registerDefaultRequestHandlers.
	self registerDefaultComponents! !
!WAEnvironment class categoriesFor: #configureApplicationDefaults!class initialization!public! !
!WAEnvironment class categoriesFor: #configureApplicationExceptionHandlingDefaults!class initialization!public! !
!WAEnvironment class categoriesFor: #initialize!class initialization!public! !
!WAEnvironment class categoriesFor: #registerDefaultComponents!public!registration! !
!WAEnvironment class categoriesFor: #registerDefaultRequestHandlers!public!registration! !
!WAEnvironment class categoriesFor: #reloadApplications!public!registration! !

WAHtmlErrorHandler guid: (GUID fromString: '{347FBCAD-D733-489D-918B-2B60AD1C3B6C}')!
WAHtmlErrorHandler comment: '
This exception handler returns a simple HTML error message response for Warnings and Errors.

It provides two methods to allow the easy creation of custom error responses:

#titleForException:  - can be used to override the text displayed in the title
#renderContentForException:  - can be used to fully customize the HTML output'!
!WAHtmlErrorHandler categoriesForClass!Unclassified! !
!WAHtmlErrorHandler methodsFor!

handleDefault: anException
	self requestContext respond: [ :response |
		response
			internalError;
			contentType: WAMimeType textHtml;
			nextPutAll: (self rendererClass builder
				fullDocument: true;
				rootBlock: [ :root | self updateHtmlRoot: root forException: anException ];
				render: [ :html | self renderContentForException: anException on: html ]) ]!

renderContentForException: anException on: html
	html heading level1; with: anException description.
	html paragraph: 'Your request could not be completed. An exception occurred.'!

rendererClass
	^ WARenderCanvas!

titleForException: anException
	^ anException description!

updateHtmlRoot: anHtmlRoot forException: anException
	anHtmlRoot title: (self titleForException: anException)! !
!WAHtmlErrorHandler categoriesFor: #handleDefault:!actions!public! !
!WAHtmlErrorHandler categoriesFor: #renderContentForException:on:!actions!public! !
!WAHtmlErrorHandler categoriesFor: #rendererClass!accessing!public! !
!WAHtmlErrorHandler categoriesFor: #titleForException:!content!public! !
!WAHtmlErrorHandler categoriesFor: #updateHtmlRoot:forException:!content!public! !

WAHtmlFileHandlerListing guid: (GUID fromString: '{5E2AF4F1-79FB-4732-8E6F-452C5B3197F8}')!
WAHtmlFileHandlerListing comment: ''!
!WAHtmlFileHandlerListing categoriesForClass!Unclassified! !
!WAHtmlFileHandlerListing methodsFor!

responseForLibraries
	| title contents |
	title := 'Index of ', handler url pathString.
	contents := WARenderCanvas builder
		fullDocument: true;
		rootBlock: [ :html | html title: title ];
		render: [ :html |
			html heading: title.
			html unorderedList: [
				handler libraries do: [ :each |
					html listItem: [
						html anchor
							url: (each urlOf: '');
							with: each name ] ] ] ].
	context respond: [ :response |
		response
			contentType: WAMimeType textHtml;
			nextPutAll: contents ]!

responseForLibrary: aLibrary
	| title contents |
	title := 'Index of ', aLibrary name.
	contents := WARenderCanvas builder
		fullDocument: true;
		rootBlock: [ :html | html title: title ];
		render: [ :html |
			html heading level: 1; with: title.
			html table: [
				html tableRow: [
					html tableHeading: 'Name'.
					html tableHeading: 'Size'.
					html tableHeading: 'Description' ].
				aLibrary fileSelectors do: [ :each |
					html tableRow: [
						html tableData: [
							html anchor
								url: (aLibrary urlOf: each);
								with: (aLibrary asFilename: each) ].
						html tableData: (aLibrary fileSizeOf: each).
						html tableData: (aLibrary mimetypeForFile: (aLibrary asFilename: each)) ] ] ] ].
	context respond: [ :response |
		response
			contentType: WAMimeType textHtml;
			nextPutAll: contents ]! !
!WAHtmlFileHandlerListing categoriesFor: #responseForLibraries!public!responding! !
!WAHtmlFileHandlerListing categoriesFor: #responseForLibrary:!public!responding! !

WALegacyRedirectionHandler guid: (GUID fromString: '{65FACBF7-6E34-4653-AD2D-CBEA22792FA1}')!
WALegacyRedirectionHandler comment: 'I provide compaitility with old Seaside URLs that have Seaside in the path (eg. ''/seaside/examples/counter'') by simply removing it.'!
!WALegacyRedirectionHandler categoriesForClass!Unclassified! !
!WALegacyRedirectionHandler methodsFor!

handleFiltered: aRequestContext
	| url |
	url := aRequestContext request url copy.
	url path removeFirst.
	aRequestContext response
		movedPermanently;
		location: url.
	aRequestContext respond! !
!WALegacyRedirectionHandler categoriesFor: #handleFiltered:!public!request-handling! !

!WALegacyRedirectionHandler class methodsFor!

description
	^ 'Legacy redirection'! !
!WALegacyRedirectionHandler class categoriesFor: #description!accessing!public! !

WAHtmlResponseGenerator guid: (GUID fromString: '{D1A549FD-A665-4C0B-8818-B063CDF1C481}')!
WAHtmlResponseGenerator comment: 'This subclass overrides certain generation methods to create prettier HTML responses, instead of plain text.'!
!WAHtmlResponseGenerator categoriesForClass!Unclassified! !
!WAHtmlResponseGenerator methodsFor!

forbidden
	self response
		forbidden;
		contentType: WAMimeType textHtml;
		nextPutAll: (WARenderCanvas builder
			fullDocument: true;
			rootBlock: [ :root | root title: 'Forbidden' ];
			render: [ :html |
				html heading: [
					html text: 'Error: you are forbidden to access "'.
					html text: self request url pathString.
					html text: '"' ] ])!

notFound
	self response
		notFound;
		contentType: WAMimeType textHtml;
		nextPutAll: (WARenderCanvas builder
			fullDocument: true;
			rootBlock: [ :root | root title: 'Not Found' ];
			render: [ :html |
				html heading: [
					html text: 'Error: "'.
					html text: self request url pathString.
					html text: '" not found' ] ])!

pageExpired
	self response
		doNotCache;
		contentType: WAMimeType textHtml;
		nextPutAll: (WARenderCanvas builder
			fullDocument: true;
			rootBlock: [ :root | root title: 'Page Expired' ];
			render: [ :html | 
				html heading: 'Page Expired'.
				html paragraph: 'This page has expired. Please use the back-button to navigate to the last valid page.' ])!

refreshAfter: aNumber location: aUrlOrString message: aString 
	self response
		contentType: WAMimeType textHtml;
		refreshAfter: aNumber
			location: aUrlOrString;
		nextPutAll: (WARenderCanvas builder
				fullDocument: true;
				render: 
					[ :html | 
					html heading: aString.
					html paragraph: 
						[ html text: 'You are being redirected to '.
						html anchor
							url: aUrlOrString;
							with: aUrlOrString greaseString ] ])! !
!WAHtmlResponseGenerator categoriesFor: #forbidden!generation!public! !
!WAHtmlResponseGenerator categoriesFor: #notFound!generation!public! !
!WAHtmlResponseGenerator categoriesFor: #pageExpired!generation!public! !
!WAHtmlResponseGenerator categoriesFor: #refreshAfter:location:message:!generation!public! !

"Binary Globals"!

