| package |
package := Package name: 'Seaside-RenderLoop'.
package paxVersion: 1;
	basicComment: ''.

package basicPackageVersion: '72'.


package classNames
	add: #WAActionPhaseContinuation;
	add: #WAMissingResponseError;
	add: #WARenderLoopConfiguration;
	add: #WARenderLoopContinuation;
	add: #WARenderLoopMain;
	add: #WARenderPhaseContinuation;
	yourself.

package methodNames
	add: #WASession -> #presenter;
	add: 'GRPackage class' -> #seasideRenderLoop;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Grease\Grease-Core';
	add: 'Seaside-Component';
	add: 'Seaside-Core';
	add: 'Seaside-Session';
	yourself).

package!

"Class Definitions"!

WAError subclass: #WAMissingResponseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WASystemConfiguration subclass: #WARenderLoopConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAMain subclass: #WARenderLoopMain
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WASessionContinuation subclass: #WARenderLoopContinuation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WARenderLoopContinuation subclass: #WAActionPhaseContinuation
	instanceVariableNames: 'renderContext'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WARenderLoopContinuation subclass: #WARenderPhaseContinuation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!GRPackage class methodsFor!

seasideRenderLoop
	^ self new
		name: 'Seaside-RenderLoop';
		description: 'Defines the low-level rendering and callback processing functionality.';
		addDependency: 'Seaside-Session';
		addDependency: 'Seaside-Component';
		url: #seasideUrl;
		yourself! !
!GRPackage class categoriesFor: #seasideRenderLoop!*seaside-renderloop!public! !

!WASession methodsFor!

presenter
	^ self properties at: #presenter ifAbsent: [ nil ]! !
!WASession categoriesFor: #presenter!*seaside-renderloop!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

WAMissingResponseError guid: (GUID fromString: '{5312A854-17BB-43B4-8264-AD4F958CF5B0}')!
WAMissingResponseError comment: ''!
!WAMissingResponseError categoriesForClass!Unclassified! !
!WAMissingResponseError methodsFor!

seasidePossibleCauses
	^ #(
		'A Component attempted to use #call: or a similar method while rendering. Calling and answering can only be done in a callback.'
	)! !
!WAMissingResponseError categoriesFor: #seasidePossibleCauses!accessing!public! !

WARenderLoopConfiguration guid: (GUID fromString: '{C05D4934-1FB0-4F02-9B20-41CF72D48851}')!
WARenderLoopConfiguration comment: '
WARenderLoopConfiguration defines attributes required by the Render Loop classes (see the Seaside-Core-RenderLoop category). Any application using these classes (e.g. WARenderLoopMain) should include this configuration (this is done by default for new applications).

See #addAttributes: for details on provided attributes.'!
!WARenderLoopConfiguration categoriesForClass!Unclassified! !
!WARenderLoopConfiguration methodsFor!

continuationClasses
	^ WARenderLoopContinuation allSubclasses asSortedCollection: [ :a :b | a name <= b name ]!

describeOn: config
	(config list: #actionPhaseContinuationClass)
		beAdvanced;
		group: #renderLoop;
		comment: 'The continuation used for the Render Loop''s action phase.';
		options: [ self continuationClasses ];
		default: WAActionPhaseContinuation.
	(config list: #renderPhaseContinuationClass)
		beAdvanced;
		group: #renderLoop;
		comment: 'The continuation used for the Render Loop''s render phase.';
		options: [ self continuationClasses ];
		default: WARenderPhaseContinuation.
	(config list: #rootClass)
		comment: 'The root component of this seaside application.';
		options: [ self rootClasses ].
	(config collection: #rootDecorationClasses)
		comment: 'The decoration clases that should wrap the root component.';
		options: [ self rootDecorationClasses ].
	config
		at: #mainClass put: WARenderLoopMain!

parents
	"We override several attributes of the application configuration."
	^ Array with: WAApplicationConfiguration instance!

rootClasses
	^ (WAComponent allSubclasses select: [ :each | each canBeRoot ])
		asSortedCollection: [ :a :b | a name <= b name ]!

rootDecorationClasses
	^ (WADecoration allSubclasses select: [ :each | each canBeRoot ])
		asSortedCollection: [ :a :b | a name <= b name ]! !
!WARenderLoopConfiguration categoriesFor: #continuationClasses!description!public! !
!WARenderLoopConfiguration categoriesFor: #describeOn:!description!public! !
!WARenderLoopConfiguration categoriesFor: #parents!ancestry!public! !
!WARenderLoopConfiguration categoriesFor: #rootClasses!description!public! !
!WARenderLoopConfiguration categoriesFor: #rootDecorationClasses!description!public! !

WARenderLoopMain guid: (GUID fromString: '{00BAD521-182B-41D6-ABD7-E39419BC1AED}')!
WARenderLoopMain comment: '
When a new session on a WAApplication is started WARenderLoopMain initializes the application, that is it:
 
	creates the top level component of the application, 
	informs each component(WAPresenter) of the application that the session started (via WAPresenter>>initialRequest:)
	starts a WARenderLoop to handle the request
'!
!WARenderLoopMain categoriesForClass!Unclassified! !
!WARenderLoopMain methodsFor!

createRoot
	^ self rootDecorationClasses
		inject: self rootClass new
		into: [ :component :decorationClass |
			component 
				addDecoration: decorationClass new;
				yourself ]!

prepareRoot: aComponent
	(WAAllPresenterGuide client:
		(WAInitialRequestVisitor request: self requestContext request))
		start: aComponent.
	(WAVisiblePresenterGuide client: (WATaskVisitor new)) start: aComponent!

rootClass
	^ self application preferenceAt: #rootClass!

rootDecorationClasses
	^ self application preferenceAt: #rootDecorationClasses!

start
	| root |
	root := self createRoot.
	self session properties at: #presenter put: root.
	self prepareRoot: root.
	((self application preferenceAt: #renderPhaseContinuationClass) new) captureAndInvoke! !
!WARenderLoopMain categoriesFor: #createRoot!creational!public! !
!WARenderLoopMain categoriesFor: #prepareRoot:!processing!public! !
!WARenderLoopMain categoriesFor: #rootClass!accessing!public! !
!WARenderLoopMain categoriesFor: #rootDecorationClasses!accessing!public! !
!WARenderLoopMain categoriesFor: #start!processing!public! !

WARenderLoopContinuation guid: (GUID fromString: '{7638DFBD-47BD-43AE-94D6-2F4793ABAB12}')!
WARenderLoopContinuation comment: ''!
!WARenderLoopContinuation categoriesForClass!Unclassified! !
!WARenderLoopContinuation methodsFor!

createActionContinuation
	"Answer a new action phase continuation."

	^ (self application preferenceAt: #actionPhaseContinuationClass) new
			setStates: self states;
			yourself!

createRenderContinuation
	"Answer a new render phase continuation."

	^ (self application preferenceAt: #renderPhaseContinuationClass) new!

presenter
	"Answer the root presenter instance for the current session."

	^ self session presenter!

toPresenterSendRoot: anHtmlRoot
	(WAVisiblePresenterGuide client:
		(WAUpdateRootVisitor root: anHtmlRoot))
		start: self presenter!

updateRoot: anHtmlRoot
	anHtmlRoot meta contentType: self application contentType.
	anHtmlRoot meta contentScriptType: WAMimeType textJavascript.
	anHtmlRoot htmlAttributes
		at: 'xmlns' put: 'http://www.w3.org/1999/xhtml';
		at: 'xml:lang' put: 'en';
		at: 'lang' put: 'en'.
	anHtmlRoot beXhtml10Strict.
	anHtmlRoot title: 'Seaside'.
	self application libraries 
		do: [ :each | each default updateRoot: anHtmlRoot ].
	self session updateRoot: anHtmlRoot!

updateStates: aSnapshot
	super updateStates: aSnapshot.
	(WAAllPresenterGuide client:
		(WAUpdateStatesVisitor snapshot: aSnapshot))
		start: self presenter!

updateUrl: aUrl
	super updateUrl: aUrl.
	(WAVisiblePresenterGuide client: (WAUpdateUrlVisitor url: aUrl))
		start: self presenter!

withNotificationHandlerDo: aBlock
	^ aBlock on: WARenderNotification do: [ :n | ]! !
!WARenderLoopContinuation categoriesFor: #createActionContinuation!private! !
!WARenderLoopContinuation categoriesFor: #createRenderContinuation!private! !
!WARenderLoopContinuation categoriesFor: #presenter!accessing!public! !
!WARenderLoopContinuation categoriesFor: #toPresenterSendRoot:!public!updating! !
!WARenderLoopContinuation categoriesFor: #updateRoot:!public!updating! !
!WARenderLoopContinuation categoriesFor: #updateStates:!public!updating! !
!WARenderLoopContinuation categoriesFor: #updateUrl:!public!updating! !
!WARenderLoopContinuation categoriesFor: #withNotificationHandlerDo:!private! !

WAActionPhaseContinuation guid: (GUID fromString: '{8EEC58A6-EB21-4C9B-846D-6C854F5B6FF6}')!
WAActionPhaseContinuation comment: ''!
!WAActionPhaseContinuation categoriesForClass!Unclassified! !
!WAActionPhaseContinuation methodsFor!

continue
	| renderContinuation |
	renderContinuation := self createRenderContinuation.
	renderContinuation captureState.
	(self shouldRedirect) 
		ifTrue: [ self redirectToContinuation: renderContinuation ]
		ifFalse: [ renderContinuation basicValue ]!

handleRequest
	self runCallbacks.
	self continue!

renderContext
	^ renderContext!

renderContext: aRenderContext
	renderContext := aRenderContext!

runCallbacks
	[
	self withNotificationHandlerDo: [ 
		self renderContext callbacks handle: self requestContext ].
	(WAVisiblePresenterGuide client: (WATaskVisitor new)) start: self presenter ]
		ensure: [ self request isXmlHttpRequest ifTrue: [ self captureState ] ]!

shouldRedirect
	| request |
	request := self request.
	^ (request isGet and: [ request fields includesKey: '_n' ]) not! !
!WAActionPhaseContinuation categoriesFor: #continue!processing!public! !
!WAActionPhaseContinuation categoriesFor: #handleRequest!processing!public! !
!WAActionPhaseContinuation categoriesFor: #renderContext!accessing!public! !
!WAActionPhaseContinuation categoriesFor: #renderContext:!accessing!public! !
!WAActionPhaseContinuation categoriesFor: #runCallbacks!processing!public! !
!WAActionPhaseContinuation categoriesFor: #shouldRedirect!private! !

WARenderPhaseContinuation guid: (GUID fromString: '{0EDB2FAC-7CCD-4686-89DD-EA8F76ADD505}')!
WARenderPhaseContinuation comment: ''!
!WARenderPhaseContinuation categoriesForClass!Unclassified! !
!WARenderPhaseContinuation methodsFor!

createHtmlRootWithContext: aRenderContext
	^WAHtmlRoot context: aRenderContext!

createRenderContext
	| renderContext |
	renderContext := WARenderContext new.
	
	"Wrap the default visitor with a rendering guide"
	renderContext visitor: (WARenderingGuide client: renderContext visitor).
	
	^ renderContext
		!

handleRequest
	self session clearJumpTo.
	self respond: [ :response | self processRendering: response ].

	"The render phase should have returned a response"
	WAMissingResponseError signal!

processRendering: aResponse 
	| document actionContinuation renderContext actionUrl actionBaseUrl htmlRoot |
	"Prepare the response"
	aResponse contentType: self application contentType.

	"Create objects we need"
	document := self requestContext newDocument.
	actionContinuation := self createActionContinuation.
	renderContext := self createRenderContext.
	
	"Create URL"
	actionUrl := actionContinuation registerForUrl: false.
	actionBaseUrl := actionUrl copy.
	self updateUrl: actionUrl.

	"Hook objects together"
	renderContext
		document: document;
		actionUrl: actionUrl;
		actionBaseUrl: actionBaseUrl;
		resourceUrl: self application resourceBaseUrl.
	actionContinuation renderContext: renderContext.
	htmlRoot := self createHtmlRootWithContext: renderContext.
	self updateRoot: htmlRoot.
	self toPresenterSendRoot: htmlRoot.

	"Render"
	document open: htmlRoot.
	self presenter renderWithContext: renderContext.
	document close.
	renderContext destroy! !
!WARenderPhaseContinuation categoriesFor: #createHtmlRootWithContext:!private! !
!WARenderPhaseContinuation categoriesFor: #createRenderContext!private! !
!WARenderPhaseContinuation categoriesFor: #handleRequest!processing!public! !
!WARenderPhaseContinuation categoriesFor: #processRendering:!processing!public! !

"Binary Globals"!

