| package |
package := Package name: 'Seaside-Session'.
package paxVersion: 1;
	basicComment: ''.

package basicPackageVersion: '134'.


package classNames
	add: #WAApplication;
	add: #WAApplicationConfiguration;
	add: #WACurrentSession;
	add: #WAMain;
	add: #WAMutualExclusionFilter;
	add: #WASession;
	add: #WASessionContinuation;
	yourself.

package methodNames
	add: #WAObject -> #application;
	add: #WAObject -> #session;
	add: #WARequestContext -> #application;
	add: #WARequestContext -> #redirectTo:;
	add: #WARequestContext -> #session;
	add: #WARequestFilter -> #updateStates:;
	add: #WARequestHandler -> #isApplication;
	add: #WARequestHandler -> #isSession;
	add: 'GRPackage class' -> #seasideSession;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Grease\Grease-Core';
	add: 'Seaside-Core';
	yourself).

package!

"Class Definitions"!

WAObject subclass: #WACurrentSession
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAObject subclass: #WAMain
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAObject subclass: #WASessionContinuation
	instanceVariableNames: 'states'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WASystemConfiguration subclass: #WAApplicationConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WARequestFilter subclass: #WAMutualExclusionFilter
	instanceVariableNames: 'mutex'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WARequestHandler subclass: #WASession
	instanceVariableNames: 'continuations properties jumpTo'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WARegistry subclass: #WAApplication
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!GRPackage class methodsFor!

seasideSession
	^ self new
		name: 'Seaside-Session';
		description: 'Support for stateful and secure session handling.';
		addDependency: 'Seaside-Core';
		url: #seasideUrl;
		yourself! !
!GRPackage class categoriesFor: #seasideSession!*seaside-session!public! !

!WAObject methodsFor!

application
	"Answer the current seaside application, instance of WAApplication or a subclass."
	
	^self requestContext application!

session
	"Answer the current seaside session, instance of WASesssion or a subclass."
	
	^self requestContext session! !
!WAObject categoriesFor: #application!*seaside-session!public! !
!WAObject categoriesFor: #session!*seaside-session!public! !

!WARequestContext methodsFor!

application
	"Answer the current application or nil."

	^ handlers 
		detect: [ :each | each isApplication ]
		ifNone: [ nil ]!

redirectTo: aUrlOrString 
	self responseGenerator
		redirectTo: aUrlOrString;
		respond!

session
	"Answer the current session or nil."

	^ handlers 
		detect: [ :each | each isSession ]
		ifNone: [ nil ]! !
!WARequestContext categoriesFor: #application!*seaside-session!public! !
!WARequestContext categoriesFor: #redirectTo:!*seaside-session!public! !
!WARequestContext categoriesFor: #session!*seaside-session!public! !

!WARequestFilter methodsFor!

updateStates: aSnapshot
	aSnapshot register: next! !
!WARequestFilter categoriesFor: #updateStates:!*seaside-session!public! !

!WARequestHandler methodsFor!

isApplication
	^ false!

isSession
	^ false! !
!WARequestHandler categoriesFor: #isApplication!*seaside-session!public! !
!WARequestHandler categoriesFor: #isSession!*seaside-session!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

WACurrentSession guid: (GUID fromString: '{9E61A378-9187-4B86-BA66-9D2DBD48DFFA}')!
WACurrentSession comment: 'Available for backward compatibility. You should now use "self session" in any subclass of WAObject or "WACurrentRequestContext value session" anywhere else.'!
!WACurrentSession categoriesForClass!Unclassified! !
!WACurrentSession class methodsFor!

value
	| context |
	self 
		greaseDeprecatedApi: 'WACurrentSession class>>#value'
		details: 'Use "self session" on a subclass of WAObject or "WACurrentRequestContext value session".'.
	[ context := WACurrentRequestContext value ] 
		on: WARequestContextNotFound
		do: [ :e | ^ nil ].
	^ context session! !
!WACurrentSession class categoriesFor: #value!evaluating!public! !

WAMain guid: (GUID fromString: '{1E6C06D7-9CA1-4BC7-9F9D-BD83ED25D7CF}')!
WAMain comment: '
WAMain subclasses are used to initialize applications when a new session is started.

Subclasses must implement the following messages:
	start:
		Does the initializing 
'!
!WAMain categoriesForClass!Unclassified! !
!WAMain methodsFor!

start
	"The main entry point into a session. Subclasses should override this to do something
	and return a response."! !
!WAMain categoriesFor: #start!processing!public! !

WASessionContinuation guid: (GUID fromString: '{FB98163C-727B-4A4C-A768-42FF327FDABF}')!
WASessionContinuation comment: 'I represent a continuation as part of the flow of pages within a session. I am not a real continuation (as compared to those used in #call: and #answer:), I only represent a specific point in the session. I reference the root component and a memory snapshot of backtracked objects.'!
!WASessionContinuation categoriesForClass!Unclassified! !
!WASessionContinuation methodsFor!

basicValue
	self withUnregisteredHandlerDo: [ self handleRequest ]!

captureAndInvoke
	self captureState.
	self basicValue!

captureState
	"self states reset"
	"Setting states to nil matches the behaviour of Seaside 2.8. Resetting the Snapshot
	is useful for JS callbacks in that refreshing the page will show the modified state
	but can also result in many ActionPhaseContinuations all sharing the same state
	but with different callbacks, some of which may reference state that is no longer present."
	states := nil.
	self updateStates: self states!

handleRequest
	self subclassResponsibility!

redirectToContinuation: aSessionContinuation
	self respond: [ :response | response redirectTo: aSessionContinuation registerForUrl ]!

registerForUrl
	^ self registerForUrl: true!

registerForUrl: aBoolean
	| url |
	url := self session actionUrlForContinuation: self.
	aBoolean ifTrue: [ self updateUrl: url ].
	^ url!

request
	"Answer the current request object."

	^ self requestContext request!

respond: aBlock 
	self requestContext respond: aBlock!

setStates: aSnapshot
	states := aSnapshot!

states
	^ states ifNil: [ states := WASnapshot new ]!

updateStates: aSnapshot
	self session updateStates: aSnapshot!

updateUrl: aUrl
	self session updateUrl: aUrl!

url
	self 
		greaseDeprecatedApi: 'WASessionContinuation>>#url'
		details: 'Use WASessionContinuation>>registerForUrl.'.
	^ self registerForUrl!

value
	"Resume processing of a request. To ensure valid application state restore all registered states."

	self states restore.
	self basicValue!

withUnregisteredHandlerDo: aBlock 
	^ aBlock 
		on: WAUnregisteredHandlerError
		do: [ :err | self requestContext redirectTo: self application url ]! !
!WASessionContinuation categoriesFor: #basicValue!evaluating!public! !
!WASessionContinuation categoriesFor: #captureAndInvoke!evaluating!public! !
!WASessionContinuation categoriesFor: #captureState!processing!public! !
!WASessionContinuation categoriesFor: #handleRequest!processing!public! !
!WASessionContinuation categoriesFor: #redirectToContinuation:!private! !
!WASessionContinuation categoriesFor: #registerForUrl!private! !
!WASessionContinuation categoriesFor: #registerForUrl:!private! !
!WASessionContinuation categoriesFor: #request!accessing!public! !
!WASessionContinuation categoriesFor: #respond:!processing!public! !
!WASessionContinuation categoriesFor: #setStates:!initialization!public! !
!WASessionContinuation categoriesFor: #states!accessing!public! !
!WASessionContinuation categoriesFor: #updateStates:!public!updating! !
!WASessionContinuation categoriesFor: #updateUrl:!public!updating! !
!WASessionContinuation categoriesFor: #url!public! !
!WASessionContinuation categoriesFor: #value!evaluating!public! !
!WASessionContinuation categoriesFor: #withUnregisteredHandlerDo:!private! !

WAApplicationConfiguration guid: (GUID fromString: '{03DCB5A6-2954-435C-A8D2-47E8753248E9}')!
WAApplicationConfiguration comment: '
WAApplicationConfiguration defines attributes required by WAApplication. All applications should include this configuration (this is done by default for new applications).

See #addAttributes: for details on provided attributes.'!
!WAApplicationConfiguration categoriesForClass!Unclassified! !
!WAApplicationConfiguration methodsFor!

describeOn: config
	(config collection: #libraries)
		options: [ self libraryClasses ].
	(config list: #mainClass)
		beAdvanced;
		options: [ self mainClasses ];
		default: WAMain.
	(config string: #mimeType)
		beAdvanced;
		label: 'MIME Type';
		comment: 'The media type of HTML documents reported to the browser.';
		default: 'text/html'.
	(config url: #resourceBaseUrl)
		group: #server;
		comment: 'Sets the base URL for URLS generated with #resourceUrl:'.
	(config boolean: #sessionAllowTermination)
		comment: 'Whether sessions allows a special HTTP request to terminate the process currently running in the session. This is used during development and should be disabled for deployment.';
		default: false.
	(config list: #sessionClass)
		comment: 'Sessions in this application will be instances of this class.';
		options: [ self sessionClasses ];
		default: WASession
!

libraryClasses
	^ WAFileLibrary allSubclasses asSortedCollection: [ :a :b | a name <= b name ]!

mainClasses
	^ WAMain withAllSubclasses asSortedCollection: [ :a :b | a name <= b name ]!

parents
	^ Array with: WARegistryConfiguration instance!

sessionClasses
	^ WASession withAllSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !
!WAApplicationConfiguration categoriesFor: #describeOn:!description!public! !
!WAApplicationConfiguration categoriesFor: #libraryClasses!description!public! !
!WAApplicationConfiguration categoriesFor: #mainClasses!description!public! !
!WAApplicationConfiguration categoriesFor: #parents!ancestry!public! !
!WAApplicationConfiguration categoriesFor: #sessionClasses!description!public! !

WAMutualExclusionFilter guid: (GUID fromString: '{E572BB90-3DB1-4EC8-9F99-84BD78294B7F}')!
WAMutualExclusionFilter comment: ''!
!WAMutualExclusionFilter categoriesForClass!Unclassified! !
!WAMutualExclusionFilter methodsFor!

handleFiltered: aRequestContext
	(self shouldTerminate: aRequestContext)
		ifTrue: [ self terminate: aRequestContext ]
		ifFalse: [ mutex critical: [ super handleFiltered: aRequestContext ] ]!

initialize
	super initialize.
	mutex := WAMutex new!

shouldTerminate: aRequestContext
	^ (aRequestContext request fields includesKey: 'terminate')
		and: [ aRequestContext application preferenceAt: #sessionAllowTermination ]!

terminate: aRequestContext
	mutex terminateOwner.
	aRequestContext respond: [ :response |
		response
			contentType: WAMimeType textPlain;
			nextPutAll: 'Process terminated' ]! !
!WAMutualExclusionFilter categoriesFor: #handleFiltered:!processing!public! !
!WAMutualExclusionFilter categoriesFor: #initialize!initialization!public! !
!WAMutualExclusionFilter categoriesFor: #shouldTerminate:!private! !
!WAMutualExclusionFilter categoriesFor: #terminate:!private! !

WASession guid: (GUID fromString: '{4EB8595E-F770-49FA-88A0-DE47F54DD55C}')!
WASession comment: '
I am a Seaside session. A new instance of me gets created when an user accesses an application for the first time and is persistent as long as the user is interacting with it.

This class is intended to be subclasses by applications that need global state, like a user. Custom state can be added by creating instance variables and storing it there. The session can be retrieved by #session if inside a component or task or by evaluating: WACurrentRequestContext session

If the session has not been used for #defaultTimeoutSeconds, it is garbage collected by the system. To manually expire a session call #expire.

A good way to clear all sessions is the following code:

WARegistry clearAllHandlers.
WAPlatform current garbageCollect

Instance Variables
	cookiesEnabled:		<Boolean>
			
cookiesEnabled
	- Whether the client has a session cookie. This can only be true if #useSessionCookie is enabled.'!
!WASession categoriesForClass!Unclassified! !
!WASession methodsFor!

actionField
	^ '_k'!

actionUrlForContinuation: aContinuation
	^ self actionUrlForKey: (continuations store: aContinuation)!

actionUrlForKey: aString
	^ self url
		addField: self actionField value: aString;
		yourself!

addLoadScript: aString 
	self 
		greaseDeprecatedApi: 'WASession>>#addLoadScript:'
		details: 'Use WAHtmlDocument>>#addLoadScript:. For example: "html document addLoadScript:".'.

	"We don't have access to the Canvas here so we can't provide an implementation"
	self error: 'You can no longer call #addLoadScript: on WASession.'!

application
	^ self parent!

baseUrl
	self greaseDeprecatedApi: 'WASession>>#baseUrl'
		details: 'Use WAApplication>>#url.'.
	
	^ self application url!

clearJumpTo
	jumpTo := nil!

closePopup
	self 
		greaseDeprecatedApi: 'WASession>>#closePopup'
		details: 'Use WARequestContext>>#closeThisPopupAndReloadOpener.'.
	^ self requestContext closeThisPopupAndReloadOpener!

closePopupWithoutReloadingOpener
	self 
		greaseDeprecatedApi: 'WASession>>#closePopupWithoutReloadingOpener'
		details: 'Use WARequestContext>>#closeThisPopup.'.
	^ self requestContext closeThisPopup!

createCache
	"To configure the continuation cache you must currently subclass WASession and
	override this method."
	^ WACache new
		setExpiryPolicy: (WALeastRecentlyUsedExpiryPolicy new);
		yourself.!

currentRequest
	"Returns the current request object."
	self 
		greaseDeprecatedApi: 'WASession>>#currentRequest'
		details: 'Use WARequestContext>>#request directly. For example: "self requestContext request".'.
	^ self requestContext request!

expire
	self 
		greaseDeprecatedApi: 'WASession>>#expire'
		details: 'This method might be reimplemented again. In the meantime, if you just want to remove the Session from the Application, use WASession>>unregister (#unregistered will be called as a notification instead of #expired). Otherwise you should consider adding a Request Filter to the Session that implements whatever behaviour you want in order to block access to the Session.'.
	^ self unregister!

handleFiltered: aRequestContext
	| key continuation |
	key := aRequestContext request fields
		at: self actionField
		ifAbsent: [ ^ self start ].
		
	continuation := continuations
		at: key
		ifAbsent: [ ^ self unknownRequest ].
	continuation value!

initialize
	super initialize.
	self initializeFilters.
	continuations := self createCache!

initializeFilters
	self addFilter: WAMutualExclusionFilter new!

isSession
	^ true!

jumpToAnchor: aString
	jumpTo := aString!

key
	^ self application keyFor: self!

newSessionUrl
	self 
		greaseDeprecatedApi: 'WASession>>#newSessionUrl'
		details: 'Use WAApplication>>#url.'.
	^ self application url!

postCopy
	super postCopy.
	continuations := continuations copy!

properties
	"Answer a dictionary of session-related properties."
	^ properties ifNil: [ properties := GRSmallDictionary new ]!

redirect
	self 
		greaseDeprecatedApi: 'WASession>>#redirect'
		details: 'Use WARequestContext>>#redirect (requires the Seaside-Flow package).'.
	^ self requestContext redirect!

redirectResponseFor: aUrlOrString 
	self 
		greaseDeprecatedApi: 'WASession>>#redirectResponseFor:'
		details: 'Use WARequestContext>>#redirectTo:.'.
	self requestContext redirectTo: aUrlOrString!

redirectTo: aUrlOrString 
	self 
		greaseDeprecatedApi: 'WASession>>#redirectTo:'
		details: 'Use WARequestContext>>#redirectTo:.'.
	self requestContext redirectTo: aUrlOrString!

redirectWithCookie: aCookie 
	self 
		greaseDeprecatedApi: 'WASession>>#redirectWithCookie:'
		details: 'Use "self requestContent response addCookie: aCookie. self requestContext redirect" (requires the Seaside-Flow package).'.
	self requestContext response addCookie: aCookie.
	self requestContext redirect!

redirectWithMessage: aString delay: aDuration 
	self 
		greaseDeprecatedApi: 'WASession>>#redirectWithMessage:delay:'
		details: 'Use WARequestContext>>#redirectWithMessage:delay: (requires the Seaside-Flow package).'.
	^ self requestContext 
		redirectWithMessage: aString
		delay: aDuration!

returnResponse: aResponse
	self 
		greaseDeprecatedApi: 'WASession>>#returnResponse:'
		details: 'Use WARequestContext>>#respond:. For example: "self requestContext respond: [:response | ... ]".'.

	"We can't really solve this consistenly."
	self error: 'You can no longer return a new WAResponse instance. You must instead modify the Response contained in the active RequestContext. Look for senders of #respond: and #response for examples.'!

start
	self application mainClass new start!

unknownRequest
	^ self start!

updateRoot: anHtmlRoot!

updateStates: aSnapshot
	self filters do: [ :each | each updateStates: aSnapshot ].
	aSnapshot register: filter!

updateUrl: aUrl!

url
	| url |
	url := super url.
	jumpTo isNil
		ifFalse: [ url fragment: jumpTo ].
	^ url! !
!WASession categoriesFor: #actionField!accessing!public! !
!WASession categoriesFor: #actionUrlForContinuation:!accessing!public! !
!WASession categoriesFor: #actionUrlForKey:!accessing!public! !
!WASession categoriesFor: #addLoadScript:!public! !
!WASession categoriesFor: #application!accessing!public! !
!WASession categoriesFor: #baseUrl!public! !
!WASession categoriesFor: #clearJumpTo!public!scripts! !
!WASession categoriesFor: #closePopup!public! !
!WASession categoriesFor: #closePopupWithoutReloadingOpener!public! !
!WASession categoriesFor: #createCache!initialization!public! !
!WASession categoriesFor: #currentRequest!public! !
!WASession categoriesFor: #expire!public! !
!WASession categoriesFor: #handleFiltered:!handling!public! !
!WASession categoriesFor: #initialize!initialization!public! !
!WASession categoriesFor: #initializeFilters!initialization!public! !
!WASession categoriesFor: #isSession!public!testing! !
!WASession categoriesFor: #jumpToAnchor:!public!scripts! !
!WASession categoriesFor: #key!accessing!public! !
!WASession categoriesFor: #newSessionUrl!public! !
!WASession categoriesFor: #postCopy!copying!public! !
!WASession categoriesFor: #properties!accessing!public! !
!WASession categoriesFor: #redirect!public! !
!WASession categoriesFor: #redirectResponseFor:!public! !
!WASession categoriesFor: #redirectTo:!public! !
!WASession categoriesFor: #redirectWithCookie:!public! !
!WASession categoriesFor: #redirectWithMessage:delay:!public! !
!WASession categoriesFor: #returnResponse:!public! !
!WASession categoriesFor: #start!handling!public! !
!WASession categoriesFor: #unknownRequest!handling!public! !
!WASession categoriesFor: #updateRoot:!public!updating! !
!WASession categoriesFor: #updateStates:!public!updating! !
!WASession categoriesFor: #updateUrl:!public!updating! !
!WASession categoriesFor: #url!accessing!public! !

WAApplication guid: (GUID fromString: '{718E7CF8-4974-4BF6-AA20-911BEAD41300}')!
WAApplication comment: '
WAApplication is the starting point for a Seaside application. When a WAComponent is registered as a top level component a WAApplication object is added to a WADispatcher. The dispatcher forwards all requests to the WAApplication, which in turn forwards them to the correct WASession object. WAApplication''s parent class WARegistry maintains a list of all active sessions to the application. 

"configuration" contains a chain of WAConfituration classes that define attributes of the application. The attribute "rootComponent", for example, defines the top level WAComponent class for the application. The configuration chain includes WAUserConfiguration, WAGlobalConfiguration, WARenderLoopConfiguration and WASessionConfiguration. Other configurations can be added to the chain when the top level application is registered with a dispatcher. (See below)

"libraries" is a collection of WALibrary classes, which are used to serve css, javascript and images used by the application. These may be in methods or in files. Sometimes these libraries are replaced by static files served by Apache. See WAFileLibrary class comment for more information.

Registering an Application.
	An application can be registered with a dispatcher by using the Seaside configuration page or via code. Below MyComponent is a subclass of WAComponent. The following registers the component as an application, gives some values to attributes (or preferences) and adds a library and a configuration. 

MyComponent class>>initialize
	"self initialize"
	| application |
	application := self registerAsApplication: ''sample''.
	application preferenceAt: #sessionClass put: Glorp.WAGlorpSession.
	application addLibrary: SampleLibrary.
	application configuration addAncestor: GlorpConfiguration new.
	application preferenceAt: #glorpDatabasePlatform put: Glorp.PostgreSQLPlatform.
	application preferenceAt: #databaseServer put: ''127.0.0.1''.
	application preferenceAt: #databaseConnectString put: ''glorptests''.

MyComponent>>someInstanceMethod
	"example of how to access attributes (preferences)"
	self session application preferenceAt: #glorpDatabasePlatform'!
!WAApplication categoriesForClass!Unclassified! !
!WAApplication methodsFor!

addLibrary: aLibraryClass
	self preferenceAt: #libraries put: (self libraries copyWith: aLibraryClass)!

contentType
	^ self mimeType seasideMimeType
		charset: self requestContext charSet;
		yourself!

defaultConfiguration
	^ super defaultConfiguration
		removeParent: WARegistryConfiguration instance;
		addParent: WAApplicationConfiguration instance;
		yourself!

doesHandlerSupportCookies: aRequestHandler
	^ aRequestHandler isSession!

handleDefault: aRequestContext
	self handle: aRequestContext registering: self newSession
		!

handleFiltered: aRequestContext 
	"WAApplication instances only understand GET and POST requests."
	
	(self isImplemented: aRequestContext request method) 
		ifTrue: [ super handleFiltered: aRequestContext ]
		ifFalse: [ 
			aRequestContext responseGenerator
				notImplemented;
				respond ]!

isApplication
	^true!

isImplemented: aString
	"Answer whether the receiver implements the given HTTP method."

	^ #('GET' 'POST') includes: aString!

keyField
	^ '_s'!

libraries
	^ self preferenceAt: #libraries!

mainClass
	^ self preferenceAt: #mainClass!

mimeType
	^ self preferenceAt: #mimeType!

newSession
	"Create a new session which inherits our configuration."
	^ self sessionClass new
		configuration: (WAUserConfiguration new addParent: self configuration; yourself);
		yourself!

removeLibrary: aLibraryClass
	self preferenceAt: #libraries put: (self libraries copyWithout: aLibraryClass)!

resourceBaseUrl
	^ self preferenceAt: #resourceBaseUrl!

sessionAllowTermination
	^ self preferenceAt: #sessionAllowTermination!

sessionClass
	^ self preferenceAt: #sessionClass!

sessions
	
	| collection |
	collection := OrderedCollection new.
	self sessionsDo: [:session | collection add: session ].
	^collection

	!

sessionsDo: aBlock

	self handlersDo: [:handler | handler isSession ifTrue: [ aBlock value: handler ]].
! !
!WAApplication categoriesFor: #addLibrary:!libraries!public! !
!WAApplication categoriesFor: #contentType!configuration!public! !
!WAApplication categoriesFor: #defaultConfiguration!configuration!public! !
!WAApplication categoriesFor: #doesHandlerSupportCookies:!private! !
!WAApplication categoriesFor: #handleDefault:!handling!public! !
!WAApplication categoriesFor: #handleFiltered:!handling!public! !
!WAApplication categoriesFor: #isApplication!public!testing! !
!WAApplication categoriesFor: #isImplemented:!private! !
!WAApplication categoriesFor: #keyField!handling-helpers!public! !
!WAApplication categoriesFor: #libraries!configuration!public! !
!WAApplication categoriesFor: #mainClass!configuration!public! !
!WAApplication categoriesFor: #mimeType!configuration!public! !
!WAApplication categoriesFor: #newSession!private! !
!WAApplication categoriesFor: #removeLibrary:!libraries!public! !
!WAApplication categoriesFor: #resourceBaseUrl!configuration!public! !
!WAApplication categoriesFor: #sessionAllowTermination!configuration!public! !
!WAApplication categoriesFor: #sessionClass!configuration!public! !
!WAApplication categoriesFor: #sessions!accessing!public! !
!WAApplication categoriesFor: #sessionsDo:!enumerating!public! !

!WAApplication class methodsFor!

description
	^ 'Application'! !
!WAApplication class categoriesFor: #description!accessing!public! !

"Binary Globals"!

