| package |
package := Package name: 'Seaside-Tests-Development'.
package paxVersion: 1;
	basicComment: ''.

package basicPackageVersion: '35'.


package classNames
	add: #WADebugErrorHandlerTest;
	add: #WADevelopmentPlatformTest;
	add: #WAWalkbackTest;
	yourself.

package methodNames
	add: 'GRPackage class' -> #seasideTestsDevelopment;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Grease\Grease-Core';
	add: 'Seaside-Development';
	add: 'Seaside-Tests-Core';
	add: '..\..\Camp Smalltalk\SUnit\SUnit';
	yourself).

package!

"Class Definitions"!

TestCase subclass: #WADebugErrorHandlerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TestCase subclass: #WADevelopmentPlatformTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TestCase subclass: #WAWalkbackTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!GRPackage class methodsFor!

seasideTestsDevelopment
	^ self new
		name: 'Seaside-Tests-Development';
		description: 'Unit tests for the package Seaside-Development.';
		addDependency: 'Seaside-Development';
		addDependency: 'Seaside-Tests-Tools-Web';
		url: #seasideUrl;
		yourself! !
!GRPackage class categoriesFor: #seasideTestsDevelopment!*seaside-tests-development!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

WADebugErrorHandlerTest guid: (GUID fromString: '{F684ADBE-C3B0-41F7-9024-B175E3AA219E}')!
WADebugErrorHandlerTest comment: '
This isn''t really a unit test of WADebugErrorHandler but rather a test that tries to replicate a simple version of the same error handling behaviour upon which the error handler depends. The error handler itself would be hard to test because it opens a debugger as its first action.

This should at least help confirm that the assumptions that were in place when writing the handler on Squeak hold true on other platforms.'!
!WADebugErrorHandlerTest categoriesForClass!Unclassified! !
!WADebugErrorHandlerTest methodsFor!

handleException: anException debuggerSimulation: debugBlock
	debugBlock value.
	anException resume!

simulateWithDebugBlock: debugBlock progress: desiredProgress result: desiredResult 
	"Set up a known working mechanism similar to the error handler so we can
	test whether the underlying system works the same way as on Squeak
	where WADebugErrorHandler was originally developed."
	| progress semaphore result |
	progress := OrderedCollection new.
	semaphore := GRPlatform current semaphoreClass new.
	
	[ 
	[ result := 
	[ progress add: #a.
	1 / 0.
	progress add: #b.
	#resumed ] 
		on: Error
		do: 
			[ :e | 
			progress add: #c.
			self 
				handleException: e
				debuggerSimulation: 
					[ progress add: #d.
					debugBlock value.
					progress add: #e ] ] ] ensure: [ semaphore signal ] ] fork.
	semaphore wait.
	self assert: progress asArray = desiredProgress asArray.
	self assert: result = desiredResult!

testMechanismAbort
	"Test if things behave as expected when simulating a Debugger being closed
	(in other words, the process being terminated)."
	
	self
		simulateWithDebugBlock: [Processor activeProcess terminate]
		progress: #(a c d)
		result: nil!

testMechanismProceed
	"Test if things behave as expected when simulating a Debugger being proceeded
	(in other words, the process being terminated)."
	
	self
		simulateWithDebugBlock: [nil]
		progress: #(a c d e b)
		result: #resumed! !
!WADebugErrorHandlerTest categoriesFor: #handleException:debuggerSimulation:!private! !
!WADebugErrorHandlerTest categoriesFor: #simulateWithDebugBlock:progress:result:!private! !
!WADebugErrorHandlerTest categoriesFor: #testMechanismAbort!public!tests! !
!WADebugErrorHandlerTest categoriesFor: #testMechanismProceed!public!tests! !

WADevelopmentPlatformTest guid: (GUID fromString: '{FD655E4F-4E95-4A3C-AC0F-0350FC6C5D3E}')!
WADevelopmentPlatformTest comment: ''!
!WADevelopmentPlatformTest categoriesForClass!Unclassified! !
WAWalkbackTest guid: (GUID fromString: '{790AA200-8620-4B92-86DD-8881465DA24E}')!
WAWalkbackTest comment: ''!
!WAWalkbackTest categoriesForClass!Unclassified! !
!WAWalkbackTest methodsFor!

testFrameCount
	| walkback |
	[WATestError signal: 'foo']
		on: WATestError
		do: [ :e | walkback := WAWalkback current exception: e ].

	self assert: walkback frameCount > 0! !
!WAWalkbackTest categoriesFor: #testFrameCount!public!tests! !

"Binary Globals"!

