| package |
package := Package name: 'Seaside-Tests-Email'.
package paxVersion: 1;
	basicComment: ''.

package basicPackageVersion: '9'.


package classNames
	add: #WAEmailMessageTest;
	add: #WAStringEmailBodyTest;
	yourself.

package methodNames
	add: 'GRPackage class' -> #seasideTestsEmail;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Grease\Grease-Core';
	add: 'Seaside-Core';
	add: 'Seaside-Email';
	add: '..\..\Camp Smalltalk\SUnit\SUnit';
	yourself).

package!

"Class Definitions"!

TestCase subclass: #WAEmailMessageTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TestCase subclass: #WAStringEmailBodyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!GRPackage class methodsFor!

seasideTestsEmail
	^ self new
		name: 'Seaside-Tests-Email';
		description: 'Unit tests for the package Seaside-Email.';
		addDependency: 'Seaside-Email';
		addDependency: 'Seaside-Tests-Core';
		url: #seasideUrl;
		yourself! !
!GRPackage class categoriesFor: #seasideTestsEmail!*seaside-tests-email!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

WAEmailMessageTest guid: (GUID fromString: '{D6DB1806-FC9A-440C-831A-F9B0B251B3A1}')!
WAEmailMessageTest comment: ''!
!WAEmailMessageTest categoriesForClass!Unclassified! !
!WAEmailMessageTest methodsFor!

philippeMarschall
	^WAEmailAddress
		address: 'philippe.marschall@gmail.com'
		username: 'Philippe Marschall'!

seasideDev
	^WAEmailAddress
		address: 'seaside-dev@lists.squeakfoundation.org'
		username: 'Seaside Developers Mailinglist'!

testFromToSubjectBody
	| message cr |
	cr := String with: Character cr.
	message := WAEmailMessage
		from: self philippeMarschall
		to: self seasideDev
		subject: 'new email package'.
	message setStringBody: 'Hello' wrap: false.
		
	self assert:
'From: "Philippe Marschall" <philippe.marschall@gmail.com>', cr,
'To: "Seaside Developers Mailinglist" <seaside-dev@lists.squeakfoundation.org>',cr ,
'Subject: new email package', cr, cr,
'Hello' = message plainMessage! !
!WAEmailMessageTest categoriesFor: #philippeMarschall!private! !
!WAEmailMessageTest categoriesFor: #seasideDev!private! !
!WAEmailMessageTest categoriesFor: #testFromToSubjectBody!public!tests! !

WAStringEmailBodyTest guid: (GUID fromString: '{424EBCD7-5068-4A71-A953-15DDD4293246}')!
WAStringEmailBodyTest comment: ''!
!WAStringEmailBodyTest categoriesForClass!Unclassified! !
!WAStringEmailBodyTest methodsFor!

testCrLf
	| cr lf crlf body |
	cr := String with: Character cr.
	lf := String with: Character lf.
	crlf := cr , lf.
	body := WAStringEmailBody string: 'hello' , crlf , 'world'.
	self assert: ('hello' , cr , 'world') = body greaseString!

testSplitIntoLines
	| cr toSplit actual |
	cr := String with: Character cr.
	
	toSplit := cr, cr, 'a', cr, cr.
	actual := WAStringEmailBody splitIntoLines: toSplit.
	self assert: actual size = 5.
	#('' '' 'a' '' '') keysAndValuesDo: [ :index :expected |
		self assert: (actual at: index) = expected ].
	
	toSplit := cr, cr, 'a', cr.
	actual := WAStringEmailBody splitIntoLines: toSplit.
	self assert: actual size = 4.
	#('' '' 'a' '') keysAndValuesDo: [ :index :expected |
		self assert: (actual at: index) = expected ].
	
	toSplit := cr, cr, 'a'.
	actual := WAStringEmailBody splitIntoLines: toSplit.
	self assert: actual size = 3.
	#('' '' 'a') keysAndValuesDo: [ :index :expected |
		self assert: (actual at: index) = expected ].!

testWrapping
	| cr body lf crlf |
	cr := String with: Character cr.
	lf := String with: Character lf.
	crlf := cr , lf.
	body := WAStringEmailBody 
		string: '1111 222 2 33'
		lineLength: 5.
	self assert: ('1111' , cr , '222 2' , cr , '33') = body greaseString.
	body := WAStringEmailBody 
		string: '0123456789'
		lineLength: 5.
	self assert: ('01234' , cr , '56789') = body greaseString.
	body := WAStringEmailBody 
		string: '012345678'
		lineLength: 5.
	self assert: ('01234' , cr , '5678') = body greaseString.
	body := WAStringEmailBody 
		string: '01234567890'
		lineLength: 5.
	self assert: ('01234' , cr , '56789' , cr , '0') = body greaseString.
	body := WAStringEmailBody 
		string: '0123456789' , crlf , crlf , '0123456789'
		lineLength: 5.
	self assert: ('01234' , cr , '56789' , cr , cr , '01234' , cr , '56789') = body greaseString! !
!WAStringEmailBodyTest categoriesFor: #testCrLf!public!tests! !
!WAStringEmailBodyTest categoriesFor: #testSplitIntoLines!public!tests! !
!WAStringEmailBodyTest categoriesFor: #testWrapping!public!tests! !

"Binary Globals"!

