| package |
package := Package name: 'Seaside-Tests-Session'.
package paxVersion: 1;
	basicComment: ''.

package basicPackageVersion: '42'.


package classNames
	add: #WAApplicationEncodingFallbackTest;
	add: #WAApplicationEncodingFromCodecTest;
	add: #WAApplicationKeyHandlingTest;
	add: #WAApplicationTest;
	add: #WACurrentSessionTest;
	add: #WAInteractionTest;
	add: #WASessionCookieTest;
	add: #WATestSession;
	yourself.

package methodNames
	add: 'GRPackage class' -> #seasideTestsSession;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Grease\Grease-Core';
	add: 'Seaside-Core';
	add: 'Seaside-Session';
	add: 'Seaside-Tests-Core';
	add: '..\..\Camp Smalltalk\SUnit\SUnit';
	yourself).

package!

"Class Definitions"!

WASession subclass: #WATestSession
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TestCase subclass: #WACurrentSessionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAContextTest subclass: #WAInteractionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WARegistryTest subclass: #WAApplicationEncodingFallbackTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WARegistryTest subclass: #WAApplicationEncodingFromCodecTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WARegistryTest subclass: #WAApplicationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WARegistryTest subclass: #WASessionCookieTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WARegistryKeyHandlingTest subclass: #WAApplicationKeyHandlingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!GRPackage class methodsFor!

seasideTestsSession
	^ self new
		name: 'Seaside-Tests-Session';
		description: 'Unit tests for the package Seaside-Session.';
		addDependency: 'Seaside-Session';
		addDependency: 'Seaside-Tests-Core';
		url: #seasideUrl;
		yourself! !
!GRPackage class categoriesFor: #seasideTestsSession!*seaside-tests-session!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

WATestSession guid: (GUID fromString: '{DDC1DFF5-A871-41EC-98DD-361C452A925D}')!
WATestSession comment: ''!
!WATestSession categoriesForClass!Unclassified! !
!WATestSession methodsFor!

start
	(self requestContext response)
		nextPutAll: 'start: ';
		nextPutAll: self key! !
!WATestSession categoriesFor: #start!handling!public! !

WACurrentSessionTest guid: (GUID fromString: '{49BA7F4E-151F-45B6-97C5-6982037F7B23}')!
WACurrentSessionTest comment: '
A WACurrentSessionTest is to make sure

WACurrentSession value

works as in Seaside 2.9 as it did in earlier versions.
'!
!WACurrentSessionTest categoriesForClass!Unclassified! !
!WACurrentSessionTest methodsFor!

testWithoutContext
	self assert: WACurrentSession value isNil!

testWithoutSession
	| context |
	context := WARequestContext request: nil response: nil.
	WACurrentRequestContext
		use: context
		during: [ self assert: WACurrentSession value isNil ]!

testWithValue
	| session context |
	session := WASession new.
	context := WARequestContext request: nil response: nil.
	context
		push: session
		while: [ self assert: WACurrentSession value = session ]! !
!WACurrentSessionTest categoriesFor: #testWithoutContext!public!tests! !
!WACurrentSessionTest categoriesFor: #testWithoutSession!public!tests! !
!WACurrentSessionTest categoriesFor: #testWithValue!public!tests! !

WAInteractionTest guid: (GUID fromString: '{761B11C0-502C-42EB-9F9E-C74E5A91A965}')!
WAInteractionTest comment: 'The purpose of WAInteractionTest is to provide a fully initialized WContext all the neccsary objects are are normally there in a Seaside application like request, response, dispatcher applicaion, session and so forth.'!
!WAInteractionTest categoriesForClass!Unclassified! !
!WAInteractionTest methodsFor!

application
	^self requestContext application!

createHandlers
	| dispatcher application session root |
	root := WADispatcher new.
	dispatcher := root register: WADispatcher new at: 'rails'.
	application := dispatcher register: WAApplication new at: 'homepage'.
	session := WASession new.
	application register: session.

	^super createHandlers, (Array with: root with: dispatcher with: application with: session)!

session
	^self requestContext session!

testDeleteNetscapeCookie
	| requestCookie responseCookies responseCookie |
	self application preferenceAt: #serverPath put: '/en/private_kunden/wealth_management'.
	requestCookie := WARequestCookie key: 'password' value: 'sex'.
	self requestContextWithUrl: '/' cookies: (Array with: requestCookie).
	self requestContext response deleteCookie: requestCookie.
	
	responseCookies := self requestContext response cookies.
	self assert: responseCookies size = 1.
	responseCookie := responseCookies any.
	
	self assert: (responseCookie isKindOf: WACookie).
	self assert: responseCookie value = ''.
	self assert: responseCookie path = self application url pathString!

testDeleteRfc2965Cookie
	| requestCookie responseCookies responseCookie |
	requestCookie := (WARequestCookie key: 'password' value: 'sex')
		path: '/en/private_kunden/wealth_management';
		version: 1;
		yourself.
	self requestContextWithUrl: '/' cookies: (Array with: requestCookie).
	self requestContext response deleteCookie: requestCookie.
	
	responseCookies := self requestContext response cookies.
	self assert: responseCookies size = 1.
	responseCookie := responseCookies any.
	
	self assert: (responseCookie isKindOf: WACookie).
	self assert: responseCookie value = ''.
	self assert: responseCookie path = '/en/private_kunden/wealth_management'!

testNewCookie
	| cookie |
	cookie := self requestContext newCookie.
	self assert: cookie path = self application url pathString.
	self assert: cookie ports isNil.
	self assert: cookie httpOnly.
	self deny: cookie secure.
	self assert: cookie domain isNil.
	
	self application preferenceAt: #serverPath put: '/en/private_kunden/wealth_management'.
	cookie := self requestContext newCookie.
	self assert: cookie path = '/en/private_kunden/wealth_management'.
	self assert: cookie ports isNil.
	self assert: cookie httpOnly.
	self deny: cookie secure.
	self assert: cookie domain isNil.
	
	self application preferenceAt: #serverPort put: 8080.
	self application preferenceAt: #serverProtocol put: 'https'.
	self application preferenceAt: #serverHostname put: 'www.lgt.com'.
	cookie := self requestContext newCookie.
	self assert: cookie path = '/en/private_kunden/wealth_management'.
	self deny: cookie ports isNil.
	self assert: cookie ports size = 1.
	self assert: cookie ports first = 8080.
	self assert: cookie httpOnly.
	self assert: cookie secure.
	self assert: cookie domain = 'www.lgt.com'! !
!WAInteractionTest categoriesFor: #application!accessing!public! !
!WAInteractionTest categoriesFor: #createHandlers!private! !
!WAInteractionTest categoriesFor: #session!accessing!public! !
!WAInteractionTest categoriesFor: #testDeleteNetscapeCookie!public!tests! !
!WAInteractionTest categoriesFor: #testDeleteRfc2965Cookie!public!tests! !
!WAInteractionTest categoriesFor: #testNewCookie!public!tests! !

WAApplicationEncodingFallbackTest guid: (GUID fromString: '{76F1B6D5-AB41-48C9-9A97-70829DC2FAD4}')!
WAApplicationEncodingFallbackTest comment: ''!
!WAApplicationEncodingFallbackTest categoriesForClass!Unclassified! !
!WAApplicationEncodingFallbackTest methodsFor!

newHandler
	^ WAApplication new
		preferenceAt: #charSet put: 'utf-16';
		yourself!

testContentType
	self assert: handler contentType = ('text/html' seasideMimeType
		charset: 'utf-16';
		yourself)! !
!WAApplicationEncodingFallbackTest categoriesFor: #newHandler!public!running! !
!WAApplicationEncodingFallbackTest categoriesFor: #testContentType!public!testing! !

WAApplicationEncodingFromCodecTest guid: (GUID fromString: '{3DE5E761-8E2D-42D6-ADF1-529AD973DC54}')!
WAApplicationEncodingFromCodecTest comment: ''!
!WAApplicationEncodingFromCodecTest categoriesForClass!Unclassified! !
!WAApplicationEncodingFromCodecTest methodsFor!

createCodec
	^ GRCodec forEncoding: 'iso-8859-1'!

newHandler
	^ WAApplication new
		preferenceAt: #charSet put: 'utf-16';
		yourself!

testContentType
	self assert: handler contentType = ('text/html' seasideMimeType
		charset: 'iso-8859-1';
		yourself)! !
!WAApplicationEncodingFromCodecTest categoriesFor: #createCodec!configuration!public! !
!WAApplicationEncodingFromCodecTest categoriesFor: #newHandler!public!running! !
!WAApplicationEncodingFromCodecTest categoriesFor: #testContentType!public!testing! !

WAApplicationTest guid: (GUID fromString: '{8FA16E54-40F6-42B8-9984-37ABF07B3F35}')!
WAApplicationTest comment: ''!
!WAApplicationTest categoriesForClass!Unclassified! !
!WAApplicationTest methodsFor!

newHandler
	^ WAApplication new!

requestWithMethod: aString
	^ WARequest
		method: aString
		uri: '/app/oneo'!

testNotImplemented
	| context request |
	#('OPTIONS' 'HEAD' 'PUT' 'DELETE' 'TRACE' 'CONNECT') do: [ :each |
		request := self requestWithMethod: each.
		context := WARequestContext request: request response: WABufferedResponse new.
		[ handler handle: context ]
			on: WAResponseNotification
			do: [ :notification | ].
		self assert: context response status = 501 ].
	
	#('GET' 'POST') do: [ :each |
		request := self requestWithMethod: each.
		context := WARequestContext request: request response: WABufferedResponse new.
		[ handler handle: context ]
			on: WAResponseNotification
			do: [ :notification | ].
		self assert: context response status ~= 501 ]!

testPathConsumption
	| context request |
	request := self requestWithMethod: 'GET'.
	context := WARequestContext request: request response: WABufferedResponse new.
	context consumer next.
	self assert: context consumer peek = 'oneo'.
	[ handler handle: context ]
		on: WAResponseNotification
		do: [ :notification | ].
	self assert: context consumer peek = 'oneo'! !
!WAApplicationTest categoriesFor: #newHandler!public!running! !
!WAApplicationTest categoriesFor: #requestWithMethod:!public!running! !
!WAApplicationTest categoriesFor: #testNotImplemented!public!tests! !
!WAApplicationTest categoriesFor: #testPathConsumption!public!tests! !

!WAApplicationTest class methodsFor!

shouldInheritSelectors
	^ true! !
!WAApplicationTest class categoriesFor: #shouldInheritSelectors!public!testing! !

WASessionCookieTest guid: (GUID fromString: '{AF48D0EC-1F0A-4D41-8546-0899C5A45FF2}')!
WASessionCookieTest comment: ''!
!WASessionCookieTest categoriesForClass!Unclassified! !
!WASessionCookieTest methodsFor!

newHandler
	^ WAApplication new
		preferenceAt: #useCookies put: true;
		preferenceAt: #serverHostname put: 'www.seaside.st';
		yourself!

requestWithMethod: aString
	^ WARequest
		method: aString
		uri: '/'!

testDeleteSessionCookieDomain
	| context request response cookie |
	request := self requestWithMethod: 'GET'.
	request setCookies: (Array with: (WARequestCookie key: '_s' value: 'goner')).
	response := WABufferedResponse new.
	context := WARequestContext request: request response: response.
	[ handler handle: context ]
		on: WAResponseNotification
		do: [ :notification | ].
	self assert: response status = 302. "session gone"
	self assert: response cookies size = 1.
	cookie := response cookies first.
	self assert: cookie domain = 'www.seaside.st'!

testNewSessionCookieDomain
	| context request response cookie |
	request := self requestWithMethod: 'GET'.
	response := WABufferedResponse new.
	context := WARequestContext request: request response: response.
	[ handler handle: context ]
		on: WAResponseNotification
		do: [ :notification | ].
	self assert: response cookies size = 1.
	cookie := response cookies first.
	self assert: cookie domain = 'www.seaside.st'!

testUrlFor
	| context request response documentHandler url |
	request := self requestWithMethod: 'GET'.
	request setCookies: (Array with: (WARequestCookie key: '_s' value: 'sessionid')).
	response := WABufferedResponse new.
	documentHandler := (WADocumentHandler
		document: 'body: {color: red}'
		mimeType: WAMimeType textCss
		fileName: nil).
	
	context := WARequestContext request: request response: response.
	url := WACurrentRequestContext
		use: context
		during: [ handler register: documentHandler;
			urlFor: documentHandler ].
	self assert: (url queryFields includesKey: '_s').
	request url addField: '_s' value: (url queryFields at: '_s').
		
	[  handler handle: context ]
		on: WAResponseNotification
		do: [ :notification | ].
		
	self assert: response status = 200.
	self assert: (response cookies isEmpty)! !
!WASessionCookieTest categoriesFor: #newHandler!public!running! !
!WASessionCookieTest categoriesFor: #requestWithMethod:!public!running! !
!WASessionCookieTest categoriesFor: #testDeleteSessionCookieDomain!public!tests! !
!WASessionCookieTest categoriesFor: #testNewSessionCookieDomain!public!tests! !
!WASessionCookieTest categoriesFor: #testUrlFor!public!tests! !

!WASessionCookieTest class methodsFor!

shouldInheritSelectors
	^ true! !
!WASessionCookieTest class categoriesFor: #shouldInheritSelectors!public!testing! !

WAApplicationKeyHandlingTest guid: (GUID fromString: '{96E8994D-AB07-461F-9621-F045B8009277}')!
WAApplicationKeyHandlingTest comment: ''!
!WAApplicationKeyHandlingTest categoriesForClass!Unclassified! !
!WAApplicationKeyHandlingTest methodsFor!

assertDefault: aResponse
	"Instead of returning 404 on a default request as WARegistry does, WAApplication creates a new
	Session and dispatches to it."
	self assert: registry cache size = (registrySize + 1).
	self assert: aResponse status = 200.
	self assert: (aResponse contents beginsWithSubCollection: 'start:')!

createRegistry
	^ WADispatcher new
		register: (WAApplication new
			preferenceAt: #sessionClass put: WATestSession;
			yourself)
		at: self registryName!

fieldName
	^ '_s'!

registerNewHandler
	^ registry register: WATestNoopSessionHandler new!

testNoKey
	"There is no URL field and no cookie. This is a new request to the Registry.
	
	Instead of returning a 404, like WARegistry does, WAApplication creates a new WASession
	and also adds the cookie."
	
	| response context |
	registry := self createRegistryWithCookiesEnabled.
	context := self createRequestContextWithFieldKey: nil cookieKey: nil.
	response := self dispatch: context to: registry.
	
	self assertCookieAddition: response.
	self assert: ((registry cache at: response cookies first value) isKindOf: WASession).
	self assertDefault: response! !
!WAApplicationKeyHandlingTest categoriesFor: #assertDefault:!private! !
!WAApplicationKeyHandlingTest categoriesFor: #createRegistry!private! !
!WAApplicationKeyHandlingTest categoriesFor: #fieldName!private! !
!WAApplicationKeyHandlingTest categoriesFor: #registerNewHandler!private! !
!WAApplicationKeyHandlingTest categoriesFor: #testNoKey!public!tests! !

!WAApplicationKeyHandlingTest class methodsFor!

shouldInheritSelectors
	^ true! !
!WAApplicationKeyHandlingTest class categoriesFor: #shouldInheritSelectors!public!testing! !

"Binary Globals"!

