| package |
package := Package name: 'Seaside-Tests-UTF8'.
package paxVersion: 1;
	basicComment: ''.

package basicPackageVersion: '9'.


package classNames
	add: #WAUtf8FileTest;
	add: #WAUtf8ResponseTest;
	add: #WAUtf8UrlTest;
	yourself.

package methodNames
	add: 'GRPackage class' -> #seasideTestsUtf8;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Grease\Grease-Core';
	add: 'Seaside-Core';
	add: 'Seaside-Tests-Core';
	add: '..\..\Camp Smalltalk\SUnit\SUnit';
	yourself).

package!

"Class Definitions"!

TestCase subclass: #WAUtf8FileTest
	instanceVariableNames: 'file'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAContextTest subclass: #WAUtf8ResponseTest
	instanceVariableNames: 'response'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
WAContextTest subclass: #WAUtf8UrlTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!GRPackage class methodsFor!

seasideTestsUtf8
	^ self new
		name: 'Seaside-Tests-UTF8';
		description: 'Unit tests for UTF8 support.';
		addDependency: 'Seaside-Tests-Core';
		url: #seasideUrl;
		yourself! !
!GRPackage class categoriesFor: #seasideTestsUtf8!*seaside-tests-utf8!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

WAUtf8FileTest guid: (GUID fromString: '{C97EBBF3-6DD8-4E6E-8F3A-EF142B685035}')!
WAUtf8FileTest comment: ''!
!WAUtf8FileTest categoriesForClass!Unclassified! !
!WAUtf8FileTest methodsFor!

asString: aCollectionOfIntegers
	^String streamContents: [ :stream |
		aCollectionOfIntegers do: [ :each |
			stream nextPut: (Character codePoint: each) ] ]!

brokenUnicodeString
	^ self asString: #(83 195 169 195 164 115 195 174 100 195 169)!

setUp
	file := WAFile new!

testContents
	file contents: #(83 195 169 195 164 115 195 174 100 195 169) asByteArray.
	self assert: file rawContents = #(83 195 169 195 164 115 195 174 100 195 169) asByteArray.
	self assert: file contents = self brokenUnicodeString.
	self assert: (file contentsDecodedUsing: 'UTF-8') = self unicodeString!

testContentsDecoded
	file contents:  #(83 195 169 195 164 115 195 174 100 195 169) asByteArray.
	self should: [ file contentsDecoded ] raise: WAIllegalStateException.
	file contentType: 'text/plain'.
	self should: [ file contentsDecoded ] raise: WAIllegalStateException.
	file contentType: 'text/plain; charset=utf-8' seasideMimeType.
	self assert: file contentsDecoded = self unicodeString
	!

unicodeString
	^ self asString: #(83 233 228 115 238 100 233)! !
!WAUtf8FileTest categoriesFor: #asString:!private! !
!WAUtf8FileTest categoriesFor: #brokenUnicodeString!private! !
!WAUtf8FileTest categoriesFor: #setUp!public!running! !
!WAUtf8FileTest categoriesFor: #testContents!public!tests! !
!WAUtf8FileTest categoriesFor: #testContentsDecoded!public!tests! !
!WAUtf8FileTest categoriesFor: #unicodeString!private! !

WAUtf8ResponseTest guid: (GUID fromString: '{ADCE8F78-C2D8-4666-BC75-86383E5D31DA}')!
WAUtf8ResponseTest comment: ''!
!WAUtf8ResponseTest categoriesForClass!Unclassified! !
!WAUtf8ResponseTest methodsFor!

assertLines: anArray
	self assert: self lines = anArray!

asString: aCollectionOfIntegers
	^String streamContents: [ :stream |
		aCollectionOfIntegers do: [ :each |
			stream nextPut: (Character codePoint: each) ] ]!

contents
	| stream |
	stream := WriteStream on: String new.
	self response writeOn: stream.
	^ stream contents!

createCodec
	^ GRCodec forEncoding: 'utf-8'!

lines
	"Answer the response as an array of lines, to be used within the tests."
	| result stream |
	result := OrderedCollection new.
	stream := self contents readStream.
	[ stream atEnd ] whileFalse: 
		[ result add: (stream greaseUpToAll: (String 
					with: Character cr
					with: Character lf)) ].
	^ result asArray!

response
	^ response ifNil: [ response := WABufferedResponse new ]!

testRedirectTo
	| url |
	url :=  WAUrl absolute: 'http://www.seaside.st/'.
	url addToPath: self unicodeString.
	self response redirectTo: url.
	self assertLines:  #('HTTP/1.1 302 Found'
		'Location: http://www.seaside.st/%C2%A5%C2%A3%E2%82%AC%24%C2%A2%E2%82%A1%E2%82%A2%E2%82%A3%E2%82%A4%E2%82%A5%E2%82%A6%E2%82%A7%E2%82%A8%E2%82%A9%E2%82%AA%E2%82%AB%E2%82%AD%E2%82%AE%E2%82%AF'
		'Content-Length: 0' '')!

unicodeString
	^ self asString: #(165 163 8364 36 162 8353 8354 8355 8356 8357 8358 8359 8360 8361 8362 8363 8365 8366 8367)! !
!WAUtf8ResponseTest categoriesFor: #assertLines:!private! !
!WAUtf8ResponseTest categoriesFor: #asString:!private! !
!WAUtf8ResponseTest categoriesFor: #contents!private! !
!WAUtf8ResponseTest categoriesFor: #createCodec!configuration!public! !
!WAUtf8ResponseTest categoriesFor: #lines!private! !
!WAUtf8ResponseTest categoriesFor: #response!private! !
!WAUtf8ResponseTest categoriesFor: #testRedirectTo!public!tests! !
!WAUtf8ResponseTest categoriesFor: #unicodeString!private! !

WAUtf8UrlTest guid: (GUID fromString: '{04124958-B90C-45F3-BB5E-FF3619DF7D37}')!
WAUtf8UrlTest comment: ''!
!WAUtf8UrlTest categoriesForClass!Unclassified! !
!WAUtf8UrlTest methodsFor!

asString: aCollectionOfIntegers
	^String streamContents: [ :stream |
		aCollectionOfIntegers do: [ :each |
			stream nextPut: (Character codePoint: each) ] ]!

createCodec
	^ GRCodec forEncoding: 'utf-8'!

testPrintString
	| url |
	url :=  WAUrl absolute: 'http://www.seaside.st/'.
	url addToPath: self unicodeString.
	self assert: url printString = 'http://www.seaside.st/%C2%A5%C2%A3%E2%82%AC%24%C2%A2%E2%82%A1%E2%82%A2%E2%82%A3%E2%82%A4%E2%82%A5%E2%82%A6%E2%82%A7%E2%82%A8%E2%82%A9%E2%82%AA%E2%82%AB%E2%82%AD%E2%82%AE%E2%82%AF'!

unicodeString
	^ self asString: #(165 163 8364 36 162 8353 8354 8355 8356 8357 8358 8359 8360 8361 8362 8363 8365 8366 8367)! !
!WAUtf8UrlTest categoriesFor: #asString:!private! !
!WAUtf8UrlTest categoriesFor: #createCodec!configuration!public! !
!WAUtf8UrlTest categoriesFor: #testPrintString!public!tests! !
!WAUtf8UrlTest categoriesFor: #unicodeString!private! !

"Binary Globals"!

