| package |
package := Package name: 'Seaside-Tools-Core'.
package paxVersion: 1;
	basicComment: ''.

package basicPackageVersion: '18'.


package classNames
	add: #WAAdmin;
	yourself.

package methodNames
	add: 'GRPackage class' -> #seasideToolsCore;
	add: 'WAComponent class' -> #registerAsApplication:;
	add: 'WARegistry class' -> #clearAllHandlers;
	add: 'WARequestHandler class' -> #register:;
	add: 'WARequestHandler class' -> #register:in:;
	add: 'WAServerAdaptor class' -> #default;
	add: 'WAServerAdaptor class' -> #start;
	add: 'WAServerAdaptor class' -> #startOn:;
	add: 'WAServerAdaptor class' -> #stop;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\Grease\Grease-Core';
	add: 'Seaside-Component';
	add: 'Seaside-Core';
	add: 'Seaside-RenderLoop';
	add: 'Seaside-Session';
	yourself).

package!

"Class Definitions"!

WAObject subclass: #WAAdmin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: 'configuredApplicationDefaults configuredExceptionHandlingDefaults'!

"Global Aliases"!


"Loose Methods"!

!GRPackage class methodsFor!

seasideToolsCore
	^ self new
		name: 'Seaside-Tools-Core';
		description: 'Administrative tools, useful to have for production and development environments.';
		addDependency: 'Seaside-Core';
		addDependency: 'Seaside-Component'; "only for deprecated class-side method extension"
		addDependency: 'Seaside-RenderLoop'; "for class references in WAAdmin"
		addDependency: 'Seaside-Session'; "for class reference in WAAdmin"
		url: #seasideUrl;
		yourself! !
!GRPackage class categoriesFor: #seasideToolsCore!*seaside-tools-core!public! !

!WAComponent class methodsFor!

registerAsApplication: aString 
	self 
		greaseDeprecatedApi: 'WAComponent class>>#registerAsApplication:'
		details: 'Use WAAdmin class>>#register:asApplicationAt:.'.
	^ WAAdmin 
		register: self
		asApplicationAt: aString! !
!WAComponent class categoriesFor: #registerAsApplication:!*seaside-tools-core!public! !

!WARegistry class methodsFor!

clearAllHandlers
	self 
		greaseDeprecatedApi: 'WARegistry class>>#clearAllHandlers'
		details: 'Use WAAdmin class>>#clearSessions instead.'.
	^ WAAdmin clearSessions! !
!WARegistry class categoriesFor: #clearAllHandlers!*seaside-tools-core!public! !

!WARequestHandler class methodsFor!

register: aString 
	self 
		greaseDeprecatedApi: 'WARequestHandler class>>#register:'
		details: 'Use WAAdmin class>>#register:at:.'.
	^ WAAdmin 
		register: self
		at: aString!

register: aString in: aDispatcher 
	self 
		greaseDeprecatedApi: 'WARequestHandler class>>#register:in:'
		details: 'Use WAAdmin class>>#register:at:in:.'.
	^ WAAdmin 
		register: self
		at: aString
		in: aDispatcher! !
!WARequestHandler class categoriesFor: #register:!*seaside-tools-core!public! !
!WARequestHandler class categoriesFor: #register:in:!*seaside-tools-core!public! !

!WAServerAdaptor class methodsFor!

default
	"Helper method that returns a single adaptor of this class from the default ServerManager.
	If there are no adaptors of this class registered, return nil. If there are multiple adaptors
	of this type, signal an error."
	
	| adaptors |
	adaptors := WAAdmin serverAdaptors select: [ :each | each isKindOf: self ].
	(adaptors size > 1) ifTrue: [ self error: 'You cannot use this convenience method when more than one adaptor of the same type are registered. Use WAServerManager or a graphical interface when dealing with multiple adaptors of the same type.'].
	^ adaptors isEmpty
		ifTrue: [ nil ]
		ifFalse: [ adaptors first ]!

start
	"Convenience method to start a single already registered adaptor of the type of the
	receiver."

	self default start!

startOn: portInteger
	"Convenience method to start a single adaptor on the specified port. If an existing
	adaptor of this type is already registered, it will be restarted on the specified port."

	| adaptor |
	adaptor := self default.
	adaptor isNil
		ifFalse: [ adaptor stop; port: portInteger ]
		ifTrue: [ adaptor := self port: portInteger ].
	adaptor start!

stop
	"Convenience method to stop a currently running adaptor of the type of the receiver
	as long as there is only one such adaptor currently registered."

	| default |
	default := self default.
	default isNil ifFalse: [ default stop ]! !
!WAServerAdaptor class categoriesFor: #default!*seaside-tools-core!public! !
!WAServerAdaptor class categoriesFor: #start!*seaside-tools-core!public! !
!WAServerAdaptor class categoriesFor: #startOn:!*seaside-tools-core!public! !
!WAServerAdaptor class categoriesFor: #stop!*seaside-tools-core!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

WAAdmin guid: (GUID fromString: '{66EBEC59-D0F8-4544-8444-2A8BCABCB56F}')!
WAAdmin comment: ''!
!WAAdmin categoriesForClass!Unclassified! !
!WAAdmin class methodsFor!

applicationDefaults
	| config |
	config := WASharedConfiguration named: 'Application Defaults'.
	(configuredApplicationDefaults ifNil: [ false ]) ifFalse: [
		self configureApplicationDefaults: config.
		configuredApplicationDefaults := true ].
	^ config!

applicationExceptionHandlingDefaults
	| config |
	config := WASharedConfiguration named: 'Application Exception Handling Defaults'.
	(configuredExceptionHandlingDefaults ifNil: [ false ]) ifFalse: [
		self configureApplicationExceptionHandlingDefaults: config.
		configuredExceptionHandlingDefaults := true ].
	^ config!

clearAll
	self clearDefaultDispatcher.
	self clearConfigurationCaches.
	self clearSessions.!

clearConfigurationCaches
	WASystemConfiguration clearAllDescriptions!

clearDefaultDispatcher
	WADispatcher resetDefault!

clearSessions
	self defaultServerManager adaptors do: [ :adaptor |
		adaptor requestHandler withDescendantsDo: [ :handler |
			handler isApplication ifTrue: [ handler clear ] ] ]!

configureApplicationDefaults: aSharedConfiguration
	"WARenderLoopConfiguration is added by default because most users will use WASession and the Render Loop."
	(aSharedConfiguration parents includes: WARenderLoopConfiguration instance) ifFalse: [
		aSharedConfiguration addParent: WARenderLoopConfiguration instance ]!

configureApplicationExceptionHandlingDefaults: aSharedConfiguration
	(aSharedConfiguration parents includes: WAExceptionFilterConfiguration instance) ifFalse: [
		aSharedConfiguration addParent: WAExceptionFilterConfiguration instance ]!

configureNewApplication: anApplication
	| filter |
	anApplication configuration
		removeParent: WAApplicationConfiguration instance; "make the chain slightly cleaner"
		addParent: WARenderLoopConfiguration instance;
		addParent: self applicationDefaults.
		
	filter := WAExceptionFilter new.
	filter configuration addParent: self applicationExceptionHandlingDefaults.
	anApplication addFilter: filter!

defaultDispatcher
	^ WADispatcher default!

defaultServerManager
	"Return the default server manager"
	
	^ WAServerManager default!

register: aComponentClass asApplicationAt: aString
	"Use this to programatically register a component as an application."
	| application |
	application := self register: WAApplication at: aString.
	self configureNewApplication: application.
	application preferenceAt: #rootClass put: aComponentClass.
	^ application!

register: aComponentClass asApplicationAt: aString user: user password: password 
	"Like #register:asApplicationAt: but additionally adds password protection."
	| application |
	application := self 
		register: aComponentClass
		asApplicationAt: aString.
	application configuration addParent: WAAuthConfiguration instance.
	application 
		preferenceAt: #login
		put: user.
	application 
		preferenceAt: #passwordHash
		put: (GRPlatform current secureHashFor: password).
	application addFilter: WAAuthenticationFilter new.
	^ application!

register: anEntryPointClass at: aString
	^ self register: anEntryPointClass at: aString in: self defaultDispatcher!

register: aRequestHandlerClass at: aString in: aDispatcher
	| path dispatcher |
	path := aString greaseSubStrings: '/'.
	dispatcher := path allButLast
		inject: aDispatcher
		into: [ :result :each |
			result handlers 
				at: each
				ifAbsent: [ result register: WADispatcher new at: each ] ].
	^ dispatcher register: aRequestHandlerClass new at: path last!

serverAdaptors
	"Return a collection of Server Adaptors that are currently registered in the default Server Manager."
	^ self defaultServerManager adaptors!

unregister: aString
	"Unregister the application aString in the default dispatcher."

	^ self unregister: aString in: self defaultDispatcher!

unregister: aString in: aDispatcher
	"Unregister the application aString in aDispatcher."

	| path dispatcher |
	path := aString greaseSubStrings: '/'.
	dispatcher := path
		inject: aDispatcher
		into: [ :result :each |
			result handlers
				at: each
				ifAbsent: [ ^ nil ] ].
	^ dispatcher parent
		unregister: dispatcher! !
!WAAdmin class categoriesFor: #applicationDefaults!configuration!public! !
!WAAdmin class categoriesFor: #applicationExceptionHandlingDefaults!configuration!public! !
!WAAdmin class categoriesFor: #clearAll!development!public! !
!WAAdmin class categoriesFor: #clearConfigurationCaches!development!public! !
!WAAdmin class categoriesFor: #clearDefaultDispatcher!development!public! !
!WAAdmin class categoriesFor: #clearSessions!development!public! !
!WAAdmin class categoriesFor: #configureApplicationDefaults:!initialization!public! !
!WAAdmin class categoriesFor: #configureApplicationExceptionHandlingDefaults:!initialization!public! !
!WAAdmin class categoriesFor: #configureNewApplication:!public!registration! !
!WAAdmin class categoriesFor: #defaultDispatcher!configuration!public! !
!WAAdmin class categoriesFor: #defaultServerManager!public!server adaptors! !
!WAAdmin class categoriesFor: #register:asApplicationAt:!public!registration! !
!WAAdmin class categoriesFor: #register:asApplicationAt:user:password:!public!registration! !
!WAAdmin class categoriesFor: #register:at:!public!registration! !
!WAAdmin class categoriesFor: #register:at:in:!public!registration! !
!WAAdmin class categoriesFor: #serverAdaptors!public!server adaptors! !
!WAAdmin class categoriesFor: #unregister:!public!registration! !
!WAAdmin class categoriesFor: #unregister:in:!public!registration! !

"Binary Globals"!

