| package |
package := Package name: 'Sport Tests'.
package paxVersion: 1;
	basicComment: 'Sport is a Smalltalk Portability library.
All Sport implementations are available under the LGPL.

This package includes the tests for Sport on Dolphin X6 platform.

Sport provides a consistent interface to things which are implemented differently in most Smalltalk dialects. Sport provides wrappers for:
    * Exceptions
    * Times & Dates
    * Files
    * Sockets
     ... and more.

So far, Sport is implemented in: 
 - VisualWorks
 - Gemstone
 - Squeak
 - Visual Age for Smalltalk
 - Dolphin Smalltalk X6

Website: http://wiki.cs.uiuc.edu/VisualWorks/Sport

===============
Dolphin X6 port by: Esteban A. Maringolo <eMaringolo at gmail.com>

Note: This version of Sport has been tested only with the Swazoo version that runs on it (1.1.4).
It may include methods not yet implemented, and other unknown errors. 
If you fix some, please report them to my mail address. Thank you.

================
'.

package basicPackageVersion: '2.031.jsc'.


package classNames
	add: #SpDateTest;
	add: #SpEnvironmentTest;
	add: #SpFilenameTest;
	add: #SpMiscellaneous;
	add: #SpSocketTest;
	add: #SpStringUtilitiesTest;
	add: #SpTestCase;
	add: #SpTimestampTest;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\..\Object Arts\Dolphin\Sockets\Dolphin Sockets';
	add: 'Sport';
	add: '..\..\Camp Smalltalk\SUnit\SUnit';
	yourself).

package!

"Class Definitions"!

TestCase subclass: #SpTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SpTestCase subclass: #SpDateTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SpTestCase subclass: #SpEnvironmentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SpTestCase subclass: #SpFilenameTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SpTestCase subclass: #SpMiscellaneous
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SpTestCase subclass: #SpSocketTest
	instanceVariableNames: 'socket serverSocket newSpServerSocket semaphore acceptProcess'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SpTestCase subclass: #SpStringUtilitiesTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SpTestCase subclass: #SpTimestampTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

"End of package definition"!

"Source Globals"!

"Classes"!

SpTestCase guid: (GUID fromString: '{9517F2A9-D6F3-4720-A62D-5C9D3F0EE98A}')!
SpTestCase comment: ''!
!SpTestCase categoriesForClass!Unclassified! !
SpDateTest guid: (GUID fromString: '{54810B8A-8296-4918-BB7E-48551BD69853}')!
SpDateTest comment: ''!
!SpDateTest categoriesForClass!Unclassified! !
!SpDateTest methodsFor!

testDayAddition

	| date1 date2 |
	date1 := Date fromString: '17/05/1982' format: 'dd/mm/yyyy'.
	date2 := Date fromString: '01/06/1983' format: 'dd/mm/yyyy'.
	self assert: ((SpDate onDate: date1) addDays: 380) = (SpDate onDate: date2).
	self assert: ((SpDate onDate: date2) addDays: -380) = (SpDate onDate: date1)!

testIntantiationOnDate

	self assert: (SpDate onDate: Date today) underlyingDate = Date today!

testIntantiationOnToday

	self assert: (SpDate today) underlyingDate = Date today! !
!SpDateTest categoriesFor: #testDayAddition!public! !
!SpDateTest categoriesFor: #testIntantiationOnDate!public! !
!SpDateTest categoriesFor: #testIntantiationOnToday!public! !

SpEnvironmentTest guid: (GUID fromString: '{A9956FB0-2FE2-41BB-BDC1-8A93A9394326}')!
SpEnvironmentTest comment: ''!
!SpEnvironmentTest categoriesForClass!Unclassified! !
!SpEnvironmentTest methodsFor!

testAllSubclassesOfAClass

	self assert: (SpEnvironment allSubclassesOf: Magnitude) = Magnitude allSubclasses asArray!

testBase64DecodeAsString
	
	self assert: (SpEnvironment base64DecodeAsString: 'SEFQUFkgQU5EIFJFQUw=') = 'HAPPY AND REAL'!

testBase64EncodeString
	
	self assert: (SpEnvironment base64EncodeString: 'HAPPY AND REAL') = 'SEFQUFkgQU5EIFJFQUw='!

testCharacterFromInteger
	
	self assert: (SpEnvironment characterFromInteger: 101) = $e!

testCompilerEvaluation

	self assert: (SpEnvironment evaluate: 'nil class' in: nil) = UndefinedObject.
	self assert: (SpEnvironment evaluate: '[:x | x + 4 ] value: 3' in: nil) = 7!

testDolphinEnvironment

	self assert: SpEnvironment isDolphin.
	self deny: SpEnvironment isVisualWorks.
	self deny: SpEnvironment isGemStone!

testIntegerFromString
	
	self assert: (SpEnvironment integerFromString: '1' ) = 1.
	self assert: (SpEnvironment integerFromString: '8r2') = 2. 
	self assert: (SpEnvironment integerFromString: '10e10') = 100000000000.
	!

testStreamStartPosition
	
	self assert: SpEnvironment streamStartPosition isZero! !
!SpEnvironmentTest categoriesFor: #testAllSubclassesOfAClass!public! !
!SpEnvironmentTest categoriesFor: #testBase64DecodeAsString!public! !
!SpEnvironmentTest categoriesFor: #testBase64EncodeString!public! !
!SpEnvironmentTest categoriesFor: #testCharacterFromInteger!public! !
!SpEnvironmentTest categoriesFor: #testCompilerEvaluation!public! !
!SpEnvironmentTest categoriesFor: #testDolphinEnvironment!public! !
!SpEnvironmentTest categoriesFor: #testIntegerFromString!public! !
!SpEnvironmentTest categoriesFor: #testStreamStartPosition!public! !

SpFilenameTest guid: (GUID fromString: '{B9CA8F0F-1287-4873-8A00-43ED5B201E83}')!
SpFilenameTest comment: ''!
!SpFilenameTest categoriesForClass!Unclassified! !
!SpFilenameTest methodsFor!

testExists

	| spFilename |
	spFilename := SpFilename named: SessionManager current imageFileName.
	self assert: spFilename exists!

testLastModified

	| spFilename |
	spFilename := SpFilename named: SessionManager current imageFileName.
	self assert: spFilename lastModified asSeconds = spFilename lastModified asSeconds.
	self assert: spFilename lastModified = spFilename lastModified.
	self assert: spFilename lastModified asTime  = spFilename lastModified asTime.
	self assert: spFilename lastModified asTime asMilliseconds = spFilename lastModified asTime asMilliseconds.!

testReadStream

	| spFilename reader |
	spFilename := SpFilename named: SessionManager current imageFileName.
	reader := spFilename readStream.
	self assert: reader next ~=nil
	
	! !
!SpFilenameTest categoriesFor: #testExists!public! !
!SpFilenameTest categoriesFor: #testLastModified!public! !
!SpFilenameTest categoriesFor: #testReadStream!public! !

SpMiscellaneous guid: (GUID fromString: '{23A20647-4F92-4C09-A13D-90F5DD1551A6}')!
SpMiscellaneous comment: ''!
!SpMiscellaneous categoriesForClass!Unclassified! !
SpSocketTest guid: (GUID fromString: '{D4674497-6462-4082-AA77-8313C4AE1252}')!
SpSocketTest comment: ''!
!SpSocketTest categoriesForClass!Unclassified! !
!SpSocketTest methodsFor!

execute: aBlock timeOutMilliseconds: anInteger 
	"Private - Executes aBlock or terminates it and raise an error if its evaluation takes longer than anInteger seconds."

	| proc |
	proc := aBlock newProcess.
	[(Delay forMilliseconds: anInteger) wait.
		(proc notNil and: [proc isAlive]) ifTrue: [
			self halt.
			"proc terminate."
			self signalFailure: 'Execution timeout reached.'
		]
	] fork.
	proc resume!

tearDown
	socket notNil ifTrue: [socket close].
	serverSocket notNil ifTrue: [socket close].
	newSpServerSocket notNil ifTrue: [newSpServerSocket close].
	acceptProcess ifNotNil: [acceptProcess terminate].
	socket := serverSocket := newSpServerSocket := nil!

testBasicInstantiation
	self should: [(socket := SpSocket newTCPClientSocket) notNil].
	self should: [(socket := SpSocket newTCPServerSocket) notNil]!

testConnectionAndClose
	
	serverSocket := ServerSocket2 port: 9999.
	socket := SpSocket connectToServerOnHost: 'localhost' port: 9999.
	self assert: socket isActive.
	socket close.
	self deny: socket isActive.
	serverSocket close.
	!

testConnectionToHostAndPort
	
	serverSocket := ServerSocket2 port: 9999.
	socket := SpSocket connectToServerOnHost: 'localhost' port: 9999.
	self assert: socket isActive!

testGetPeerAddress
	
	| pair |
	pair := SpSocket newSocketPair.
	serverSocket := pair first. 
	socket := pair last.
	!

testSocketConnectionAccept
	
	semaphore := Semaphore new.
	serverSocket := SpSocket port: 9999.
	acceptProcess := [newSpServerSocket := serverSocket accept. semaphore signal ] fork.
	socket := SpSocket connectToServerOnHost: 'localhost' port: 9999.
	semaphore wait.
	self assert: newSpServerSocket notNil.
	self assert: newSpServerSocket isActive
	!

testSocketPair
	
	| pair |
	pair := SpSocket newSocketPair.
	serverSocket := pair first. 
	socket := pair last.
	pair do: [:each |
		self 
			assert: (each isKindOf: SpSocket);
			assert: each isActive
			]
	
	!

testSocketSendAndReceive
	
	| bytes |
	serverSocket := SpSocket port: 9999.
	semaphore := Semaphore new.
	acceptProcess := [newSpServerSocket := serverSocket accept. semaphore signal] fork.
	socket := SpSocket connectToServerOnHost: 'localhost' port: 9999.
	semaphore wait.
	socket write: 'Hello portable world!!'.
	self assert: newSpServerSocket readyForRead.
	self assert: (newSpServerSocket  read: 5) asString = 'Hello'.
	self assert: (newSpServerSocket  readInto: (bytes := ByteArray new: 15)  startingAt: 1 for: 15) = ' portable world' size.
	self assert: (bytes copyFrom: 1 to: ' portable world' size) asString = ' portable world'.
	self assert: newSpServerSocket  readyForRead	!

testTCPServerSocketInstantiation
	
	serverSocket := SpSocket port: 9999.
	socket := SpSocket connectToServerOnHost: 'localhost' port: 9999.
	self assert: socket isActive.
	socket close.
	self deny: socket isActive.
	serverSocket close.
	!

waitAMoment
	"Private - Suspends the current process by a moment. "

	(Delay forMilliseconds: 400) wait! !
!SpSocketTest categoriesFor: #execute:timeOutMilliseconds:!public! !
!SpSocketTest categoriesFor: #tearDown!private! !
!SpSocketTest categoriesFor: #testBasicInstantiation!public! !
!SpSocketTest categoriesFor: #testConnectionAndClose!public! !
!SpSocketTest categoriesFor: #testConnectionToHostAndPort!public! !
!SpSocketTest categoriesFor: #testGetPeerAddress!public! !
!SpSocketTest categoriesFor: #testSocketConnectionAccept!public! !
!SpSocketTest categoriesFor: #testSocketPair!public! !
!SpSocketTest categoriesFor: #testSocketSendAndReceive!public! !
!SpSocketTest categoriesFor: #testTCPServerSocketInstantiation!public! !
!SpSocketTest categoriesFor: #waitAMoment!private! !

SpStringUtilitiesTest guid: (GUID fromString: '{EE3AD4DE-5485-4C74-A669-A6D213C9A6E7}')!
SpStringUtilitiesTest comment: ''!
!SpStringUtilitiesTest categoriesForClass!Unclassified! !
!SpStringUtilitiesTest methodsFor!

testPrevIndexOfAnElementFromTo

	| string |
	string := 'Dolphin Smalltalk'.
	self assert: (SpStringUtilities prevIndexOf: $m from: string size to: 1 in: string) = 10.
	self assert: (SpStringUtilities prevIndexOf: Character space from: string size to: 1 in: string) = 8!

testTokenizationBasedOnSeparatorString

	self assert: (SpStringUtilities tokensBasedOn: '/' in: 'hello/portable/world')  = (Array with: 'hello' with: 'portable'  with: 'world')!

testTrimBlanks

	self assert: (SpStringUtilities trimBlanksFrom:  ' hello portable world   ')  = 'hello portable world'! !
!SpStringUtilitiesTest categoriesFor: #testPrevIndexOfAnElementFromTo!public! !
!SpStringUtilitiesTest categoriesFor: #testTokenizationBasedOnSeparatorString!public! !
!SpStringUtilitiesTest categoriesFor: #testTrimBlanks!public! !

SpTimestampTest guid: (GUID fromString: '{D49289D8-48C2-4F2F-B5AB-236EE0AB5AE3}')!
SpTimestampTest comment: ''!
!SpTimestampTest categoriesForClass!Unclassified! !
!SpTimestampTest methodsFor!

testAsRFC1123String
	
	| spTs1 |
	spTs1 := SpTimestamp fromDate: (Date fromString: '06/11/1994' format: 'dd/mm/yyyy') andTime: (Time fromString: '08:49:37').
	self assert: spTs1 asRFC1123String = 'Sun, 06 Nov 1994 08:49:37 GMT'!

testAsSeconds

	 | ts1 spTs1 |
	ts1 := TimeStamp current.
	spTs1 := SpTimestamp fromDate: ts1 date andTime: ts1 time.
	self assert: ts1 asSeconds = spTs1 asSeconds!

testComparison

	 | ts1 spTs1 spTs2 |
	ts1 := TimeStamp current.
	spTs1 := SpTimestamp fromDate: ts1 date andTime: ts1 time.
	spTs2 := SpTimestamp fromDate: (ts1 date addDays: 10) andTime: ts1 time.
	self assert: spTs1 < spTs2.
	self deny: spTs1 >= spTs2!

testEquality

	 | ts1 spTs1 spTs2 |
	ts1 := TimeStamp current.
	spTs1 := SpTimestamp fromDate: ts1 date andTime: ts1 time.
	spTs2 := SpTimestamp fromDate: ts1 date andTime: ts1 time.
	self assert: spTs1 = spTs2 !

testInstantiationFromDateAndTime
	| ts1 spTs1 |
	ts1 := TimeStamp current.
	spTs1 := SpTimestamp fromDate: ts1 date andTime: ts1 time.
	self assert: ts1 = spTs1 underlyingTimestamp!

testInstantiationFromRFC1123String

	| ts |
	ts := TimeStamp date: (Date fromString: '01/01/2006' format: 'dd/MM/yyyy') time: (Time fromString: '14:15:59').
	self assert: ts = (SpTimestamp fromRFC1123String: ts rfc1123String) underlyingTimestamp! !
!SpTimestampTest categoriesFor: #testAsRFC1123String!public! !
!SpTimestampTest categoriesFor: #testAsSeconds!public! !
!SpTimestampTest categoriesFor: #testComparison!public! !
!SpTimestampTest categoriesFor: #testEquality!public! !
!SpTimestampTest categoriesFor: #testInstantiationFromDateAndTime!public! !
!SpTimestampTest categoriesFor: #testInstantiationFromRFC1123String!public! !

"Binary Globals"!

